################################################################################
# BEGIN COPYRIGHT
################################################################################
#
# THIS PRODUCT CONTAINS RESTRICTED MATERIALS OF IBM
# All Rights Reserved * Licensed Materials - Property of IBM
# 5655-FLW (C) Copyright IBM Corporation 2005, 2009.
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
################################################################################
# END COPYRIGHT
################################################################################
#
# Jython module containing helpers for the BPC Reporting Feature
#
# Version: 1.16 09/10/15 04:45:08
#
################################################################################
__file__="bpelib.py"


################################################################################
#
# imported jython/python modules
#
################################################################################
import sys
import time
import traceback
import messageCatalog

import os.path
import jarray


#import jreload  # allows to dynamically load java classes


################################################################################
#
# imported java classes
#
################################################################################
import java.lang.Exception
#import com.ibm.ws.management.commands.sib.SIBAdminCommandException as AdminCommandException
import com.ibm.ws.scripting.ScriptingException                     as ScriptingException
import com.ibm.ws.exception.ConfigurationWarning
from com.ibm.wsspi.scripting import ScriptingExtensionHelper

import java.lang.Runnable         # used for PasswordReader

from java.lang import *
from java.util import *
from java.io import *

from java.util.regex import Pattern, Matcher

import java.net.URLClassLoader as URLClassLoader
import java.net.URL as URL


#just for dev
reload(messageCatalog)

################################################################################
#
# Global settings
#
################################################################################

#get the global system properties, we get a dictionary here,
# -> access as follows: osname=javaEnv["os.name"]
javaEnv=java.lang.System.getProperties()
lineSep=javaEnv["line.separator"]
osEnv=java.lang.System.getenv()


# String representations for true/false returned by the Was Objects
true  = 'true'
false = 'false'


#helper for debugging
def bp():
  import pdb;
  pdb.set_trace()


################################################################################
# global message catalog
################################################################################
mcat = messageCatalog.MessageCatalog()
mget = mcat.get

################################################################################
#
# FileBuffer
#
# get a file into a list of line
#
################################################################################
class FileBuffer:

  def __init__(self, filename):

    self.filename=filename
    # bp()
    f=open(filename,'r')
    self.lines=f.readlines()
    f.close()
    return


  def reload(self):

    self.__init__(self.filename)
    return


from threading import Thread

class Timer(Thread):

  def __init__(self, triggerFunction, delay, *fkt_args):
      Thread.__init__(self)
      self.delay = delay
      self.function = triggerFunction
      self.args = fkt_args
      self.start()

  def run(self):
      time.sleep(self.delay)
      return self.function(*self.args)

#-------------------------------------------------------------------
# Trace
#
#-------------------------------------------------------------------
class Tracer:

  isTracing=0

  theTracer=None   #static member to store the instance
  fileBufferCache   = {}    # get a static dictionary for storing the differnt files

  #------------------------
  # CTOR
  #------------------------
  def __init__(self, file=None):

    self.maxFileNameLength = 20
    self.doInteractive     = 0
    self.printLocals       = 0
    self.printGlobals      = 0
    self.previousLocals    = None
    self.previousGlobals   = None
    self.disabled          = 0
    self.flushTime         = 0
    #self.lineCounter       = 0
    #self.lineFlush         = 10   # flush each 10 lines

    if file==None:
      self.traceFile=sys.stdout
      self.traceIndent       = "          "
    else:
      try:
        self.traceFile=open(file,'w')
        self.traceIndent       = ""
      except:
        log("Warning: Cannot write tracefile ", file, ". Write trace to console? (y/n): ")
        ans=raw_input()
        if ans !='y':
          self.disabled = 1
        else:
          self.traceFile=sys.stdout
          self.traceIndent       = "          "


  #------------------------
  # Stop the tracing
  #------------------------
  def stop(self):
    sys.settrace(None)
    Tracer.isTracing=0
    self.traceFile.write(lineSep)
    self.traceFile.flush()
    if self.flushTime!=0:
      del self.flushTimer


  #------------------------
  # Flush the file
  #------------------------
  def flushFile(self):
    if self.traceFile!=None:
      self.traceFile.flush()


  #------------------------
  # Start the tracing
  #------------------------
  def start(self):
    if self.disabled == 0:
      sys.settrace(self.trace)
      if self.flushTime!=0:
        self.flushTimer=Timer( self.flushFile, self.flushTime)


  #------------------------
  # trace: Produce the trace entry
  #------------------------
  def trace(self, frame, event, arg):

    try:

      Tracer.isTracing = 1

  #     if ("PRINTLOCALS" in locals().keys() or "PRINTLOCALS" in globals().keys()):
  #       printLocals=1
  #
  #     if ("PRINTGLOBALS" in locals().keys() or "PRINTLOCALS" in globals().keys()):
  #       printGlobals=1

      lineno = frame.f_lineno

      #function = frame.f_globals["__name__"]
      function = frame.f_code.co_name


      file=""
      appendix=""
      sourceline=""
      basefile=""
      previousFrame=None

      try:
        file=frame.f_globals["__file__"]
      except:
        file= "notset"

      if file!="notset":

        #check if we already have read in the file -> check in our file cache fileBufferCache
        basefile=os.path.basename(file)
        fileBuff=self.fileBufferCache.get(file)

        if fileBuff==None:

          # first time use of the file, read it.
          fileBuff=FileBuffer(file)
          self.fileBufferCache[file]=fileBuff

        sourceline=fileBuff.lines[lineno-1]
        sourceline=sourceline.replace("\r"," ")
        sourceline=sourceline.replace("\n"," ")

      if event == "line":
        #appendix=sourceline
        appendix =""
      elif event == "call":
        appendix="<function entry>"
      elif event == "return":
        appendix="<function returns> " + repr(arg) + " "
      elif event == "exception":
        appendix="<exception occured> " + repr(arg) + " "

      #prefix=basefile.ljust(self.maxFileNameLength) + ":" + function + "(" + str(lineno).rjust(4) + "):"
      prefix=time.strftime("%X ") + basefile.ljust(self.maxFileNameLength) + " (" + str(lineno) + ")"
      func=function.ljust(20)

      codeOut=self.traceIndent+prefix+":"+func+" "+sourceline

      previousFrame=sys._getframe(1)
      previousLocals=previousFrame.f_locals
      previousGlobals=previousFrame.f_globals

      varout=""
      if self.printLocals and appendix=="":
        for z in previousLocals.keys():
          if not self.previousLocals.has_key(z):
            varout="(new var " + z + "=" + str(previousLocals[z]) + ")"
          elif previousLocals[z]!=self.previousLocals[z]:
            varout="(changed var " + z + "=" + str(previousLocals[z]) + ")"

      if varout=="":
        varout="\n"     # do not use lineSep here, "\n" it translated by the write fkt
      else:
        varout= varout + "\n" # do not use lineSep here, "\n" it translated by the write fkt
      self.traceFile.write(varout)

      self.previousLocals=previousLocals.copy()
      self.previousGlobals=previousGlobals.copy()

      self.traceFile.write(codeOut + appendix )

      if event == "exception":
        self.traceFile.flush()

      if self.doInteractive:

        x=raw_input()

        while x!="":
          try:
            exec x in previousGlobals, previousLocals
            print previousLocals
          except:
            handleException()

          x=raw_input()

    finally:
#       self.lineCounter+=1
#       if self.lineCounter==self.lineFlush:
#         self.traceFile.flush()
#         self.lineCounter==0
      if frame!=None:
        del frame
      if previousFrame!=None:
        del previousFrame

    return self.trace



################################################################################
#
#  check if we must trace
#
################################################################################
def trace(shouldTrace=0, traceFile=None):

  if (shouldTrace==1) or ("-trace" in sys.argv) or ("TRACE" in locals().keys()) or ("TRACE" in globals().keys()):

    if "-trace" in sys.argv:
      pos=sys.argv.index("-trace")
      if (pos+1) < len(sys.argv):
        if not "-" in sys.argv[pos+1] :
          if traceFile==None:
            traceFile=sys.argv[pos+1]

    Tracer.theTracer = Tracer(traceFile)
    Tracer.theTracer.start()

  else:
    if Tracer.theTracer != None:
      Tracer.theTracer.stop()


# ensure that tracing is checked when loading the module
trace()






################################################################################
#
# ConsoleHandler
#
# provide functions on the console
#
################################################################################
class ConsoleHandler:


  # provide default answer
  YES   = mcat.get("Yes")
  NO    = mcat.get("No")
  SKIP  = mcat.get("Skip")
  CANCEL= mcat.get("Cancel")

  YES_SHORT   = YES[0]
  NO_SHORT    = NO[0]
  SKIP_SHORT  = SKIP[0]
  CANCEL_SHORT = CANCEL[0]


  SET_YES_NO_ANSWERS        = [ YES_SHORT, NO_SHORT ]  # = ['y','n']
  SET_YES_NO_CHOICES        = [ YES, NO ]              # = ['yes','no']

  SET_YES_NO_SKIP_ANSWERS   = [ YES_SHORT, NO_SHORT, SKIP_SHORT ] # = ['y','n','s']
  SET_YES_NO_SKIP_CHOICES   = [ YES, NO, SKIP ]                   # =['yes','no', 'skip']

  SET_YES_NO_CANCEL_ANSWERS   = [ YES_SHORT, NO_SHORT, CANCEL_SHORT ] # = ['y','n','c']
  SET_YES_NO_CANCEL_CHOICES   = [ YES, NO, CANCEL ]                   # =['yes','no', 'cancel']



  #-------------------------------------------------------------------
  # getAnswer
  #
  # wrapper for raw_input
  #-------------------------------------------------------------------
  def inspect(self):

    answer=""

    depth=2
    currFrame=sys._getframe(depth)

    print "------------------------------------------------------"
    print "Interactive debug environment"
    print "  Enter 'help' for Help "
    print "------------------------------------------------------"

    while 1:

      try:
          file=currFrame.f_globals["__file__"]
      except:
          file= "--"


      try:
          className=currFrame.f_locals["self"].__class__.__name__
      except:
          className= "--"



      answer=self.getAnswer("<dbg:" + file+ ":" + className+ ":" + currFrame.f_code.co_name + "> ")
      if answer[0:1]=="c":
        print
        print "==== ENDING INSPECITON ===="
        print
        return

      if answer[0:1]=="h":
        print
        print "  c[ont]         return to normal execution"
        print "  u[p], o[ut]    go on frame up, return to calling function"
        print "  d[own], i[n]   go down frame up, enter called function"
        print "  q              exit programm execution"
        print "  lo[cals]       list local variables"
        print "  l[ist]         list 10 lines around code object"
        print "  de[pth]        print frame depth"
        print "  pdb            invoke pdb"
        print
        print "To indicate, that this command must go to the interpreter, you may prefix it with '-'."
        print
        continue

      if answer[0:1]=="u" or answer[0:1]=="o":
        depth=depth+1
        currFrame=sys._getframe(depth)
        continue

      if answer[0:1]=="pdb" :
        import pdb
        pdb.set_trace()
        return

      if answer[0:1]=="q" :
        sys.exit()

      if answer[0:2]=="de":
        print "depth=",depth
        continue

      if answer[0:1]=="d" or answer[0:1]=="i":
        depth=depth-1
        currFrame=sys._getframe(depth)
        continue

      if answer[0:2]=="lo":
        print currFrame.f_code.co_varnames
        continue

      # list 10 lines around code object
      if answer[0:1]=="l":

        try:
          file=currFrame.f_globals["__file__"]
        except:
          file= "notset"
          print "File not set"

        if file!="notset":

          #check if we already have read in the file -> check in our file cache fileBufferCache
          basefile=os.path.basename(file)
          fileBuff=Tracer.fileBufferCache.get(file)

          if fileBuff==None:

            # first time use of the file, read it.
            fileBuff=FileBuffer(file)
            Tracer.fileBufferCache[file]=fileBuff

          #startLine=currFrame.f_code.co_firstlineno - 10
          startLine=currFrame.f_lineno -  10
          if startLine<0:
            startLine=0

          for line in range(startLine,startLine+20):
            try:
              if line==currFrame.f_lineno-1:
                print "==> "+ str(line), fileBuff.lines[line],
              else:
                print "   #"+ str(line), fileBuff.lines[line],
            except:
              print "#<EOF>"
              break

        continue

      try:
        if answer[0:1]=="-":
          answer=answer[1:]
        exec answer in currFrame.f_globals, currFrame.f_locals
      except:
        tb=traceback.format_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
        for l in tb:
          print l






  #-------------------------------------------------------------------
  # getAnswer
  #
  # wrapper for raw_input
  #-------------------------------------------------------------------
  def getAnswer(self, msg):

    answer=raw_input( msg )

    if answer==":q!":
      sys.exit()

    if answer==":b!":
      sys.exit("ReturnToMain")

    if answer==":i!":
      self.inspect()
      answer=raw_input( msg )

    return answer


  #-------------------------------------------------------------------
  # selectFromList
  #
  # provide a prompt on the console with default value and selection
  #
  # in:
  #   promptMessage      : The text to be displayed
  #   listOfValidAnswers : a list object containing all valid anwsers
  #   defaultAnswer      : the index of one of the members of the list, none,
  #                        if no default is given -> enforce an entry
  #   listOfChoices      : Textual names for the choices to echo to console
  #
  # out:
  #   index of the selected answer
  #-------------------------------------------------------------------
  def selectFromList(self,promptMessage, listOfValidAnswers, defaultAnswer=None, listOfChoices=None):

    rc=-1

    outMsg=promptMessage

    if listOfChoices==None:
      if defaultAnswer!=None:
        outMsg += " [" + str(listOfValidAnswers[defaultAnswer]) + "] "
    else:
      #calculate from listOfChoices if not given
      if listOfValidAnswers==None:
        listOfValidAnswers=[]
        for i in range(0, len(listOfChoices)):
          listOfValidAnswers.append(listOfChoices[i][0].lower())

      # build up menu
      for mitem in range(0,len(listOfValidAnswers)) :
        outMsg += "\n  " + str(listOfValidAnswers[mitem]) + ") " + listOfChoices[mitem]

      # appen prompt message
      if defaultAnswer==None:
        outMsg += "\n" + mcat.get("YourSelection") + " "
      else:
        outMsg += "\n" + mcat.get("YourSelection") + "[" + str(listOfValidAnswers[defaultAnswer]) + "] "

    # allow lowercase selection only
    if listOfValidAnswers!=None:
      for i in range(0, len(listOfValidAnswers)):
        listOfValidAnswers[i]=str(listOfValidAnswers[i]).lower()

    while 1:

      answer=self.getAnswer( outMsg )

      if answer=="" and defaultAnswer != None:
        rc=defaultAnswer
        break

      try:
        i=listOfValidAnswers.index(answer)
        # found a valid entry
        rc=i
        break

      except ValueError:
        print mcat.get("Menu_InvalidSelection", answer)
        print mcat.get("NewLine", answer)
        continue


    if listOfChoices==None:
      print "==>", listOfValidAnswers[rc]
    else:
      print "==>", listOfChoices[rc]

    return rc

  #-------------------------------------------------------------------
  # selectString
  #
  # provide a prompt for a free from string
  #
  # in:
  #   promptMessage      : The text to be displayed
  #   defaultAnswer      : the default string to be used
  #                        if no default is given (== None) -> enforce an entry
  # out:
  #   user input string
  #-------------------------------------------------------------------
  def selectString(self,promptMessage, defaultAnswer=None):

    rc=""

    while 1:
      if defaultAnswer==None :
        answer=self.getAnswer( promptMessage )
      else:
        answer=self.getAnswer( promptMessage + " [" + defaultAnswer + "] " )

      if answer=="" :
        if defaultAnswer != None:
          rc=defaultAnswer
          break
        else:
          print mcat.get("Menu_InvalidSelection", answer)
          print mcat.get("NewLine")
          continue
      else:
        rc=answer
        break

    print "==>", rc

    return rc

  #-------------------------------------------------------------------
  # selectInteger
  #
  # provide a prompt for a integer
  #
  # in:
  #   promptMessage      : The text to be displayed
  #   defaultAnswer      : the default number to be used
  #                        if no default is given (== None) -> enforce an entry
  # out:
  #   user input (int)
  #-------------------------------------------------------------------
  def selectInteger(self,promptMessage, _defaultAnswer=None):

    if _defaultAnswer!=None:
      strDefaultAnswer = str(_defaultAnswer)
      intDefaultAnswer = int(_defaultAnswer)
    else:
      strDefaultAnswer=None
      intDefaultAnswer=None

    rc=""

    while 1:
      if strDefaultAnswer==None :
        answer=self.getAnswer( promptMessage )
      else:
        answer=self.getAnswer( promptMessage + " [" + strDefaultAnswer + "] " )

      if answer=="" :
        if intDefaultAnswer != None:
          rc=intDefaultAnswer
          break
        else:
          print mcat.get("Menu_InvalidSelection", answer)
          print mcat.get("NewLine")
          continue
      else:
        rc=answer

      try:
        int(rc)
      except:
        print mcat.get("Menu_InvalidSelection", rc)
        print mcat.get("NewLine")
        continue

      break

    print "==>", rc

    return int(rc)


  #-------------------------------------------------------------------
  # selectYesNo
  #
  # provide a prompt for YesNo
  #
  # in:
  #   promptMessage      : The text to be displayed
  #   defaultAnswer      : y/n
  #                        if no default is given -> y
  # out:
  #   user input string
  #-------------------------------------------------------------------
  def selectYesNo(self, promptMessage, defaultAnswer=""):

    rc=""

    if defaultAnswer==None or defaultAnswer=="" or defaultAnswer==self.YES_SHORT or defaultAnswer==self.YES :
      default=0
    else:
      default=1

    ans= self.selectFromList(promptMessage,
                             self.SET_YES_NO_ANSWERS,
                             default,
                             self.SET_YES_NO_CHOICES)
    if ans == 0:
      rc=ConsoleHandler.YES
    elif ans == 1:
      rc=ConsoleHandler.NO

    return rc

  #-------------------------------------------------------------------
  # selectYesNoSkip
  #
  # provide a prompt for YesNo
  #
  # in:
  #   defaultAnswer      : y/n/s
  #                        if no default is given -> y
  # out:
  #   ConsoleHandler.Yes, ConsoleHandler.No, ConsoleHandler.Skip
  #-------------------------------------------------------------------
  def selectYesNoSkip(self, msg="",defaultAnswer=None ):

    rc=""

    if defaultAnswer==None or defaultAnswer=="" or defaultAnswer==self.YES_SHORT or defaultAnswer==self.YES :
      default=0
    else:
      default=1

    ans=self.selectFromList( msg,
                             self.SET_YES_NO_SKIP_ANSWERS,
                             default,
                             self.SET_YES_NO_SKIP_CHOICES)

    if ans == 0:
      rc=ConsoleHandler.YES
    elif ans == 1:
      rc=ConsoleHandler.NO
    elif ans == 2:
      rc=ConsoleHandler.SKIP


    return rc

  #-------------------------------------------------------------------
  # selectYesNoCancel
  #
  # provide a prompt for YesNo
  #
  # in:
  #   defaultAnswer      : y/n/s
  #                        if no default is given -> y
  # out:
  #   ConsoleHandler.Yes, ConsoleHandler.No, ConsoleHandler.Abort
  #-------------------------------------------------------------------
  def selectYesNoCancel(self, msg="",defaultAnswer=None ):

    rc=""

    if defaultAnswer==None or defaultAnswer=="" or defaultAnswer==self.YES_SHORT or defaultAnswer==self.YES :
      default=0
    else:
      default=1

    ans=self.selectFromList( msg,
                             self.SET_YES_NO_CANCEL_ANSWERS,
                             default,
                             self.SET_YES_NO_CANCEL_CHOICES)

    if ans == 0:
      rc=ConsoleHandler.YES
    elif ans == 1:
      rc=ConsoleHandler.NO
    elif ans == 2:
      rc=ConsoleHandler.CANCEL

    return rc


  #-------------------------------------------------------------------
  # formatwithMargins
  #
  # format a string containing multiple words on word basis in margins
  # return a list of strings
  #-------------------------------------------------------------------
  def formatWithMargins(self, description, leftMargin=0, rightMargin=80 ):

    words=description.split()
    lineCount=0
    indent=""
    if leftMargin>0:
      indent=" ".ljust(leftMargin)
    outList=[" "]
    outList[lineCount]=indent

    for word in words:
      tmpStr=outList[lineCount] + " " + word
      if len(tmpStr) <= rightMargin:
        #word fits to line
        outList[lineCount] = tmpStr
      else:
        #we need a new line
        outList[lineCount]+= "\n"
        outList.append(indent + " " + word)
        lineCount=lineCount+1

    return outList


# Provide a global instance
console=ConsoleHandler()

################################################################################
#
# class for User id handling
#
################################################################################
class UserId:

  def __init__(self, username=None, password=None):

    self.username=username
    self.password=password


  ########################
  # prompt for the user
  def promptUserName(self, message, defaultUser=None):

    inp = console.selectString( message, defaultUser )
    self.username=inp

    return inp


  ########################
  # get password
  def promptPassword(self, allowEmpty=0 ):
    pwr=PasswordReader()

    while 1 :

       print mcat.get("PromptPassword", self.username ), # leave comma to avoid newline
       inp=pwr.readPassword()
       if inp!="" or allowEmpty==1:
          self.password=inp
          break

    return inp

################################################################################
#
# logFile impl
#
################################################################################
class LogFile:

  #static member function to store the default log object
  # set this to any logFile object to have the logs of all bpelib functions
  # in your log
  defaultLog = None

  ##############
  #
  def __init__(self,filename="",leaveClosed=0):

    self.printToConsole = 0
    self.prefixWithTimestamp = 0
    self.logFile=None
    self.fileName=""

    if filename!="" and leaveClosed==0 :
      self.logFile=open(filename,'a+')
      self.fileName=filename
      self.log( mcat.get("LineSep"),"\n" ,
                self.logTime() + " ",
                mcat.get("NewRunStarted"))

    if self.logFile==None:
      self.printToConsole = 1


  ##############
  #
  def __del__(self):

    if self.logFile!=None:
      self.logFile.close()


  def logTime(self):
    return time.strftime("%Y-%m-%d %X " + time.tzname[time.daylight])

  ##############
  #
  def close(self):

    if self.logFile!=None:
      self.logFile.close()


  ##############
  # log to file
  def log(self,*params):

    outStr=""

    for i in params:
     if i != None:
       outStr = outStr + str(i)

    if self.printToConsole ==1:
      print outStr

    outStr = outStr + "\n"

    if self.logFile!=None:

      if self.logFile.closed==1:
        # reopen
        self.logFile=open(self.fileName,'a+')

      if self.prefixWithTimestamp==1:
        self.logFile.write(self.logTime() + ' ' + outStr)
      else:
        self.logFile.write(outStr)

      self.logFile.flush()



####################################
# have a default log function


def log(*params):

  outStr=""

  for i in params:
   if i != None:
     outStr = outStr + str(i)

  # print to console, log with time. but keep Settings
  if LogFile.defaultLog != None:

    ptc=LogFile.defaultLog.printToConsole
    pwt=LogFile.defaultLog.prefixWithTimestamp
    LogFile.defaultLog.printToConsole=1
    LogFile.defaultLog.prefixWithTimestamp=1

    LogFile.defaultLog.log(outStr)

    LogFile.defaultLog.printToConsole=ptc
    LogFile.defaultLog.prefixWithTimestamp=pwt

  else:
     print outStr


def logOnly(*params):

  outStr=""

  for i in params:
    if i != None:
      outStr = outStr + str(i)

  if LogFile.defaultLog != None:

    ptc=LogFile.defaultLog.printToConsole
    pwt=LogFile.defaultLog.prefixWithTimestamp

    LogFile.defaultLog.printToConsole=0
    LogFile.defaultLog.prefixWithTimestamp=1

    LogFile.defaultLog.log(outStr)

    LogFile.defaultLog.printToConsole=ptc
    LogFile.defaultLog.prefixWithTimestamp=pwt

  else:
    print outStr





################################################################################
#
# PasswordReader
# helper class to hide the password prompt
#
################################################################################
class PasswordReader( java.lang.Runnable ):

   # Attributes must be defined in the Ctor. Else they are imterpreted as
   # static/class attributes
   #    stop = ""
   #    prompt = " "

   #-----------------------
   def __init__(self):
     self.stop = ""
     self.prompt = " "


   #-----------------------
   def __del__ (self):
      self.stopMasking()


   def run (self):

     if Tracer.isTracing==1:
       sys.settrace(None)

     self.stop = false

     while self.stop==false:
       java.lang.System.out.print("\010" + self.prompt)
       java.lang.Thread.currentThread().sleep(1)


     if Tracer.isTracing==1 and Tracer.theTracer!=None:
       sys.settrace(Tracer.theTracer.trace)


   #-----------------------
   def stopMasking(self):
      self.stop = true

   #-----------------------
   def readPassword (self):

     if Tracer.isTracing==1:
       sys.settrace(None)

     maskThread = PasswordReader()
     mask = java.lang.Thread(maskThread)

     if (javaEnv["os.name"].find("OS/400") > -1):
       # no masking in qshell possible
       password = raw_input(" ")
     else:

       mask.start()

       try:
         password = raw_input(" ")

       except:
         maskThread.stopMasking()
         raise

       maskThread.stopMasking()


     if Tracer.isTracing==1:
       sys.settrace(Tracer.theTracer.trace)

     return password



################################################################################
#
# DatabaseManager
#
################################################################################

class DatabaseManager:
  "Encapsulate the different admin tasks of the various DBMS systems "


  DBMS_TYPE_DERBY      = "Derby"
  DBMS_TYPE_DB2        = "DB2"
  DBMS_TYPE_DB2ISERIES = "DB2iSeries"
  DBMS_TYPE_DB2ZOSV8   = "DB2zOSV8"
  DBMS_TYPE_DB2ZOSV9   = "DB2zOSV9"
  DBMS_TYPE_ORACLE     = "Oracle"

  UDF_IMPL_JAVA        = mcat.get("Java")
  UDF_IMPL_SQL         = mcat.get("SQL")

  DATABASE_CS_BPEDB_PATH =  os.path.join(javaEnv["user.install.root"],"/databases/BPEDB")




  #----------------------------------------------------------------------------
  # Constructor
  #----------------------------------------------------------------------------
  def __init__(self):

    self.dbmsType          = ""
    self.connectionPrefix  = ""
    self.driverName        = ""
    self.databaseName      = ""
    self.databaseType      = "" # CEI or AUDIT
    self.databaseUser      = ""
    self.databasePassword  = ""
    # schema must be private, because @SCHEMA@ in tokensToReplace must be updatetd
    self.__schema          = None

    self.connectionPort    = 0
    self.connectionHost    = "localhost"
    self.commandBuffer     = ""
    self.connection        = None # java.sql.Connection()
    self.dataSource        = None
    self.dataSourceImpl    = ""
    self.attachedToJdbc    = 0
    self.jdbcHome          = ""
    self.wasVarJdbcDriverPath = ""
    self.jdbcDriverJars    = []
    self.jdbcDriverType    = 2
    self.urlClassloader    = None

    self.tableSpaceClause     = ""
    self.ObserverTableSpaces  = []
    self.lastSqlException     = None
    self.useDefaultTablespace = 0
    self.tokensToReplace      = {} # provide a dictionary for tags/tokens to replace
    self.schemaToken          = "@SCHEMA@"
    self.sqlViaAdminTaskResults   = {}

    self.delayConfig     = 0
    self.createDatabase  = 0
    self.sqlOutFile      = ""
    self.udfImplementation = self.UDF_IMPL_SQL
    self.jarLocation       = os.path.join( javaEnv["was.install.root"], "lib" )


  #----------------------------------------------------------------------------
  # destructor
  #----------------------------------------------------------------------------
  #def __del__(self):
     # self.disconnectUncommited()

  #----------------------------------------------------------------------------
  # reset singletons
  #
  # invalidate singletons like connection or JDBC attachments
  #----------------------------------------------------------------------------
  def resetSingletons(self):

     # reset the connection
     self.disconnect()

     # detach from JDBC Classes
     self.attachedToJdbc=0
     self.dataSource=None


  #----------------------------------------------------------------------------
  # resolveJdbcHome
  #
  # resolve path to the JDBC driver as set in WAS env vars
  #----------------------------------------------------------------------------
  def resolveJdbcHome(self, serverName=""):

    jdbcHome = ""

    while 1:

      classPath=osEnv["EC_CLASSPATH"]

      if classPath!=None and classPath!="":
      # try the CLASSPATH env var we set in the invoking batch
        for l in classPath.split(os.pathsep):
          if l.find(self.jdbcDriverJars[0]) > -1:
            pth=os.path.dirname(l)
            if os.path.exists(pth):
              jdbcHome=pth
              break  # inner for

        if jdbcHome!="":
          break # we found one, leave outer while

      cellName=javaEnv["local.cell"]
      nodeName=javaEnv["local.node"]

      if serverName!="":

        #first try server scope
        scope="(cells/"+cellName+"/nodes/"+nodeName+"/servers/"+serverName+")"
        jdbcHome=getWASVariable(self.wasVarJdbcDriverPath,scope)
        if jdbcHome!=None and jdbcHome!="":
          break  #we found one

      # second, try node scope
      scope="(cells/"+cellName+"/nodes/"+nodeName+")"
      jdbcHome=getWASVariable(self.wasVarJdbcDriverPath,scope)
      if jdbcHome!=None and jdbcHome!="":
        break  #we found one

      # try any match
      jdbcHome=getWASVariable(self.wasVarJdbcDriverPath)
      break  #leave loop, even we din't find one

    if jdbcHome==None:
      jdbcHome=""

    return jdbcHome.strip()

  #----------------------------------------------------------------------------
  # retrieveTablespacesFromFile()
  #
  # dynamically determine the required tbs by parsing the createTablespace file
  #----------------------------------------------------------------------------
  def retrieveTablespacesFromFile(self, _tbsFileName="", pattern=r"\bTABLESPACE @(\w+)@" ):

    if _tbsFileName=="":
      tbsFile=os.path.join( javaEnv["was.install.root"], "dbscripts" )
      tbsFile=os.path.join( tbsFile, "ProcessChoreographer" )
      tbsFile=os.path.join( tbsFile, os.path.join( self.getDbmsType()))
      tbsFile=os.path.join( tbsFile, "createTablespace_Observer.sql")
    else:
      tbsFile=_tbsFileName

    fb=FileBuffer(tbsFile)

    # filter out comments
    for l in range(0,len(fb.lines)):
      cmt=fb.lines[l].find("--")
      if cmt>-1:
        fb.lines[l]=fb.lines[l][0:cmt]

    if pattern=="" or pattern==None:
      # relay on hardcoded list
      return self.ObserverTableSpaces

    ex=Pattern.compile(pattern, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE)
    m=ex.matcher(java.lang.String(str(fb.lines)))

    tbss=[]

    #import pdb
    #pdb.set_trace()
    while m.find():
      # m.group returns "TABLESPACE OBSVRTS"
      tbsName=m.group(1)
      tbss.append(tbsName.strip().upper())

    return tbss


  #----------------------------------------------------------------------------
  # attachToJdbcDriver()
  #
  # attach the jython instance to a Jdbc class
  #----------------------------------------------------------------------------
  def attachToJdbcDriver(self):


    if not self.attachedToJdbc:

      try:

        fileList = []
        urls = java.util.ArrayList()

        for jar in self.jdbcDriverJars:
          jarFile=os.path.join(self.jdbcHome, jar)
          fileList.append(jarFile)
          url=URL("file", "", -1, jarFile)
          urls.add(url)

        xa=jarray.array(urls, java.net.URL)
        self.urlClassloader = URLClassLoader(xa, sys.classLoader)
        dsClass=self.urlClassloader.loadClass(self.dataSourceImpl)
        ds=dsClass.newInstance()

        #doImport   = "import " + self.dataSourceImpl
        #getInstance= "ds = " + self.dataSourceImpl + "() "   # e.g. ds=oracle.jdbc.pool.OracleDataSource()

        #exec(doImport)
        #exec(getInstance)

        if ds!=None:
          self.dataSource = ds
        else:
          raise UnboundLocalError, mcat.get("ErrorNotInitialized", "dataSource")

        self.attachedToJdbc = 1

      except :

        msg = mcat.get("CannotLoadJdbcDriver", self.dataSourceImpl, str(fileList)[1:-1]  )
        print mget("NewLine")
        log( msg  )
        raise LookupError, msg

    return

  #----------------------------------------------------------------------------
  # connect()
  #
  # make a singelton connection to the DB
  #----------------------------------------------------------------------------
  def connect(self):

    rc=0

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."


    try:

      if self.connection==None:

        databaseName=self.databaseName
        self.dataSource.setDatabaseName(databaseName)
        self.connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)

        logOnly( mcat.get("ConnectedTo", databaseName) )


    except java.sql.SQLException, xcp :
      rc=xcp
      raise

    return rc

  #----------------------------------------------------------------------------
  #
  # promptJdbcConnectionType
  #
  # if supported, implemented in childs
  #
  #----------------------------------------------------------------------------
  def promptJdbcConnectionType(self, promptTitle="PromptJdbcConnectionType"):

    return self.jdbcDriverType

  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # should be implemented in the childs
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jarId, replaceExisting=1, refreshClasses=1):

    #print mcat.get("NewLine")
    #print mcat.get("CurrentlyNotSupported", self.dbmsType)
    return

  #----------------------------------------------------------------------------
  # connectInteractive()
  #
  # development: make a connection to the DB, prompting relevant values
  #----------------------------------------------------------------------------
  def ci(self,dbName="", user="", password="", host="", port=0):
    self.connectInteractive(dbName, user, password, host, port)


  def connectInteractive(self, dbName="", user="", password="", host="", port=0 ):

    rc=0

    try:

      #ensure new singletons
      self.resetSingletons()

      if self.jdbcHome=="" or self.jdbcHome==None:
        self.promptJdbcHome()

      self.attachToJdbcDriver()

      # self.databaseName=raw_input("DB_NAME:")
      if dbName=="":
        self.promptDatabaseName()
      else:
        self.setDatabaseName(dbName)
      if self.jdbcDriverType==4:
        if host=="":
          self.promptDatabaseHostName()
        else:
          self.connectionHost=host
        if port==0:
          self.promptDatabasePort()
        else:
           self.connectionPort=port

      if user=="":
        self.promptCredentials()
      else:
        self.setDatabaseUser(user)
        self.setDatabasePassword( password )

      self.setSchema(self.getDatabaseUser())

      self.dataSource.setDatabaseName(self.databaseName)
      self.connect()

      print "--> connected"

    except java.sql.SQLException, xcp :
      rc=self.handleSqlException(xcp)

    return rc

  #----------------------------------------------------------------------------
  # commit()
  #----------------------------------------------------------------------------
  def commit(self):
    self.connection.commit()

  #----------------------------------------------------------------------------
  # disconnect()
  #----------------------------------------------------------------------------
  def disconnect(self):
     if self.connection != None :
       self.connection.commit()
       self.connection.close()
       del self.connection
       self.connection=None
       logOnly( mcat.get("DisconnectedFrom", self.databaseName) )

  #----------------------------------------------------------------------------
  # disconnectUncommited()
  #----------------------------------------------------------------------------
  def disconnectUncommited(self):
      if self.connection != None :
        self.connection.close()
        del self.connection
        self.connection=None
        logOnly( mcat.get("DisconnectedFrom", self.databaseName) )

  #----------------------------------------------------------------------------
  # getSchemaVersion()
  #----------------------------------------------------------------------------
  def getSchemaVersion(self):

    try:

      observerVersion=0
      versionCmd="SELECT OBSERVER_VERSION FROM "+ self.getSchema() + ".OBSERVER_VERSION";

      stmt = self.connection.createStatement()
      logOnly(mcat.get("ExecutingSQLStmt") + " : " + versionCmd)
      rs=stmt.executeQuery(versionCmd)
      if rs.next():
        observerVersion = rs.getInt(1)

      logOnly(mcat.get("InfoDatabaseVersion", str(observerVersion)))
      stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise

    return observerVersion

  #----------------------------------------------------------------------------
  # isJavaOrSqlUdfActive()
  #
  #----------------------------------------------------------------------------
  def isJavaOrSqlUdfActive(self):

    rc=mcat.get("undefined")
    stmt=None

    try:

      observerVersion=self.getSchemaVersion()
      if observerVersion == 602:
        # on 602, we only had the Java implementation
        rc=DatabaseManager.UDF_IMPL_JAVA

      else:

        cmd = "SELECT " + self.getSchema() + ".OBSVR_JAR_ACTIVE() AS IS_JAVA FROM " + self.getSchema() + ".OBSERVER_VERSION"
        logOnly(mcat.get("ExecutingSQLStmt") + " : " + cmd)
        stmt = self.connection.createStatement()
        rs=stmt.executeQuery(cmd)

        row=rs.next()   # returns true if there are (more) columns
        if row==1:
          IS_JAVA=rs.getInt("IS_JAVA")

        if IS_JAVA==0:
          rc=DatabaseManager.UDF_IMPL_SQL  # "SQL"
        else:
          rc=DatabaseManager.UDF_IMPL_JAVA # "Java"

        stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise

    return rc

  #----------------------------------------------------------------------------
  # isUdfWorking()
  #
  #----------------------------------------------------------------------------
  def isUdfWorking(self, schema=""):

    rc=mcat.get("undefined")
    stmt=None
    if schema=="":
      schema=self.getSchema()
    try:

      cmd = "SELECT " + schema + ".INTERVALIN(2,'2005-01-31-11.40.00.000000','2006-01-31-11.40.00.000000') AS RS FROM " + schema + ".OBSERVER_VERSION"
      logOnly(mcat.get("ExecutingSQLStmt") + "1: " + cmd)
      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)
      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        RS=rs.getInt("RS")

      if RS==31536000:
        rc="working"

      if stmt!=None:
        stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise

    return rc

  #----------------------------------------------------------------------------
  # listKnownObserverSchemas()
  #
  #----------------------------------------------------------------------------
  def listKnownObserverSchemas(self):

    rc=mcat.get("undefined")
    stmt=None
    schemas=[]

    try:

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(self.observerSchemaSelect)
      column=rs.next()
      while column!=0:
        schema=rs.getString("OBSSCHEMA")
        schema=schema.strip()
        schemas.append(schema)
        column=rs.next();

    except :

      handleException()

    if stmt!=None:
      stmt.close()

    return schemas

  #----------------------------------------------------------------------------
  # getLogToUse()
  #----------------------------------------------------------------------------
  def getLogToUse(self, logToUse=None):

    if logToUse==None:
      if LogFile.defaultLog==None:
        logToUse=LogFile()
      else:
        logToUse=LogFile.defaultLog

    return logToUse


  #----------------------------------------------------------------------------
  # handleSqlException()
  #----------------------------------------------------------------------------
  def handleSqlException(self,xcp, printToConsole=0, logToUse=None):

    self.lastSqlException=xcp

    logToUse=self.getLogToUse(logToUse)
    ptc=logToUse.printToConsole

    # tempoary overwrite log settings per request
    if printToConsole:
        logToUse.printToConsole=1

    logToUse.log( xcp.getMessage() )
    logToUse.log( mcat.get("ErrorCode", xcp.getErrorCode()))
    logToUse.log( mcat.get("SqlState",  xcp.getSQLState()))
    logToUse.log( )

    nextXcp=xcp.getNextException()
    if nextXcp!=None:
      self.handleSqlException(nextXcp,printToConsole,logToUse)

    logToUse.printToConsole=ptc

    return xcp.getSQLState()

  #----------------------------------------------------------------------------
  # deleteTbsClause()
  #
  # delete a Tablespace clause in a SQL command for all TBSs
  # current restriction: case must matched manually
  #----------------------------------------------------------------------------
  def deleteTbsClause(self, cmd ):

    ex=Pattern.compile(self.tableSpaceClause, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE)
    m=ex.matcher(java.lang.String(cmd));
    if m.find():
      cmd=m.replaceAll("")  # replace all matches with "", thus delete them
    return cmd

  #----------------------------------------------------------------------------
  # checkforTablespaces()
  #
  # check if tablespaces listed in listOfTablespaces exist in a DB.
  # DBMS specific select clause needs to be passed in cmd.
  # return an list if one is missing.
  #----------------------------------------------------------------------------
  def checkforTablespaces(self,listOfTablespaces,cmd):

    if len(listOfTablespaces)==0:
      return ""

    missing=listOfTablespaces

    try:

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)

      column=rs.next()

      while column!=0:

        tbs=rs.getString(column)
        tbs=tbs.strip()
        if str(listOfTablespaces).find(tbs)>-1 :
          missing.remove(tbs)

        column=rs.next();

    except :

      handleException()

    if stmt!=None:
      stmt.close()

    return missing


  #----------------------------------------------------------------------------
  # prepareCommandList()
  #----------------------------------------------------------------------------
  def prepareCommandList(self):

    resultList= []
    inBlock=0
    jdbcCmd=""
    isFunction=0

    self.replaceTokens()

    for line in self.commandBuffer:


       #print ( line ),
       if line.strip() == "":
         #log( "skip empty" )
         continue

       if not inBlock and line.strip().upper().find("OBSVR_FUNCTION_BEGIN")>-1:
         inBlock=1
         #print "BLOCK",
         continue

       if inBlock and line.strip().upper().find("OBSVR_FUNCTION_END")>-1:
         inBlock=0
         #stmt complete
         resultList.append(jdbcCmd)
         jdbcCmd=""
         #print "ENDBLOCK",
         continue

       if line.strip()[0:2] == "--":   # line is comment
         #log( "skip comment" )
         continue

       if line.strip().upper() == "QUIT":   # Oracle SQLPlus STMT, no SQL
          continue
       if line.strip()== "/":   # Oracle SQLPlus STMT, no SQL
          continue

       if not inBlock:

         if line.find(";")>-1:
           line=line.replace(";"," ")
           jdbcCmd=jdbcCmd+line
           resultList.append(jdbcCmd)
           jdbcCmd=""

         else:
           jdbcCmd=jdbcCmd+line

       else:
         jdbcCmd=jdbcCmd+line
         #print "BlockStmt",


    # Filter out the tablespaces if desired
    if self.useDefaultTablespace==1:
      for line in resultList:
        resultList[resultList.index(line)]=self.deleteTbsClause(line)

    return resultList

  #----------------------------------------------------------------------------
  # executeCommandBuffer()
  #----------------------------------------------------------------------------
  def executeCommandBuffer(self,interactive=0, logToUse=None):
      rs = 0
      rc = 0
      stmtCount = 0
      cmdLen =""
      cmdList =""
      cmd = ""
      cmdOut =""

      cmdList = self.prepareCommandList()
      cmdCount=len(cmdList)

      if self.delayConfig:
        f=open(self.sqlOutFile,"a+")
        for cmd in cmdList:
          f.write(cmd)

        f.close()

        return rc

      logToUse=self.getLogToUse(logToUse)
      stmt = self.connection.createStatement()

      ######################################
      # now execute the buffer
      for cmd in cmdList:

         stmtCount=stmtCount+1

         try:
           logOut=mcat.get("ExecutingSQLStmt")  + str(stmtCount) + ": " + cmd
           if logOut[-1]=="\n":
             logOut=logOut[0:-1]  # remove last newline for from output
           logToUse.log( logOut )
           print "\b" + mcat.get("ExecutingSQLStmtOf", str(stmtCount), str(cmdCount) ),  "\r\b" , #please leave comma
           rs = stmt.executeUpdate(cmd)
           logToUse.log( mcat.get("ResultCode") + str(rs) )

         except java.sql.SQLException, inst :
           logToUse.log( mcat.get("ErrorOnStatement", str(stmtCount)) )
           self.handleSqlException(inst, interactive, logToUse)
           rc=-1

         except :

           logToUse.log( lineSep )
           logToUse.log( mcat.get("ErrorOnStatement", str(stmtCount)) )
           handleException(interactive, not interactive, "", logToUse)   # if we are non-interactive, we log only to the logfile
           rc=-1

      print "                                                  "
      self.connection.commit()
      stmt.close()

      return rc


  #----------------------------------------------------------------------------
  # loadDdlFile()
  #----------------------------------------------------------------------------
  def loadDdlFile(self, filename):

    f=open(filename,'r')
    self.commandBuffer=f.readlines()

 #      for line in lines :
 #         self.commandBuffer=self.commandBuffer+line

    f.close

  #----------------------------------------------------------------------------
  # databaseLogfileName()
  #
  # return the name of the DB logfile as defined by the Common DB LI
  #----------------------------------------------------------------------------
  def databaseLogfileName(self, isObserver=0):

    sep="_"
    fn  = self.getDbmsType()     + sep
    fn += self.getDatabaseName() + sep
    fn += self.getSchema()
    if isObserver:
      fn += sep + 'Observer'

    fn += ".log"

    fn = os.path.join(AppServer.logLocation, fn)

    return fn


  #----------------------------------------------------------------------------
  # replaceTokens()
  #
  # replace tokens in the commandBuffer listed in tokensToReplace with given values
  # List tokensToReplace must consist of tupels as follows t="TOKENNAME","SUBSTITUE"
  # e.g t1="@DB_NAME@","OBSVRDB"
  #----------------------------------------------------------------------------
  def replaceTokens(self):

    for lineNo in range(0,len(self.commandBuffer)):
      for token in self.tokensToReplace.keys():
        self.commandBuffer[lineNo]=self.commandBuffer[lineNo].replace(token, self.tokensToReplace[token])

  #----------------------------------------------------------------------------
  # runDdlFile
  #----------------------------------------------------------------------------
  def runDdlFile(self, sqlFile,interactive=0,logfile=None):

    self.loadDdlFile(sqlFile)
    return self.executeCommandBuffer(interactive, logfile)


  #----------------------------------------------------------------------------
  # runSqlViaAdminTask
  #
  # fill out and execute a SQL file via the Admin Task
  # processBPCDatabaseScript ( to fulfill CommonDB Li)
  #
  #----------------------------------------------------------------------------
  def runSqlViaAdminTask(self, msg="ProcessSqlFile", addtionalParameters="", dbNameOverwrite="" ):

    #######################################################
    # install the collector
    #######################################################
    log( mget(msg) )

    ##########################################
    # -createNew (true|false)
    #   use an existing or create a new db
    #   currently for observer only use existing is supported
    createNewClause=" -createNew " + str(Boolean(self.createDatabase))

    ##########################################
    # -delayConfig (true|false)
    #   true:  only the SQL file should be created
    #   false: create SQL file an execute in DB
    delayConfigClause=" -delayConfig " + str(Boolean(self.delayConfig))

    ##########################################
    # -databaseType
    #   the type of the DB as the name is in the fileSystem under
    #   dbscripts/ProcessChoreographer
    databaseTypeClause =" -databaseType " + self.getDbmsType()

    ##########################################
    # -databaseName
    # the name of the DB
    if dbNameOverwrite!="":
      dbName=dbNameOverwrite
    else:
      dbName=self.getDatabaseName()
    if dbName.find(";") > -1:
      dbName=dbName[0:dbName.find(";")]
    databaseNameClause=" -databaseName " + dbName

    ##########################################
    # -databaseSchema
    # the schema in the db to be used
    databaseSchemaClause=" -databaseSchema " + self.getSchema()

    ##########################################
    # -databaseUser
    # the userid to connect to the DB
    if self.getDatabaseUser()=="":
      databaseUserClause=""
      databasePasswordClause=""

    else:

      databaseUserClause=" -databaseUser " + self.getDatabaseUser()

      ##########################################
      # -databaseUser
      # the pwd to connect to the DB
      databasePasswordClause=" -databasePassword " + self.getDatabasePassword()

    databasePortClause=""
    databaseServerClause=""

    ##########################################
    # -databaseServer
    # the server hosting the DB
    if self.jdbcDriverType==4:
      databaseServerClause=" -databaseServer " + self.connectionHost

      ##########################################
      # -databasePort
      # the port where the server is listening
      databasePortClause=" -databasePort " + str(self.connectionPort)



    ##########################################
    # -jdbcDriverPath
    # the port where the server is listening
    jdbcDriverPathClause=' -jdbcDriverPath "' + os.path.normpath(self.jdbcHome) + '"'

    ##########################################
    # -observer (true|false)
    # indicate to AdminTask that this is for the Observer
    observerClause=" -observer true"

    ##########################################
    # -udfDir (path)
    # indicate where the jar file bpcodbutil
    # can be found on the database server
    udfJarClause=' -udfDir "' + self.jarLocation.replace("\\","/") + '"'

    ##########################################
    # -outputDir
    # tell where to place the oufilled SQL file
    # currently not used
    outputdirClause=""

    ##########################################
    # -tablespace (true|false)
    # tell if the tablespace clauses should
    # be omitted. Currently it is implemented
    # as follwow:
    # if -tablespacedir is missing:
    #   do not create TBS and also omit TBS clauses
    # if -tablespacedir is set but w/o value:
    #   do not create TBS but use TBS clauses
    # if -tablespacedir is set with value x:
    #   create TBS in dir x and use TBS clauses
    if self.useDefaultTablespace==1:
      tablespaceClause=''
    else:
      tablespaceClause=' -tablespaceDir '


    ##########################################
    # -udf (Sql|Java)
    # tell if the JAVA or SQL implementation of
    # the intervalin function should be used
    if self.udfImplementation==DatabaseManager.UDF_IMPL_JAVA:
      udfClause=" -udf Java"
    else:
      udfClause=" -udf Sql"


    #compose the command
    #cmdline="[" + dsClause + jmsClause + srvClause + meClause + schemaClause +"]"
    cmdline  = "["
    cmdline += createNewClause + delayConfigClause + databaseTypeClause
    cmdline += databaseNameClause + databaseSchemaClause + databaseUserClause
    cmdline += databasePasswordClause + databaseServerClause + databasePortClause
    cmdline += jdbcDriverPathClause + observerClause + outputdirClause
    cmdline += tablespaceClause + udfClause + udfJarClause + addtionalParameters
    cmdline += "]"

    if databasePasswordClause!="":
      logout="AdminTask.processBPCDatabaseScript('" + cmdline.replace(databasePasswordClause, " -databasePassword ********") + "')"
    else:
      logout="AdminTask.processBPCDatabaseScript('" + cmdline + "')"

    logOnly( logout  )

    resultList = AppServer.AdminTask.processBPCDatabaseScript( cmdline ).split(lineSep)

    self.sqlViaAdminTaskResults={ "exception" : "" ,
                                  "sqlfile"   : "" ,
                                  "logfile"   : "" }  # cleanup exisiting results
    for line in resultList :
      if line=="" or line.find("=") == -1:
        continue

      key=line.split("=",1)[0]
      val=line.split("=",1)[1]
      self.sqlViaAdminTaskResults[key]=val

    return self.sqlViaAdminTaskResults



  #----------------------------------------------------------------------------
  # removeJarFile()
  #
  # remove a jar file installed in the db
  # if supported by DBMS, should be implemented in the children
  #----------------------------------------------------------------------------
  def removeJarFile(self, _jarId="", _jarPath=""):

    print mcat.get("NewLine")
    print mcat.get("CurrentlyNotSupported", self.dbmsType)


  #----------------------------------------------------------------------------
  #
  # promptCredentials
  #
  # prompt uid and pwd for a given user of type bpelib.UserId()
  #----------------------------------------------------------------------------
  def promptCredentials(self, promptTitle="PromptDbUser"):

    user=None

    ########################
    # prompt for the user
    print mcat.get("NewLine")
    user=UserId()
    user.promptUserName(mcat.get(promptTitle, self.databaseName) , self.getDatabaseUser())

    ########################
    # get password
    if user.username.strip()=="":
      user.password=""
    else:
      print mcat.get("NewLine")
      user.promptPassword()

    self.databaseUser     = user.username
    self.databasePassword = user.password

    return user

  #----------------------------------------------------------------------------
  #
  # promptJdbcHome
  #
  #----------------------------------------------------------------------------
  def promptJdbcHome(self, serverName="", promptTitle="promptJdbcHome"):

    print mcat.get("NewLine")

    if self.jdbcHome=="":
      jdbcHome=self.resolveJdbcHome(serverName)
    else:
      jdbcHome=self.jdbcHome

    while 1:

      newJdbcHome = console.selectString(mcat.get(promptTitle), jdbcHome )

      # check for existance of path
      if not os.path.exists(newJdbcHome):
        print
        print mget("ErrorPathDoesNotExist", newJdbcHome)
        print
      else:
        break

    self.jdbcHome=newJdbcHome

    return self.jdbcHome

  #----------------------------------------------------------------------------
  #
  # checkJdbcHome
  #
  #----------------------------------------------------------------------------
  def checkJdbcHome(self, jdbcHome=""):

    rc=""
    missingFiles=[]
    if jdbcHome=="":
      jdbcHome=self.jdbcHome

    # check for existance of path
    if not os.path.exists(self.jdbcHome):
      rc=mget("ErrorPathDoesNotExist", self.jdbcHome)

    #check for required files
    for jar in self.jdbcDriverJars:
      jarFile=os.path.join(self.jdbcHome, jar)
      if not os.path.exists(jarFile):
        missingFiles.append(jarFile)

    if len(missingFiles)!=0:
      rc=mget("ErrorJarMissing", str(missingFiles)[1:-1])

    return rc

  #----------------------------------------------------------------------------
  #
  # promptSchema
  #
  #----------------------------------------------------------------------------
  def promptSchema(self, promptTitle="PromptSchema"):

    print mcat.get("NewLine")
    schema = console.selectString(mcat.get(promptTitle), self.getSchema() )
    self.setSchema(schema)

    return self.getSchema()

  #----------------------------------------------------------------------------
  #
  # promptDatabaseHostName
  #
  #----------------------------------------------------------------------------
  def promptDatabaseHostName(self, promptTitle="PromptServerHost"):

    print mcat.get("NewLine")
    self.connectionHost = console.selectString(mcat.get(promptTitle), self.connectionHost)
    return self.connectionHost

  #----------------------------------------------------------------------------
  #
  # promptDatabasePort
  #
  #----------------------------------------------------------------------------
  def promptDatabasePort(self, promptTitle="PromptListenerPort"):

    print mcat.get("NewLine")
    self.connectionPort=console.selectInteger(mcat.get(promptTitle), self.connectionPort)
    return self.connectionPort

  #----------------------------------------------------------------------------
  #
  # promptDatabaseName
  #
  #----------------------------------------------------------------------------
  def promptDatabaseName(self, promptTitle="PromptDbName"):

    print mcat.get("NewLine")
    self.databaseName = console.selectString(mcat.get("PromptDbName"),self.databaseName)
    return self.databaseName

  #----------------------------------------------------------------------------
  # listJars()
  #
  #----------------------------------------------------------------------------
  def listJars(self,jarId):

    print mcat.get("NewLine")
    print mcat.get("CurrentlyNotSupported", self.dbmsType)


  #----------------------------------------------------------------------------
  # Getters and Setters
  #----------------------------------------------------------------------------

  def getConnection(self):
      return self.connection

  def setConnection(self, connection):
      self.connection = connection


  def getConnectionPrefix(self):
      return self.connectionPrefix

  def setConnectionPrefix(self, connectionPrefix):
      self.connectionPrefix = connectionPrefix


  def getDatabaseName(self):
      return self.databaseName

  def setDatabaseName(self, databaseName):
      self.databaseName = databaseName


  def getDatabasePassword(self):
      return self.databasePassword

  def setDatabasePassword(self, databasePassword):
      self.databasePassword = databasePassword


  def getDatabaseUser(self):
      return self.databaseUser

  def setDatabaseUser(self, databaseUser):
      self.databaseUser = databaseUser


  def getDbmsType(self):
      return self.dbmsType

  def setDbmsType(self, dbmsType):
      self.dbmsType = dbmsType


  def getDriverName(self):
      return self.driverName

      self.driverName = driverName


  def getCommandBuffer(self):
      return self.commandBuffer

  def setCommandBuffer(self, commandBuffer):
      self.commandBuffer = commandBuffer


  def getDatabaseType(self):
      return self.databaseType

      self.databaseType = databaseType


  def getSchema(self):
      return self.__schema

  def setSchema(self, schema):
      self.__schema = schema.upper()
      self.tokensToReplace[self.schemaToken]=self.__schema



#----------------------------------------------------
# class for DB2 databases (primarily DB2 universal)
#----------------------------------------------------
class DB2(DatabaseManager):

  JARFILES_UNIVERSAL    = [ "db2jcc.jar", "db2jcc_license_cisuz.jar", "db2jcc_license_cu.jar"  ]

  DATASOURCE_UNIVERSAL  = "com.ibm.db2.jcc.DB2SimpleDataSource"

  WAS_VAR_DB2_UNIVERSAL = "DB2UNIVERSAL_JDBC_DRIVER_PATH"

  def __init__(self):

    DatabaseManager.__init__(self)

    self.setDbmsType(self.DBMS_TYPE_DB2)
    self.setConnectionPrefix("jdbc:db2:")

    self.setDatabaseName("BPEDB")
    self.setDatabasePassword("")

    name=javaEnv["user.name"].strip()
    if name!="" or name!=None:
      self.setDatabaseUser(name)
    else:
      self.setDatabaseUser("db2admin")

    # names of database objects that vary over the differnt implemtations of db2
    # to be overwritten in childs
    self.jarTableName="SYSIBM.SYSJAROBJECTS"
    self.jarAlteredField="ALTEREDTS"
    self.jarCreatedField="CREATEDTS"

    self.ObserverTableSpaces=self.retrieveTablespacesFromFile()
    self.observerSchemaSelect="SELECT TABSCHEMA AS OBSSCHEMA FROM SYSCAT.TABLES WHERE TABNAME = 'OBSERVER_VERSION' AND TYPE = 'T'"

    self.tableSpaceClause=r"\bIN (\w+)\b"  # equals " IN <TBSNAME>", where TBSNAME is put into matcher.group(1)

    self.jdbcDriverType=2

    # default port for type 4
    self.connectionPort = 50000

    self.dataSourceImpl=DB2.DATASOURCE_UNIVERSAL
    self.jdbcDriverJars=DB2.JARFILES_UNIVERSAL
    self.wasVarJdbcDriverPath=DB2.WAS_VAR_DB2_UNIVERSAL


  #----------------------------------------------------
  # handleSqlException()
  #
  # DB2 specific error handling different to JDCB error
  # handling
  #----------------------------------------------------

  def handleSqlException(self,xcp,printToConsole=0,  logToUse=None):

    rc=0

    logToUse=self.getLogToUse(logToUse)
    ptc=logToUse.printToConsole

    # tempoary overwrite log settings per request
    if printToConsole:
        logToUse.printToConsole=1


    try:
      cl=self.urlClassloader.loadClass("com.ibm.db2.jcc.DB2Diagnosable")
    except:
      # if we can not load the DB2 specific classes, delegate to
      # generic SQL XCP handler
      return DatabaseManager.handleSqlException(self,xcp,printToConsole,logToUse)

    if not cl.isInstance(xcp):
      # if we do not have a diagnosable also delegate to
      # generic SQL XCP handler
      return DatabaseManager.handleSqlException(self,xcp,printToConsole,logToUse)

    sqlca=xcp.getSqlca()

    if sqlca==None:
      return DatabaseManager.handleSqlException(self,xcp,printToConsole,logToUse)

    errorCode=sqlca.getSqlCode()
    sqlState=sqlca.getSqlState()
    try:
      msg=sqlca.getMessage()
    except :
      msg="<message retrival failed>"

    logToUse.log( msg )
    logToUse.log( mcat.get("ErrorCode", errorCode ))
    logToUse.log( mcat.get("SqlState", sqlState ))
    logToUse.log( )

    self.lastSqlException=java.sql.SQLException(msg, sqlState, errorCode)

    logToUse.printToConsole=ptc

    return sqlState


  #----------------------------------------------------------------------------
  #
  # promptJdbcConnectionType
  #
  # set the internally used type of the datasource
  #
  #----------------------------------------------------------------------------
  def promptJdbcConnectionType(self, promptTitle="PromptJdbcConnectionType"):

    ## generate menu

    menuList =[]
    validAnswers =[]
    itemCount=0

    # Type2
    JDBCType2=itemCount
    menuList.append( mcat.get("Menu_JDBCType2") )
    validAnswers.append('2')
    if self.jdbcDriverType==2:
      default=itemCount
    itemCount+=1

    # Type4
    JDBCType4=itemCount
    menuList.append( mcat.get("Menu_JDBCType4")+"\n")
    validAnswers.append('4')
    if self.jdbcDriverType==4:
      default=itemCount
    itemCount+=1

    # prompt for database manager
    print mcat.get("NewLine")

    # show menu
    inp= console.selectFromList( mcat.get(promptTitle)+"\n", validAnswers, default , menuList )

    if inp == JDBCType2:
      self.jdbcDriverType=2

    elif inp == JDBCType4:
      self.jdbcDriverType=4

    return self.jdbcDriverType

  #----------------------------------------------------------------------------
  # connect()
  #
  # make a singelton connection to the DB
  #----------------------------------------------------------------------------
  def connect(self):

    rc=0

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."

    try:


      if self.connection==None:
        databaseName=self.databaseName
        self.dataSource.setDatabaseName(databaseName)
        self.dataSource.setDriverType( self.jdbcDriverType )
        if self.jdbcDriverType==4:
          self.dataSource.setServerName(self.connectionHost)
          port=int(self.connectionPort)
          self.dataSource.setPortNumber( port )

        self.connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)

        logOnly( mcat.get("ConnectedTo", databaseName) )


    except java.sql.SQLException, xcp :
      rc=xcp
      raise

    return rc

  #----------------------------------------------------------------------------
  # checkforObserverTablespace()
  #
  # check for tablespace prereqs for an evnet collector db setup
  #----------------------------------------------------------------------------
  def checkforObserverTablespace(self):

    # check if there is our tablespace OBSVRTS
    # to do so, we try to create a test table in it
    tbsAsString=str(self.ObserverTableSpaces)[1:-1]
    cmd="SELECT TBSPACE FROM SYSCAT.TABLESPACES WHERE TBSPACE IN (" + tbsAsString + ")"
    return self.checkforTablespaces( self.ObserverTableSpaces , cmd)


  #----------------------------------------------------------------------------
  # checkforJarInstalledInDb()
  #
  # check if the jar with the JAR_ID is installed in the database
  # Please note that we do a fully qualified query using the schema
  # return 1 if installed, 0 otherwise
  #----------------------------------------------------------------------------
  def checkforJarInstalledInDb(self, jarId):

    rc=0
    stmt=None
    schema = self.getSchema()
    jarId  = jarId.upper()

    try:

      # check if we have a fully qualified jarId like DB2ADMIN.bpcodbutil
      if jarId.find(".") >-1 :
        parts=jarId.split(".",1)
        schema = parts[0]
        jarId  = parts[1]

      cmd = "SELECT JAR_ID,JARSCHEMA FROM " + self.jarTableName + " WHERE JAR_ID IN ('"+ jarId + "')"
      cmd += " AND JARSCHEMA IN ('" + schema + "')"
      cmdOut=mcat.get("ExecutingSQLStmt") + "1" + ": " + cmd
      logOnly( cmdOut )
      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)
      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        schema=rs.getString("JARSCHEMA")
        jid=rs.getString("JAR_ID")
        logOnly(mcat.get("JarFileAllreadyInstalled",schema.strip() + '.' + jid, self.databaseName))
        rc=1

    except java.sql.SQLException, xcp:
      self.handleSqlException(xcp)

    except :
      handleException()

    if stmt!=None:
      stmt.close()
    return rc


  #----------------------------------------------------------------------------
  # listJars()
  #
  #----------------------------------------------------------------------------
  def listJars(self,jarId):

    rc=0
    stmt=None
    schema = self.getSchema()
    jarId  = jarId.upper()

    try:

      # check if we have a fully qualified jarId like DB2ADMIN.bpcodbutil
      if jarId.find(".") >-1 :
        parts=jarId.split(".",1)
        jarId  = parts[1]
        schema = parts[0]

      cmd = "SELECT JAR_ID,JARSCHEMA," + self.jarAlteredField + ","
      cmd += self.jarCreatedField + " FROM " + self.jarTableName
      cmd += " WHERE JAR_ID IN ('"+ jarId + "')"

      cmdOut=mcat.get("ExecutingSQLStmt") + "1" + ": " + cmd
      logOnly( cmdOut )

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)

      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        print "JARSCHEMA".ljust(9), "JAR_ID".ljust(12), "Last Update (local time)".ljust(12)
        print "----------------------------------------------------------------"
      else:
        print mget("NoJarFileFound")
      while row==1:
        schema=rs.getString("JARSCHEMA")
        jid=rs.getString("JAR_ID")
        lastUpdate=rs.getString(self.jarAlteredField)
        created=rs.getString(self.jarCreatedField)

        if lastUpdate==None:
          if created==None:
            lastUpdate=""
          else:
            lastUpdate=created

        if lastUpdate!=None and lastUpdate!="":
          lastUpdate=lastUpdate+"0000000"  # if there are zeros at the end, they are skipped, see 452197

        print schema.ljust(9), jid.ljust(12), lastUpdate[0:26]
        row=rs.next()

      if stmt!=None:
        stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise


  #----------------------------------------------------------------------------
  # removeJarFile()
  #
  # remove a jar file installed in the db
  # _jarPath is not used for DB2
  #----------------------------------------------------------------------------
  def removeJarFile(self, _jarId, _jarPath=""):

    jarId = self.getSchema() + "." + _jarId.upper()
    self.connection.removeDB2JavaStoredProcedure( jarId )



  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # install a jar file containing UDFs etc
  # requires to be connected
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jarId, replaceExisting=1, refreshClasses=1):

    stmt=None

    try:

      #to ensure we get the correct match in the syscat, we need to do to_upper
      jarId = jarId.upper()

      #check if we must replace it
      replace=0
      if self.checkforJarInstalledInDb(jarId):
        if replaceExisting:
          log(mcat.get("JarFileUpdate",jarId))
          replace=1
        else:
          log(mcat.get("NoJarFileUpdate",jarId))

      # For DB2, we need to have the File as a BLOB
      # because it seems that sqlj.install_jar() is not avail over JDBC
      log(mcat.get("JarFileInstalling",jarFileName))
      logOnly(mcat.get("JarFileId",jarId))
      jar=java.io.File(jarFileName)
      jarFis=java.io.FileInputStream(jar)
      fileAsBlob=jarray.zeros(jar.length(),'b')
      # to create a BLOB, we need a java array
      #jarFis.read(fileAsBlob)

      if replace==1:

        self.connection.replaceDB2JavaStoredProcedure(jarFis,jar.length(), jarId)

        # stmt = self.connection.prepareCall("CALL SQLJ.DB2_REPLACE_JAR(?,?,?)");
        # stmt.setString(1, jarId);
        # stmt.executeUpdate();
      else:
        self.connection.installDB2JavaStoredProcedure(jarFis,jar.length(), jarId)

      if replace==1 and refreshClasses==1 :
        #and force DB2 to reload the classes
        stmt = self.connection.prepareCall("CALL SQLJ.REFRESH_CLASSES()");
        stmt.executeUpdate();

      #close stmt
      if stmt!=None:
        stmt.close()

      self.connection.commit()

    except:
      #close stmt
      if stmt!=None:
        self.connection.rollback()
        stmt.close()
      raise

  #----------------------------------------------------------------------------
  #
  # promptDatabaseName
  #
  #----------------------------------------------------------------------------
  def promptDatabaseName(self, promptTitle=""):

    print mcat.get("NewLine")

    if promptTitle=="":
      if self.jdbcDriverType==2:
        msg="PromptLocalDbName"
      else:
        msg = "PromptDbName"

    inp = console.selectString(mcat.get(msg),self.databaseName)
    self.databaseName=inp.upper()
    return self.databaseName


  #----------------------------------------------------------------------------
  #
  # resolveJdbcHome
  #
  #----------------------------------------------------------------------------
  def resolveJdbcHome(self, serverName=""):

    jdbcHome=""
    dbHome=osEnv["DB2PATH"]

    if dbHome!=None and dbHome.strip()!="":
      dbHome=dbHome.strip()
      jdbcHome=os.path.join(dbHome,"java")

    if jdbcHome=="":
      jdbcHome=DatabaseManager.resolveJdbcHome(self, serverName)

    # we have problems with this driver. Don't offerit.
    #if jdbcHome=="" :
    #  jdbcHome=os.path.join(javaEnv["was.install.root"], "universalDriver_wbi/lib")

    return jdbcHome

#----------------------------------------------------
# class for DB2 z/OS databases
# (overwrite/inherit  from DB2 universal)
#----------------------------------------------------
class DB2zOS(DB2):
  "Abstract class for common DB2 Datasources, USE DB2zOSV8 or DB2zOSV9!"

  def __init__(self):

    DB2.__init__(self)

    self.__storageGroup="OBSVRSG"
    self.__wlmId=""
    self.__locationName = ""
    self.__remoteDatabaseName="OBSVRDB"

    # tokens to be replaced in the SQL files
    self.remoteDatabaseNameToken = "@DB_NAME@"
    self.storageGroupToken       = "@STOGRP@"
    self.wlmIdToken              = "@WLMID@"

    # default port for type 4
    self.connectionPort = 446

    self.observerSchemaSelect="SELECT CREATOR AS OBSSCHEMA FROM SYSIBM.SYSTABLES WHERE NAME = 'OBSERVER_VERSION' AND TYPE = 'T'"

    self.runningInUSS= (javaEnv["os.name"].find("z/OS") > -1)

    self.jdbcDriverType=2
    if self.runningInUSS:
      self.jdbcDriverType=4

  # setters for the private members. This is because we need syncronize the tokens

  def getRemoteDatabaseName(self):
    return self.__remoteDatabaseName

  def setRemoteDatabaseName(self,name):
    self.tokensToReplace[self.remoteDatabaseNameToken]=name
    self.__remoteDatabaseName=name


  def getStorageGroup(self):
    return self.__storageGroup

  def setStorageGroup(self,name):
    self.tokensToReplace[self.storageGroupToken]=name
    self.__storageGroup=name


  def getLocationName(self):
    return self.__locationName

  def setLocationName(self,name):
    self.__locationName=name


  def getWlmId(self):
    return self.__wlmId

  def setWlmId(self,name):
    self.tokensToReplace[self.wlmIdToken]=name
    self.__wlmId=name

  #----------------------------------------------------------------------------
  # databaseLogfileName()
  #
  # return the name of the DB logfile as defined by the Common DB LI
  #----------------------------------------------------------------------------
  def databaseLogfileName(self, isObserver=0):

    sep="_"
    fn  = self.getDbmsType()     + sep
    fn += self.getRemoteDatabaseName() + sep
    fn += self.getSchema()
    if isObserver:
      fn += sep + 'Observer'

    fn += ".log"

    fn = os.path.join(AppServer.logLocation, fn)

    return fn

  #----------------------------------------------------------------------------
  #
  # promptDatabaseName
  #
  #----------------------------------------------------------------------------
  def promptLocationName(self, promptTitle="PromptDbLocationName"):

    print mcat.get("NewLine")
    inp = console.selectString(mcat.get(promptTitle),self.getLocationName())
    self.setLocationName(inp.upper())
    return self.getLocationName()

  #----------------------------------------------------------------------------
  # connect()
  #
  # make a singelton connection to the DB
  #----------------------------------------------------------------------------
  def connect(self):

    rc=0

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."

    try:

      if self.connection==None:
        # on z/OS we connect to the subsytem/location rather than to a db
        self.dataSource.setDriverType( self.jdbcDriverType )
        if self.jdbcDriverType==4:
          self.dataSource.setServerName(self.connectionHost)
          port=int(self.connectionPort)
          self.dataSource.setPortNumber( port )
          databaseName=self.getLocationName()
        else:
          databaseName=self.getDatabaseName()  # name in the local db dir

        self.dataSource.setDatabaseName(databaseName)
        self.connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)

        logOnly( mcat.get("ConnectedTo", databaseName) )


    except java.sql.SQLException, xcp :
      rc=xcp
      raise

    return rc

  #----------------------------------------------------------------------------
  #
  # promptRemoteDatabaseName
  #
  # DB2 z/OS : prompt for remote DB name as known by the subsystem
  #
  #----------------------------------------------------------------------------
  def promptRemoteDatabaseName(self, promptTitle="PromptRemoteDbName"):

      print mcat.get("NewLine")
      inp = console.selectString(mcat.get("PromptRemoteDbName"),self.getRemoteDatabaseName() )
      self.setRemoteDatabaseName(inp.upper())
      return self.getRemoteDatabaseName()

  #----------------------------------------------------------------------------
  #
  # promptDatabaseName
  #
  # For type 2, we need the local name and the remote db name
  # For type 4, we need the subsystem and the remote db name
  #
  #----------------------------------------------------------------------------
  def promptDatabaseName(self, promptTitle=""):

    if self.jdbcDriverType==2:
      # For type 2, we need the local name and the remote db name
      DB2.promptDatabaseName(self)
      self.promptRemoteDatabaseName()

    else:
      # For type 4, we need the subsystem/location and the remote db name
      self.promptLocationName()
      self.promptRemoteDatabaseName()
      # for type 4, connection is done to the subsystem
      self.databaseName=self.getRemoteDatabaseName()

    return self.databaseName


  #----------------------------------------------------------------------------
  #
  # promptWlmEnvironmentId
  #
  #----------------------------------------------------------------------------
  def promptWlmEnvironmentId(self, promptTitle="PromptWlmEnvironmentId"):

    print mcat.get("NewLine")
    inp=console.selectString(mcat.get(promptTitle), self.getWlmId() )
    self.setWlmId(inp)
    return self.getWlmId()

  #----------------------------------------------------------------------------
  #
  # promptStorageGroup
  #
  #----------------------------------------------------------------------------
  def promptStorageGroup(self, promptTitle="PromptStorageGroup"):

    print mcat.get("NewLine")
    inp=console.selectString(mcat.get(promptTitle), self.getStorageGroup() )
    self.setStorageGroup(inp)
    return self.getStorageGroup()


  #----------------------------------------------------------------------------
  # checkforObserverTablespace()
  #
  # check for tablespace prereqs for an evnet collector db setup
  #----------------------------------------------------------------------------
  def checkforObserverTablespace(self):

    # check if there is our tablespace OBSVRTS
    # to do so, we try to create a test table in it
    tbsAsString=str(self.ObserverTableSpaces)[1:-1]
    cmd="SELECT NAME FROM SYSIBM.SYSTABLESPACE WHERE NAME IN (" + tbsAsString + ")"

    if self.getRemoteDatabaseName()!="":
      cmd+="AND DBNAME IN ('" + self.getRemoteDatabaseName() + "')"
    else:
      cmd+="AND DBNAME IN ('" + self.databaseName + "')"

    return self.checkforTablespaces( self.ObserverTableSpaces , cmd)

  #----------------------------------------------------------------------------
  # grantExecuteOnIntervalin()
  #
  # If connectuser != SQLID, execute must be granted to the functions
  #----------------------------------------------------------------------------
  def grantExecuteOnIntervalin(self):

    functionNames = [ "INTERVALIN_TS_VC",
                      "INTERVALIN_TS_TS",
                      "INTERVALIN_VC_TS",
                      "INTERVALIN_VC_VC" ]

    stmt = self.connection.createStatement()

    for fkt in functionNames :

      cmd = "GRANT EXECUTE ON SPECIFIC FUNCTION "
      cmd += self.getSchema().upper() + "." + fkt + " TO " + self.getSchema().upper()

      rs = 0

      try:
        cmdOut=mcat.get("ExecutingSQLStmt") + ": " + cmd
        logOnly( cmdOut )
        rs = stmt.executeUpdate(cmd)
        logOnly( mcat.get("ResultCode") + str(rs) )

      except java.sql.SQLException, inst :
        logOnly( mcat.get("ErrorOnStatement", "") )
        stmt.close()
        raise

    stmt.close()

    return


  #----------------------------------------------------------------------------
  # deleteTbsClause()
  #
  # overwrite from DatabaseManager,
  # replace the Tablespace clause in a SQL command for all TBSs
  # current restriction: case must matched manually
  #----------------------------------------------------------------------------
  def deleteTbsClause(self, cmd ):

    self.tableSpaceClause=r"\bIN " + self.databaseName + "\.(\w+)\b"  # equals " IN <DBNAME>.<TBSNAME>"
    ex=Pattern.compile(self.tableSpaceClause, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE)
    m=ex.matcher(java.lang.String(cmd));
    if m.find():
      cmd=m.replaceAll("IN DATABASE " + self.databaseName )  # replace all matches with "", thus delete them

    return cmd


  #----------------------------------------------------------------------------
  # runSqlViaAdminTask
  #
  # fill out and execute a SQL file via the Admin Task
  # processBPCDatabaseScript ( to fulfill CommonDB Li)
  #
  #----------------------------------------------------------------------------
  def runSqlViaAdminTask(self, msg="ProcessSqlFile", addtionalParameters=""):

    ##########################################
    # -storageGroup
    #   specify the string that replaces @STOGRP@
    storageGroupClause=" -storageGroup " + self.getStorageGroup()

    ##########################################
    # -wlmId
    #   specify the string that replaces @WLMID@
    wlmIdClause=" -wlmID " + self.getWlmId()

    ##########################################
    # -connectionTarget
    #   specify the location name of the subsystem
    if self.jdbcDriverType==4:
      locationNameClause=" -connectionTarget " + self.getLocationName()
    else:
      locationNameClause=" -connectionTarget " + self.getDatabaseName()

    addCmdLine=storageGroupClause + wlmIdClause + locationNameClause + " " + addtionalParameters

    result=DatabaseManager.runSqlViaAdminTask(self, msg, addCmdLine, self.getRemoteDatabaseName())

    return result


  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # overwrite of DB2: install a jar file containing UDFs etc
  # requires to be connected
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jarId, replaceExisting=1, refreshClasses=0):

    # no refresh possible for z/OS
    return DB2.installJarFile(self, jarFileName, jarId, replaceExisting, 0)

#----------------------------------------------------
# class for DB2 z/OS V8 databases
# (child of DB2zOS universal)
#----------------------------------------------------
class DB2zOSV8(DB2zOS):

  def __init__(self):

    DB2zOS.__init__(self)

    self.setDbmsType(self.DBMS_TYPE_DB2ZOSV8)
    self.ObserverTableSpaces=self.retrieveTablespacesFromFile()

#----------------------------------------------------
# class for DB2 z/OS V9 databases
# (child of DB2zOS universal)
#----------------------------------------------------
class DB2zOSV9(DB2zOS):

  def __init__(self):

    DB2zOS.__init__(self)

    self.setDbmsType(self.DBMS_TYPE_DB2ZOSV9)
    self.ObserverTableSpaces=self.retrieveTablespacesFromFile()

#----------------------------------------------------
# class for DB2iSeries databases
# (overwrite/inherit  from DB2 universal)
#----------------------------------------------------
class DB2iSeries(DB2):


  DATASOURCE_NATIVE  = "com.ibm.db2.jdbc.app.UDBDataSource"        # local connections only
  DATASOURCE_TOOLBOX = "com.ibm.as400.access.AS400JDBCDataSource"  # remote connections

  JARFILES_NATIVE   = [ "db2_classes.jar" ]
  JARFILES_TOOLBOX  = [ "jt400.jar" ]

  WAS_VAR_DB2_NATIVE  = "OS400_NATIVE_JDBC_DRIVER_PATH"
  WAS_VAR_DB2_TOOLBOX = "OS400_TOOLBOX_JDBC_DRIVER_PATH"

  DBNAME_NATIVE  = "*LOCAL"
  DBNAME_TOOLBOX = "*SYSBAS"

  def __init__(self):

    DB2.__init__(self)

    self.setDbmsType(self.DBMS_TYPE_DB2ISERIES)
    self.ObserverTableSpaces=[]  # no tbs on iSeries
    self.connectionPort=446

    self.jarTableName="QSYS2.SYSJAROBJECTS"
    self.jarAlteredField="LAST_ALTERED"
    self.jarCreatedField="JAR_CREATED"
    self.useDefaultTablespace=1   # no tablespaces on i5

    self.runningInQshell = (javaEnv["os.name"].find("OS/400") > -1)

    self.observerSchemaSelect="SELECT TABLE_SCHEMA AS OBSSCHEMA FROM QSYS2.SYSTABLES WHERE TABLE_NAME = 'OBSERVER_VERSION' AND TABLE_TYPE = 'T'"

    if self.runningInQshell:
      self.jdbcDriverJars       = DB2iSeries.JARFILES_NATIVE
      self.wasVarJdbcDriverPath = DB2iSeries.WAS_VAR_DB2_NATIVE
      self.dataSourceImpl       = DB2iSeries.DATASOURCE_NATIVE
      self.databaseName         = DB2iSeries.DBNAME_NATIVE
    else:
      self.jdbcDriverJars       = DB2iSeries.JARFILES_TOOLBOX
      self.wasVarJdbcDriverPath = DB2iSeries.WAS_VAR_DB2_TOOLBOX
      self.dataSourceImpl       = DB2iSeries.DATASOURCE_TOOLBOX
      self.databaseName         = DB2iSeries.DBNAME_TOOLBOX

  #----------------------------------------------------------------------------
  # connect()
  #
  # make a singelton connection to the DB
  #----------------------------------------------------------------------------
  def connect(self):

    rc=0

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."

    try:


      if self.connection==None:
        databaseName=self.databaseName
        self.dataSource.setDatabaseName(databaseName)
        if self.jdbcDriverType==4:
          self.dataSource.setServerName(self.connectionHost)

          # port can not be coosen for iSeries
          # port=int(self.connectionPort)
          # self.dataSource.setPortNumber( port )

        self.connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)

        logOnly( mcat.get("ConnectedTo", databaseName) )


    except java.sql.SQLException, xcp :
      rc=xcp
      raise

    return rc

  #----------------------------------------------------------------------------
  #
  # promptDatabasePort
  #
  # must be 446, so do not prompt
  #
  #----------------------------------------------------------------------------
  def promptDatabasePort(self, promptTitle="PromptListenerPort"):

    return self.connectionPort

  #----------------------------------------------------------------------------
  #
  # resolveJdbcHome
  #
  #----------------------------------------------------------------------------
  def resolveJdbcHome(self, serverName=""):

    jdbcHome=DatabaseManager.resolveJdbcHome(self, serverName)

    if jdbcHome=="" and self.runningInQshell:
      if self.dataSourceImpl==DB2iSeries.DATASOURCE_NATIVE:
        jdbcHome="/QIBM/ProdData/Java400/ext"
      elif self.dataSourceImpl==DB2iSeries.DATASOURCE_TOOLBOX:
        jdbcHome="/QIBM/ProdData/HTTP/Public/jt400/lib"

      # problems with universalDriver, do not use it
      # elif self.dataSourceImpl==DB2.DATASOURCE_UNIVERSAL:
      #  jdbcHome=os.path.join(javaEnv["was.install.root"], "universalDriver_wbi/lib")

    return jdbcHome


  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # install a jar file containing UDFs etc
  # requires to be connected
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jarId, replaceExisting=1, refreshClasses=0):

    stmt=None
    try:

      #to ensure we get the correct match in the syscat, we need to do to_upper
      jarId = jarId.upper()

      #check if we must replace it
      replace=0
      if self.checkforJarInstalledInDb(jarId):
        if replaceExisting:
          logOnly(mcat.get("JarFileUpdate",jarId))
          replace=1
        else:
          logOnly(mcat.get("NoJarFileUpdate",jarId))

      # For DB2, we need to have the File as a BLOB
      # because it seems that sqlj.install_jar() is not avail over JDBC
      log(mcat.get("JarFileInstalling",jarFileName))
      logOnly(mcat.get("JarFileId",jarId))
      jar=java.io.File(jarFileName)
      jarFis=java.io.FileInputStream(jar)
      fileAsBlob=jarray.zeros(jar.length(),'b')
      # to create a BLOB, we need a java array
      jarFis.read(fileAsBlob)

      if replace==1:

        stmt = self.connection.prepareCall("CALL SQLJ.DB2_REPLACE_JAR(?,?)");
        stmt.setObject(1, fileAsBlob, java.sql.Types.BLOB);
        stmt.setString(2, jarId);
        stmt.executeUpdate();

      else:
        stmt = self.connection.prepareCall("CALL SQLJ.DB2_INSTALL_JAR(?,?,?)");
        stmt.setObject(1, fileAsBlob, java.sql.Types.BLOB);
        stmt.setString(2, jarId);
        stmt.setInt(3,0);    # this must always be null
        stmt.executeUpdate();

      self.connection.commit()

      if stmt!=None:
        stmt.close()

    except :
      if stmt!=None:
        self.connection.rollback()
        stmt.close()
      raise


  #----------------------------------------------------------------------------
  #
  # promptJdbcDriverImpl
  #
  # set the internally used class of the datasource
  #
  #----------------------------------------------------------------------------
  def promptJdbcDriverImpl(self, promptTitle="PromptJdbcDriverImpl"):

    ## generate menu

    menuList =[]
    validAnswers =[]
    itemCount=0
    default=None
    nativeDriver=-1
    toolboxDriver=-2
    universalDriver=-3

    if self.runningInQshell:

      # toolbox
      toolboxDriver=itemCount
      menuList.append( mcat.get("Menu_JdbcToolboxDriver") )       # com.ibm.as400.access.AS400JDBCDataSource
      validAnswers.append(str(toolboxDriver+1))
      if self.dataSourceImpl==DB2iSeries.DATASOURCE_TOOLBOX:
        default=itemCount
      itemCount+=1

      # Native
      nativeDriver=itemCount
      menuList.append( mcat.get("Menu_JdbcNativeDriver") + "\n" ) # com.ibm.db2.jdbc.app.UDBDataSource
      validAnswers.append(str(nativeDriver+1))
      if self.dataSourceImpl==DB2iSeries.DATASOURCE_NATIVE:
        default=itemCount
      itemCount+=1

      # Universal
      #
      # currently not supported from CDB
      #
      # universalDriver=itemCount
      #     menuList.append( mcat.get("Menu_JdbcUniversalDriver")+"\n") # com.ibm.db2.jcc.DB2SimpleDataSource
      #     validAnswers.append(str(universalDriver+1))
      #     if self.dataSourceImpl==DB2.DATASOURCE_UNIVERSAL:
      #       default=itemCount


      print mcat.get("NewLine")

      # show menu
      inp= console.selectFromList( mcat.get(promptTitle)+"\n", validAnswers, default , menuList )

    else:
      inp=toolboxDriver

    if inp == nativeDriver:
      self.jdbcDriverJars       = DB2iSeries.JARFILES_NATIVE
      self.wasVarJdbcDriverPath = DB2iSeries.WAS_VAR_DB2_NATIVE
      self.dataSourceImpl       = DB2iSeries.DATASOURCE_NATIVE
      self.databaseName         = DB2iSeries.DBNAME_NATIVE
    elif inp == universalDriver:
      self.jdbcDriverJars       = DB2.JARFILES_UNIVERSAL
      self.wasVarJdbcDriverPath = DB2.WAS_VAR_DB2_UNIVERSAL
      self.dataSourceImpl       = DB2.DATASOURCE_UNIVERSAL
      self.databaseName         = DB2iSeries.DBNAME_TOOLBOX
    elif inp == toolboxDriver:
      self.jdbcDriverJars       = DB2iSeries.JARFILES_TOOLBOX
      self.wasVarJdbcDriverPath = DB2iSeries.WAS_VAR_DB2_TOOLBOX
      self.dataSourceImpl       = DB2iSeries.DATASOURCE_TOOLBOX
      self.databaseName         = DB2iSeries.DBNAME_TOOLBOX

    return self.dataSourceImpl

  #----------------------------------------------------------------------------
  #
  # promptJdbcConnectionType
  #
  # set the internally used type of the datasource
  #
  #----------------------------------------------------------------------------
  def promptJdbcConnectionType(self, promptTitle="PromptJdbcConnectionType"):

    ## generate menu

    menuList =[]
    validAnswers =[]
    itemCount=0

    if self.dataSourceImpl==DB2iSeries.DATASOURCE_TOOLBOX:
      self.jdbcDriverType=4
    elif self.dataSourceImpl==DB2iSeries.DATASOURCE_NATIVE:
      self.jdbcDriverType=2
    if self.dataSourceImpl==DB2.DATASOURCE_UNIVERSAL:

      # Type2
      JDBCType2=itemCount
      menuList.append( mcat.get("Menu_JDBCType2") )
      validAnswers.append('2')
      if self.jdbcDriverType==2:
        default=itemCount
      itemCount+=1

      # Type4
      JDBCType4=itemCount
      menuList.append( mcat.get("Menu_JDBCType4")+"\n")
      validAnswers.append('4')
      if self.jdbcDriverType==4:
        default=itemCount
      itemCount+=1

      print mcat.get("NewLine")

      # show menu
      inp= console.selectFromList( mcat.get("PromptRetryConnect")+"\n", validAnswers, default , menuList )

      if inp == JDBCType2:
        self.jdbcDriverType=2

      elif inp == JDBCType4:
        self.jdbcDriverType=4

    return self.jdbcDriverType

  #----------------------------------------------------------------------------
  # databaseLogfileName()
  #
  # return the name of the DB logfile as defined by the Common DB LI
  #----------------------------------------------------------------------------
  def databaseLogfileName(self, isObserver=0):

    sep="_"
    fn  = self.getDbmsType()     + sep
    # ommit on iSeries : fn += self.getDatabaseName() + sep
    fn += self.getSchema()
    if isObserver:
      fn += sep + 'Observer'

    fn += ".log"

    fn = os.path.join(AppServer.logLocation, fn)

    return fn

  #----------------------------------------------------------------------------
  # removeJarFile()
  #
  # remove a jar file installed in the db
  # _jarPath is not used for DB2
  #----------------------------------------------------------------------------
  def removeJarFile(self, _jarId, _jarPath=""):

    stmt=None

    try:
      jarId = self.getSchema() + '.' + _jarId.upper()
      stmt = self.connection.prepareCall("CALL SQLJ.REMOVE_JAR(?,?)")
      stmt.setString(1, jarId);
      stmt.setInt(2,0);    # this must always be null
      stmt.executeUpdate();
      self.connection.commit()
      stmt.close()

    except :
      if stmt!=None:
        self.connection.rollback()
        stmt.close()
      raise

  #----------------------------------------------------------------------------
  # checkForCollection()
  #
  # check if the schema exists in the database
  # return 1 if it exists
  #----------------------------------------------------------------------------
  def checkForCollection(self, _schema=""):

    stmt=None
    rc=0
    if _schema=="":
      schema=self.getSchema()
    else:
      schema=_schema

    cmd = "SELECT TABLE_SCHEM FROM SYSIBM.SQLSCHEMAS WHERE TABLE_SCHEM in ('" + schema.upper() + "')"

    try:

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)

      column=rs.next()

      if column!=0:
        rc=1

    except :

      handleException()

    if stmt!=None:
      stmt.close()

    return rc

#----------------------------------------------------
# class for Oracle databases
#----------------------------------------------------
class Oracle(DatabaseManager):

  def __init__(self):

    DatabaseManager.__init__(self)

    self.connectionHost="localhost"
    self.connectionPort=1521

    self.setDbmsType(self.DBMS_TYPE_ORACLE)
    self.setConnectionPrefix("jdbc:oracle:thin:")
    self.dataSourceImpl="oracle.jdbc.pool.OracleDataSource"

    self.setDatabaseName("BPEDB")
    self.setDatabaseUser("system")
    self.setDatabasePassword("")
    self.connectionPort = 1521
    self.jdbcDriverType=4

    self.ObserverTableSpaces=self.retrieveTablespacesFromFile()
    self.tableSpaceClause=r"\bTABLESPACE (\w+)\b"  # equlas " TABLESPACE <TBSNAME>", where TBSNAME is put into mathcer.group(1)

    self.jdbcDriverJars=[  "ojdbc6.jar"  ]
    self.wasVarJdbcDriverPath="ORACLE_JDBC_DRIVER_PATH"

    self.observerSchemaSelect="SELECT OWNER AS OBSSCHEMA FROM ALL_TABLES WHERE TABLE_NAME = 'OBSERVER_VERSION'"


  #----------------------------------------------------------------------------
  #
  # promptJdbcConnectionType
  #
  #----------------------------------------------------------------------------
  def promptJdbcConnectionType(self, promptTitle="PromptJdbcConnectionType"):

    # currently only type 4 = connect over host/port supported
    self.jdbcDriverType=4
    return self.jdbcDriverType


  #----------------------------------------------------------------------------
  #
  # resolveJdbcHome
  #
  #----------------------------------------------------------------------------
  def resolveJdbcHome(self, serverName=""):

    jdbcHome=""

    if osEnv["ORACLE_HOME"]!=None:
      # try the environment:
      dbHome=osEnv["ORACLE_HOME"]
      dbHome=dbHome.strip()
      if dbHome!="":
         jdbcHome=os.path.join(dbHome,"jdbc/lib")

    if jdbcHome=="":
      jdbcHome=DatabaseManager.resolveJdbcHome(self, serverName)

    return jdbcHome


  #----------------------------------------------------------------------------
  # connect(), overwrite from DatabaseManager
  #----------------------------------------------------------------------------
  def connect(self):

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."

    #set the url to contain the
    self.dataSource.setURL(self.connectionPrefix + "@//" + self.connectionHost + ":" +
      str(self.connectionPort) + "/" + self.getDatabaseName())
    #call the super method
    return DatabaseManager.connect(self)


  #----------------------------------------------------------------------------
  # checkforObserverTablespace()
  #
  # check for tablespace prereqs for an evnet collector db setup
  #----------------------------------------------------------------------------
  def checkforObserverTablespace(self):

    tbsAsString=str(self.ObserverTableSpaces)[1:-1]
    cmd="SELECT TABLESPACE_NAME FROM USER_TABLESPACES WHERE TABLESPACE_NAME in ("+ tbsAsString + ")"
    return self.checkforTablespaces( self.ObserverTableSpaces , cmd)


  #----------------------------------------------------------------------------
  # listJars()
  #
  # select OWNER, dbms_java.longname(object_name) as name from dba_objects
  #    where object_type = 'JAVA CLASS' and object_name like '%TimestampUtil%';
  # jarId not used here
  #----------------------------------------------------------------------------
  def listJars(self,jarId):

    rc=0
    stmt=None
    try:

      selectClause = "SELECT OWNER, STATUS, TIMESTAMP, LAST_DDL_TIME AS LAST_UPDTAE, DBMS_JAVA.LONGNAME(OBJECT_NAME) AS CLASSNAME "
      fromClause   = "FROM DBA_OBJECTS "
      whereClause  = "WHERE OBJECT_TYPE = 'JAVA CLASS' AND DBMS_JAVA.LONGNAME(OBJECT_NAME) like 'com/ibm/bpe/observer/%'"
      cmd = selectClause + fromClause + whereClause

      cmdOut=mcat.get("ExecutingSQLStmt") + "1" + ": " + cmd
      logOnly( cmdOut )

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)

      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        header="OWNER".ljust(10) + "STATUS".ljust(9) + "Last Update".ljust(21) + "Class".ljust(45)
        print header
        print "-".zfill(len(header)).replace("0","-")
      else:
        print mget("NoObserverClassesFound")
      while row==1:
        schema=rs.getString("OWNER")
        status=rs.getString("STATUS")
        className=rs.getString("CLASSNAME")
        lastUpdate=rs.getString("LAST_UPDTAE")
        created=rs.getString("TIMESTAMP")

        if lastUpdate==None:
          if created==None:
            lastUpdate=""
          else:
            lastUpdate=created

        print schema.ljust(9), status.ljust(8), lastUpdate[0:19].ljust(20), className.ljust(45)
        row=rs.next()

      if stmt!=None:
        stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise
  #----------------------------------------------------------------------------
  # resolveLoadJavaUtility()
  #
  # for loading the classes into the file, we require the loadjava tool
  # Searchi if using PATH env var or som other heuristics
  #----------------------------------------------------------------------------
  def resolveLoadJavaUtility(self, toolName = "loadjava"):

    toolPath = ""
    jdbcPath="jdbc" + os.sep + "lib"
    dbHome=""

    osName=javaEnv["os.name"].upper()
    if osName.find("WIN") > -1:
      toolName += ".bat"

    while 1:

      # try to resolve oracle home from the jar path
      # since we are connected, jdbcHome should be set
      if self.jdbcHome!="":

         # check if copied into jdbc sub dir
         testPath=os.path.join(self.jdbcHome,toolName)
         if os.path.exists(testPath):
            toolPath=testPath
            break # we found one

         # check if this is a regular oracle install
         if self.jdbcHome.find(jdbcPath) > -1:
           # it is, split off the jdbcPath
           dbHome=self.jdbcHome.split(jdbcPath)[0]
           binPath=os.path.join(dbHome,"bin")
           testPath=os.path.join(binPath,toolName)
           if os.path.exists(testPath):
              toolPath=testPath
              break  # we found one

      # try env_var ORACLE_HOME
      dbHome=osEnv["ORACLE_HOME"]
      if dbHome!=None:
        dbHome=dbHome.strip()
        if dbHome!="":
          testPath=dbHome + os.sep + "bin" + os.sep + toolName
          if os.path.exists(testPath):
            toolPath=testPath
            break


      # try PATH variable
      # for any reason the following doesn't work: osEnv["PATH"]?
      pathList=str(java.lang.System.getenv("PATH")).split(os.pathsep)
      for i in pathList:
        pth=i + os.sep + toolName
        if os.path.exists(pth):
          toolPath=pth
          if pth.find(jdbcPath) > -1:
            # split off the jdbcPath
            dbHome=pth.split(jdbcPath)[0]
          break  # for loop

      # nothing found
      break #

    return toolPath, dbHome

  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # install a jar file containing UDFs etc
  # loadjava -user <user>/<password>@observer -resolve B:w\p\lib\bpcodbutil.jar
  #
  # currently, we expect the loadjava tool to be in the PATH, so
  # the JVM can find it.
  # Note: jar_id is not used here, it is only for compatibility with DB2
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jar_id, replaceExisting=1, refreshClasses=0):

    osName=javaEnv["os.name"].upper()
    rc=1
    done=0
    url=self.connectionHost + ":" + str(self.connectionPort) + ":" + self.getDatabaseName()
    toolName="loadjava"
    loadJava, dbHome = self.resolveLoadJavaUtility(toolName)
    cmd = loadJava + " -user " + self.databaseUser + "/" + self.databasePassword
    cmd += "@" + url + " -resolve " + os.path.normpath(jarFileName)
    cmd += " -thin"
    cmd += " -schema " + self.getSchema()

    if loadJava=="":   # no loadjava found on machine
      cmdOut=toolName + cmd
      cmdOut=cmdOut.replace("/" + self.databasePassword +"@", "/" + mget("<Password>") + "@")
      cmdOut=cmdOut.replace(os.path.normpath(jarFileName), mget("<PathOfJarFile>"))
      msg = mcat.get("ErrorAppDoesNotExist", toolName + "[.bat]") + "\n"
      msg += mcat.get("InfoInstallJarOnServer", os.path.normpath(jarFileName)) + "\n"
      msg += "'" + cmdOut + "'"

      raise IOError, msg

    logOnly(cmd.replace("/" + self.databasePassword +"@", '/*********@'))
    log(mcat.get("JarFileInstalling",jarFileName))

    # start a new process to execute loadjava
    rt=java.lang.Runtime.getRuntime()
    env=None
    if dbHome!="":
      home = java.util.ArrayList()
      home.add("ORACLE_HOME=" + dbHome)
      env=jarray.array(home, java.lang.String)

    prc=rt.exec(cmd,env)

    # get the output into out logfile
    cerr=java.io.BufferedReader(java.io.InputStreamReader(prc.getErrorStream()))
    cout=java.io.BufferedReader(java.io.InputStreamReader(prc.getInputStream()))

    while done==0:

      while cerr.ready() or cout.ready():
        if cerr.ready():
          logOnly( cerr.readLine() )
        if cout.ready():
          logOnly( cout.readLine() )

      #check if the process has ended:
      try:
        rc=prc.exitValue()
        done=1
      except java.lang.IllegalThreadStateException:
        #process still running...
        # wait 5 seconds, then retry...
        time.sleep(5)

    # the invoked process may have ended before all buffered output has been read - read rest here
    while cerr.ready() or cout.ready():
      if cerr.ready():
        logOnly( cerr.readLine() )
      if cout.ready():
        logOnly( cout.readLine() )

    return rc

  #----------------------------------------------------------------------------
  # removeJarFile()
  #
  # remove a jar file containing UDFs etc
  # dropjava -user <user>/<password>@observer B:w\p\lib\bpcodbutil.jar
  #
  # currently, we expect the dropjava tool to be in the PATH, so
  # the JVM can find it.
  # Note: jar_id is not used here, it is only for compatibility with DB2
  #----------------------------------------------------------------------------
  def removeJarFile(self, _jarId, _jarPath=""):

    osName=javaEnv["os.name"].upper()
    rc=1
    done=0
    jarFileName=_jarPath
    url=self.connectionHost + ":" + str(self.connectionPort) + ":" + self.getDatabaseName()

    toolName="dropjava"
    dropJava, dbHome = self.resolveLoadJavaUtility(toolName)

    cmd = dropJava + " -user " + self.databaseUser + "/" + self.databasePassword
    cmd += "@" + url + " " + os.path.normpath(_jarPath)
    cmd += " -thin"
    cmd += " -schema " + self.getSchema()

    if dropJava=="":   # no loadjava found on machine
      cmdOut=toolName + cmd
      cmdOut=cmdOut.replace("/" + self.databasePassword +"@", "/" + mget("<Password>") + "@")
      cmdOut=cmdOut.replace(os.path.normpath(jarFileName), mget("<PathOfJarFile>"))
      msg = mcat.get("ErrorAppDoesNotExist", toolName + "[.bat]") + "\n"
      msg += mcat.get("InfoInstallJarOnServer", os.path.normpath(jarFileName)) + "\n"
      msg += "'" + cmdOut + "'"

      raise IOError, msg


    logOnly(cmd.replace("/" + self.databasePassword +"@", '/*********@'))

    # start a new process to execute loadjava
    rt=java.lang.Runtime.getRuntime()
    env=None
    if dbHome!="":
      home = java.util.ArrayList()
      home.add("ORACLE_HOME=" + dbHome)
      env=jarray.array(home, java.lang.String)

    prc=rt.exec(cmd,env)

    # get the output into out logfile
    cerr=java.io.BufferedReader(java.io.InputStreamReader(prc.getErrorStream()))
    cout=java.io.BufferedReader(java.io.InputStreamReader(prc.getInputStream()))

    while done==0:

      while cerr.ready() or cout.ready():
        if cerr.ready():
          logOnly( cerr.readLine() )
        if cout.ready():
          logOnly( cout.readLine() )

      #check if the process has ended:
      try:
        rc=prc.exitValue()
        done=1
      except java.lang.IllegalThreadStateException:
        #process still running...
        # wait 5 seconds, then retry...
        time.sleep(5)

    # the invoked process may have ended before all buffered output has been read - read rest here
    while cerr.ready() or cout.ready():
      if cerr.ready():
        logOnly( cerr.readLine() )
      if cout.ready():
        logOnly( cout.readLine() )

    return rc


  #----------------------------------------------------------------------------
  # isJavaOrSqlUdfActive()
  #
  #----------------------------------------------------------------------------
  def isUdfWorking(self):

    rc=mcat.get("undefined")
    stmt=None
    try:

      cmd = "ALTER SESSION SET NLS_TIMESTAMP_FORMAT = 'yyyy-mm-dd hh24:mi:ss.ff'"
      logOnly(mcat.get("ExecutingSQLStmt") + "1: " + cmd)
      stmt = self.connection.createStatement()
      rs=stmt.executeUpdate(cmd)
      logOnly( mcat.get("ResultCode") + str(rs) )
      rc=DatabaseManager.isUdfWorking(self)
      stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise

    return rc

#----------------------------------------------------
# class for Derby databases
#----------------------------------------------------
class Derby(DatabaseManager):

  DATASOURCE_EMBEDDED   = "org.apache.derby.jdbc.EmbeddedDataSource"   # local connections only, in derby.jar
  DATASOURCE_NETWORK    = "org.apache.derby.jdbc.ClientDataSource"     # remote connections, in derbyclient.jar

  JARFILES_EMBEDDED   = [ "derby.jar" ]
  JARFILES_NETWORK    = [ "derbyclient.jar" ]


  def __init__(self):

    DatabaseManager.__init__(self)

    self.setDbmsType(self.DBMS_TYPE_DERBY)
    self.setConnectionPrefix("jdbc:db2j:")
    self.dataSourceImpl=Derby.DATASOURCE_EMBEDDED
    self.setDatabaseName(self.DATABASE_CS_BPEDB_PATH)
    self.setSchema("APP")
    self.setDatabaseUser("")
    self.setDatabasePassword("")

    self.jdbcDriverType=2
    self.connectionPort=1527

    self.jdbcDriverJars=Derby.JARFILES_EMBEDDED
    self.wasVarJdbcDriverPath="JDBC_DRIVER_PATH"

    self.jarTableName="SYS.SYSFILES"

    self.useDefaultTablespace=1   # no tablespaces on Derby

    self.observerSchemaSelect="SELECT SCHEMANAME AS OBSSCHEMA FROM SYS.SYSTABLES ST, SYS.SYSSCHEMAS SS WHERE TABLENAME = 'OBSERVER_VERSION' AND TABLETYPE = 'T' AND SS.SCHEMAID=ST.SCHEMAID"


  #----------------------------------------------------------------------------
  # shutDownDatabase
  #
  # shut down / disconnect the process from wsadmin
  # quite funny: to shutdown a Cloudscape db you must
  # a) set the property Shutdown on the dataSource
  # b) get a new connection
  # c) if success, you get a java.sql.SQLException
  #----------------------------------------------------------------------------
  def shutDownDatabase(self):

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."

    try:
      if self.dataSource!=None:
        self.dataSource.setShutdownDatabase("shutdown")
        connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)
    except :
      pass

  #----------------------------------------------------------------------------
  # disconnect()
  #
  # overwrite of DatabaseManager
  #----------------------------------------------------------------------------
  def disconnect(self):
     if self.connection != None :
       self.connection.commit()
       self.connection.close()
       del self.connection
       self.connection=None
       logOnly( mcat.get("DisconnectedFrom", self.databaseName) )

  #----------------------------------------------------------------------------
  # disconnectUncommited()
  #
  # overwrite of DatabaseManager
  #----------------------------------------------------------------------------
  def disconnectUncommited(self):
      if self.connection != None :
        self.connection.close()
        del self.connection
        self.connection=None
        logOnly( mcat.get("DisconnectedFrom", self.databaseName) )


  #----------------------------------------------------------------------------
  #
  # promptCredentials
  #
  # prompt uid and pwd for a given user of type bpelib.UserId(),
  # but only if in network: (currently for JDBC type 4 only)
  #----------------------------------------------------------------------------
  def promptCredentials(self, promptTitle="PromptDbUser"):

    user=None

    if self.jdbcDriverType==4:
      DatabaseManager.promptCredentials(self,promptTitle)
    return user

  #----------------------------------------------------------------------------
  #
  # promptDatabaseHostName
  #
  # prompt promptDatabaseHostName,
  # but only if in network: (which is currently not supported)
  #----------------------------------------------------------------------------
  def promptDatabaseHostName(self, promptTitle="PromptServerHost"):


    if self.jdbcDriverType==4:
      ########################
      # prompt for the user
      DatabaseManager.promptDatabaseHostName(self,promptTitle)

    return self.connectionHost


  #----------------------------------------------------------------------------
  #
  # promptDatabasePort
  #
  # prompt promptDatabasePort,
  # but only if in network: (which is currently not supported)
  #----------------------------------------------------------------------------
  def promptDatabasePort(self, promptTitle="PromptListenerPort"):


    if self.jdbcDriverType==4:
      ########################
      # prompt for the user
      DatabaseManager.promptDatabasePort(self,promptTitle)

    return self.connectionPort


  #----------------------------------------------------------------------------
  # databaseLogfileName()
  #
  # return the name of the DB logfile as defined by the Common DB LI
  #----------------------------------------------------------------------------
  def databaseLogfileName(self, isObserver=0):

    sep="_"

    fn  = self.getDbmsType()     + sep
    fn += os.path.basename(self.getDatabaseName())

    if self.getSchema()!="APP":
      fn +=  sep + self.getSchema()

    if isObserver:
      fn += sep + 'Observer'

    fn += ".log"

    fn = os.path.join(AppServer.logLocation, fn)

    return fn



  #----------------------------------------------------------------------------
  # listJars()
  #
  #----------------------------------------------------------------------------
  def listJars(self,jarId):

    rc=0
    stmt=None
    schema = self.getSchema()
    jarId  = jarId.upper()

    try:

      # check if we have a fully qualified jarId like DB2ADMIN.bpcodbutil
      if jarId.find(".") >-1 :
        parts=jarId.split(".",1)
        jarId  = parts[1]
        schema = parts[0]

      cmd = "SELECT FILENAME AS JAR_ID, SCHEMANAME AS JARSCHEMA"
      cmd +=" FROM " + self.jarTableName + ", SYS.SYSSCHEMAS"
      cmd +=" WHERE " + self.jarTableName + ".SCHEMAID=SYS.SYSSCHEMAS.SCHEMAID AND FILENAME='" + jarId + "'"

      cmdOut=mcat.get("ExecutingSQLStmt") + "1" + ": " + cmd
      logOnly( cmdOut )

      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)

      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        print "JARSCHEMA".ljust(10), "JAR_ID".ljust(12) #, "Last Update (local time)".ljust(12)
        print "----------------------------------------"
      else:
        print mget("NoJarFileFound")
      while row==1:
        schema=rs.getString("JARSCHEMA")
        jid=rs.getString("JAR_ID")
        #lastUpdate=rs.getString(self.jarAlteredField)
        #created=rs.getString(self.jarCreatedField)

        #if lastUpdate==None:
        #  if created==None:
        #    lastUpdate=""
        #  else:
        #    lastUpdate=created

        #if lastUpdate!=None and lastUpdate!="":
        #  lastUpdate=lastUpdate+"0000000"  # if there are zeros at the end, they are skipped, see 452197

        print schema.ljust(10), jid.ljust(12) # , lastUpdate[0:26]
        row=rs.next()

      if stmt!=None:
        stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise


  #----------------------------------------------------------------------------
  # checkforJarInstalledInDb()
  #
  # check if the jar with the JAR_ID is installed in the database
  # Please note that we do a fully qualified query using the schema
  # return 1 if installed, 0 otherwise
  #----------------------------------------------------------------------------
  def checkforJarInstalledInDb(self, jarId):

    rc=0
    stmt=None
    schema = self.getSchema()
    jarId  = jarId.upper()

    try:

      # check if we have a fully qualified jarId like DB2ADMIN.bpcodbutil
      if jarId.find(".") >-1 :
        parts=jarId.split(".",1)
        schema = parts[0]
        jarId  = parts[1]

      cmd = "SELECT FILENAME, SCHEMANAME"
      cmd +=" FROM " + self.jarTableName + ", SYS.SYSSCHEMAS"
      cmd +=" WHERE " + self.jarTableName + ".SCHEMAID=SYS.SYSSCHEMAS.SCHEMAID AND FILENAME='" + jarId + "'"
      cmd += "  AND SCHEMANAME IN ('" + schema + "')"

      cmdOut=mcat.get("ExecutingSQLStmt") + "1" + ": " + cmd
      logOnly( cmdOut )
      stmt = self.connection.createStatement()
      rs=stmt.executeQuery(cmd)
      row=rs.next()   # returns true if there are (more) columns
      if row==1:
        schema=rs.getString("SCHEMANAME")
        jid=rs.getString("FILENAME")
        logOnly(mcat.get("JarFileAllreadyInstalled",schema.strip() + '.' + jid, self.databaseName))
        rc=1

    except java.sql.SQLException, xcp:
      self.handleSqlException(xcp)

    except :
      handleException()

    if stmt!=None:
      stmt.close()
    return rc

  #----------------------------------------------------------------------------
  # getDatabaseProperty()
  #
  # get a Derby db property
  #----------------------------------------------------------------------------
  def getDatabaseProperty(self, property ):

    value=""
    cmd = "VALUES SYSCS_UTIL.SYSCS_GET_DATABASE_PROPERTY('" + property + "')"
    cmdOut=mcat.get("ExecutingSQLStmt") + ": " + cmd
    logOnly( cmdOut )
    stmt = self.connection.createStatement()
    rs=stmt.executeQuery(cmd)
    row=rs.next()   # returns true if there are (more) columns
    if row==1:
      value=rs.getString("1")

    if value==None:
      value=""

    return value

  #----------------------------------------------------------------------------
  # setDatabaseProperty()
  #
  # set a Derby db property
  #----------------------------------------------------------------------------
  def setDatabaseProperty(self, propertyName, value ):

    cmd = "CALL SYSCS_UTIL.SYSCS_SET_DATABASE_PROPERTY(?,?)"
    stmt = self.connection.prepareCall(cmd)
    stmt.setString(1, propertyName)
    stmt.setString(2, value);

    cmdOut=mcat.get("ExecutingSQLStmt") + ": " + cmd + "params: (" + propertyName + "," + value +")"
    logOnly( cmdOut )

    stmt.executeUpdate();


  #----------------------------------------------------------------------------
  # installJarFile()
  #
  # install a jar file containing UDFs etc
  # requires to be connected
  #----------------------------------------------------------------------------
  def installJarFile(self, jarFileName, jarId, replaceExisting=1, refreshClasses=1):

    stmt=None

    try:

      #to ensure we get the correct match in the syscat, we need to do to_upper
      jarId = jarId.upper()

      #check if we must replace it
      replace=0
      if self.checkforJarInstalledInDb(jarId):
        if replaceExisting:
          log(mcat.get("JarFileUpdate",jarId))
          replace=1
        else:
          log(mcat.get("NoJarFileUpdate",jarId))

      if replace==1:

        stmt = self.connection.prepareCall("CALL SQLJ.REPLACE_JAR(?,?)")
        stmt.setString(1, jarFileName)
        stmt.setString(2, jarId)
        stmt.executeUpdate()

      else:
        stmt = self.connection.prepareCall("CALL SQLJ.INSTALL_JAR(?,?,?)")
        stmt.setString(1, jarFileName)
        stmt.setString(2, jarId)
        stmt.setInt(3,0)    # this must always be null
        stmt.executeUpdate()


      # update classpath
      derbyClasspath=self.getDatabaseProperty("derby.database.classpath")
      if derbyClasspath.find(jarId)==-1:
        if derbyClasspath=="":
          self.setDatabaseProperty("derby.database.classpath", jarId )
        else:
          self.setDatabaseProperty("derby.database.classpath", jarId + ":" + derbyClasspath)

      try:
        rc=self.isJavaOrSqlUdfActive()
        # just try if we can invoke
        # only required on derby network on new installs
      except:
        log(mcat.get("RestartDbToActivateJar", self.databaseName))

      #close stmt
      if stmt!=None:
        stmt.close()

      self.connection.commit()

    except:
      #close stmt
      if stmt!=None:
        self.connection.rollback()
        stmt.close()
      raise

  #----------------------------------------------------------------------------
  # removeJarFile()
  #
  # remove a jar file installed in the db
  # _jarPath is not used for DB2
  #----------------------------------------------------------------------------
  def removeJarFile(self, _jarId, _jarPath=""):

    stmt=None

    try:

      derbyClasspath=self.getDatabaseProperty("derby.database.classpath")
      derbyClasspath=derbyClasspath.replace(jarId+":",'')  # for the case there are other entries
      derbyClasspath=derbyClasspath.replace(jarId,'')      # for the case this was the one and only entry
      self.setDatabaseProperty("derby.database.classpath", derbyClasspath)

      jarId = self.getSchema() + '.' + _jarId.upper()
      stmt = self.connection.prepareCall("CALL SQLJ.REMOVE_JAR(?,?)");
      stmt.setString(1, jarId);
      stmt.setInt(2,0);    # this must always be null
      stmt.executeUpdate();
      stmt.close()

    except :
      if stmt!=None:
        stmt.close()
      raise

  #----------------------------------------------------------------------------
  # connect()
  #
  # make a singelton connection to the DB
  #----------------------------------------------------------------------------
  def connect(self):

    rc=0

    if not self.attachedToJdbc:
      raise AssertionError, "DB class is not attached to JDBC driver. Call attachedToJdbc first."


    try:

      if self.connection==None:

        databaseName=self.databaseName
        self.dataSource.setDatabaseName(databaseName)

        if self.jdbcDriverType==4:
          self.dataSource.setServerName(self.connectionHost)
          self.dataSource.setPortNumber(self.connectionPort)

        if self.databaseUser==None or self.databaseUser=="":
          self.databaseUser="dummy"
          self.databasePassword="dummy"

        self.connection = self.dataSource.getConnection(self.databaseUser, self.databasePassword)

        logOnly( mcat.get("ConnectedTo", databaseName) )


    except java.sql.SQLException, xcp :
      rc=xcp
      raise

    return rc


  #----------------------------------------------------------------------------
  #
  # resolveJdbcHome
  #
  #----------------------------------------------------------------------------
  def resolveJdbcHome(self, serverName=""):

    # try the environment:
    jdbcHome=os.path.join(javaEnv["was.install.root"], "derby/lib")

    if jdbcHome=="":
      jdbcHome=DatabaseManager.resolveJdbcHome(self, serverName)

    return jdbcHome


  #----------------------------------------------------------------------------
  #
  # promptJdbcConnectionType
  #
  # set the internally used type of the datasource
  #
  #----------------------------------------------------------------------------
  def promptJdbcConnectionType(self, promptTitle="PromptJdbcConnectionType"):

    ## generate menu

    menuList =[]
    validAnswers =[]
    itemCount=0

    # Type2
    JDBCType2=itemCount
    menuList.append( mcat.get("Menu_DerbyEmbedded") )
    validAnswers.append('1')
    default=itemCount
    itemCount+=1

    # Type4
    JDBCType4=itemCount
    menuList.append( mcat.get("Menu_DerbyNetwork")+"\n")
    validAnswers.append('2')
    itemCount+=1

    # prompt for database manager
    print mcat.get("NewLine")

    # show menu
    inp= console.selectFromList( mcat.get(promptTitle)+"\n", validAnswers, JDBCType2 , menuList )

    if inp == JDBCType2:
      self.jdbcDriverType=2
      self.dataSourceImpl=Derby.DATASOURCE_EMBEDDED
      self.jdbcDriverJars= Derby.JARFILES_EMBEDDED

    elif inp == JDBCType4:
      self.jdbcDriverType=4
      self.dataSourceImpl=Derby.DATASOURCE_NETWORK
      self.jdbcDriverJars= Derby.JARFILES_NETWORK

    return self.jdbcDriverType


################################################################################
#
# Object type Classes
#
# class to represent the jmx scripting types
#
################################################################################
class Attribute:

  def __init__(self,key="",value=""):
    self.key=key
    self.value=value

  def asList(self):
    return [ self.key, self.value]


  def asParam(self):

    p=" -" + self.key + " " + self.value + " "
    return p




class Description:

  def __init__(self,lang="",value=""):
    self.lang=Attribute("lang", lang)
    self.value=Attribute("value", value)

  def asList(self):

    l = list()
    l.append( self.lang.asList() )
    l.append( self.value.asList())
    return l



class ConfigProperty:

  def __init__ ( self,name="", value="", description="", descriptions="",
                 type="java.lang.String", required= "false"):
    self.name         = Attribute('name',         name)           # String
    self.description  = Attribute('description',  description)    # String
    self.descriptions = Attribute('descriptions', descriptions)   # String
    self.type         = Attribute('type',         type)           # String
    self.value        = Attribute('value',        value)          # String
    self.required     = Attribute('required',     required)       #boolean?


  def asList(self):
    return [ self.name, self.description, self.descriptions, self.type, self.value ]



class J2EEResourceProperty:

  def __init__ (self, name="", description="", type="java.lang.String", value="", required="false"):

    # define our instance attributes
    self.name         = Attribute('name',        name)         # String
    self.description  = Attribute('description', description)  # String
    self.type         = Attribute('type',        type)         # String
    self.value        = Attribute('value',       value)        # String
    self.required     = Attribute('required',    required)     #boolean?

  def asList(self):
    return [ self.name.asList(), self.description.asList(), self.required.asList(), self.type.asList(), self.value.asList() ]




class PropertySet:

  def __init__(self, name=""):
    self.name=name
    self.members=list('')

  def add(self, newMember):
    self.members.append(newMember)


  def asList(self):

    result=list('')
    result.append( self.name )
    result.append( self.members )

    return result

class J2EEResourcePropertySet(PropertySet):
  #log( AdminConfig.attributes("J2EEResourcePropertySet") )
  #  resourceProperties J2EEResourceProperty*

  def __init__(self):

    PropertySet.__init__(self,name="resourceProperties")

  def add(self, newMember):
    self.members.append(newMember)


  def asList(self):

    result=list('')
    result.append( self.name )
    result.append( self.members )

    return result




################################################################################
#
# Class InstallationOptions
#
################################################################################
#
# This class is used to set/pass installation
# options when using the AdminApp.install() command
# since the options are passed as lists where the
# parameters are positional, this class allows to
# generate the options line, which quickly gets
# long and complex. If you need to add options, use
# the following:
# 1) install the application interactive by using
#    AdminApp.installInteractive("<earfile>")
# 2) change the values you want to set later
# 3) complete install
# 4) goto the wsadmin.traceout file in the directory
#    $(user.install.root)\logs
#    here look for WASX7278I, which is the generated
#    command line for the AdminApp.install() command
#----------------------------------------------------
class InstallationOptions:


   def __init__(self, optionName="", optionFields=""):

     self.optionName   = optionName
     self.optionFields = optionFields
                        # dictonary placeholder to store the option fields as
                        # key/value pairs for better handling. Order matters,
                        # because later only position is used

   ######################
   # getOptionCommandLine
   #
   #  generate a command line from option fields
   #  sample ' -MapResRefToEJB [[bpcobserverejb AuditDB bpcobserverejb.jar,META-INF/ejb-jar.xml jdbc/BPEDB javax.sql.DataSource jdbc/CEI null "" ]]'

   def getOptionCommandLine(self):

      cmdLine = ' -' + self.optionName + " [["
      for option in self.optionFields :
         cmdLine = cmdLine + option.value + " "
      cmdLine=cmdLine.rstrip() + "]]"
      return cmdLine


#------------------------------------------------------------
# option class for Mapping Resource References to EJB refs.
#------------------------------------------------------------
#
# Each element of the MapResRefToEJB option consists of the following fields:
#   module,
#   EJB,
#   uri,
#   referenceBinding,
#   resRef.type,
#   JNDI,
#   login.config.name,
#   auth.props.
# Of these fields, the following can be assigned values:
#   JNDI, login.config.name, auth.props. The JNDI field is required.

class MapResRefToEJB(InstallationOptions):

   def __init__( self, module, ejb, uri, referenceBinding, resourceType, jndiName, loginConfigurationName, properties):

      self.module=                 Attribute("module"            , module                 )
      self.ejb=                    Attribute("EJB"               , ejb                    )
      self.uri=                    Attribute("uri"               , uri                    )
      self.referenceBinding=       Attribute("referenceBinding"  , referenceBinding       )
      self.resourceType=           Attribute("resRef.type"       , resourceType           )
      self.jndiName=               Attribute("JNDI"              , jndiName               )
      self.loginConfigurationName= Attribute("login.config.name" , loginConfigurationName )
      self.properties=             Attribute("auth.props"        , properties             )

      # create the Options list
      # attention, order cares!
      optionFields = list('')
      optionFields.append(self.module)                  # 0  "Module"
      optionFields.append(self.ejb)                     # 1  "EJB"
      optionFields.append(self.uri)                     # 2  "URI"
      optionFields.append(self.referenceBinding)        # 3  "Reference binding"
      optionFields.append(self.resourceType)            # 4  "Resource type"
      optionFields.append(self.jndiName)                # 5  "JNDI name"
      optionFields.append(self.loginConfigurationName)  # 6  "Login Configuration Name"
      optionFields.append(self.properties)              # 7  "Properties"

      optionName = "MapResRefToEJB"
      #call super class Ctor
      InstallationOptions.__init__(self,optionName,optionFields)



# class MapResRefToEJB(InstallationOptions):
#
#    def __init__(self):
#
#       self.optionName = "MapResRefToEJB"
#       self.optionFields = [
#       "bpcobserverejb",                          # 0  "Module"                   :
#       "AuditDB",                                 # 1  "EJB"                      :
#       "bpcobserverejb.jar,META-INF/ejb-jar.xml", # 2  "URI"                      :
#       "jdbc/BPEDB",                              # 3  "Reference binding"        :
#       "javax.sql.DataSource",                    # 4  "Resource type"            :
#       "jdbc/BPEDB",                              # 5  "JNDI name"                :
#       '""',                                      # 6  "Login Configuration Name" :
#       '""'                                       # 7  "Properties"               :
#       ]
#

#-BindJndiForEJBMessageBinding [[bpcobservereventconsumerejb ObserverEventConsumer ceiconsumerejb.jar,META-INF/ejb-jar.xml null jms/observer/BPCObserverCEIConsumer_ActivationSpec1 jms/observer/BPCObserverCEIConsumerQueue null ]


#------------------------------------------------------------
# option class for Binding enterprise beans to listener port  names or
# activation specification JNDI names
#------------------------------------------------------------
# Fields and order
#   1) EJB module: bpcobservereventconsumerejb
#   2) EJB: ObserverEventConsumer
#   3) URI: ceiconsumerejb.jar,META-INF/ejb-jar.xml
#   4) Listener port: null
#   5) JNDI name: jms/observer/BPCObserverCEIConsumer_ActivationSpec
#   6) Destination JNDI Name: jms/observer/BPCObserverCEIConsumerQueue
#   7) ActivationSpec Authentication Alias: null

class BindJndiForEJBMessageBinding(InstallationOptions):

  def __init__( self, module, ejb, uri, listenerPort, jndiName, DstJndiName, specAlias):

     # Note: key names are not used for AdminApp.install.
     # however, set it for clearness
     self.module=       Attribute("module"        , module       )
     self.ejb=          Attribute("EJB"           , ejb          )
     self.uri=          Attribute("uri"           , uri          )
     self.listenerPort= Attribute("listenerPort"  , listenerPort )
     self.jndiName=     Attribute("JNDI"          , jndiName     )
     self.DstJndiName=  Attribute("DstJndiName"   , DstJndiName  )
     self.specAlias=    Attribute("apecAlias"     , specAlias    )

     # create the Options list
     # attention, order cares!
     optionFields = list('')
     optionFields.append(self.module)              # 1) EJB module
     optionFields.append(self.ejb)                 # 2) EJB
     optionFields.append(self.uri)                 # 3) URI
     optionFields.append(self.listenerPort)        # 4) Listener port
     optionFields.append(self.jndiName)            # 5) JNDI name
     optionFields.append(self.DstJndiName)         # 6) Destination JNDI Name
     optionFields.append(self.specAlias)           # 7) ActivationSpec Authentication Alias

     #set the option name
     optionName = "BindJndiForEJBMessageBinding"

     #call super class Ctor
     InstallationOptions.__init__(self,optionName,optionFields)

#------------------------------------------------------------
# option class for Mapping Roles To Users
#------------------------------------------------------------

class MapRolesToUsers(InstallationOptions):

  def __init__(self, role, everyone, allAuthenticated, mappedUsers, mappedGroups):

     self.role               = Attribute("role"               , role             )
     self.everyone           = Attribute("everyone"           , everyone         )
     self.allAuthenticated   = Attribute("allAuthenticated"   , allAuthenticated )
     self.mappedUsers        = Attribute("mappedUsers"        , mappedUsers      )
     self.mappedGroups       = Attribute("mappedGroups"       , mappedGroups     )

     # create the Options list
     # attention, order cares!
     optionFields = list('')
     optionFields.append( self.role             )  # 0 Role
     optionFields.append( self.everyone         )  # 1 Everyone
     optionFields.append( self.allAuthenticated )  # 2 All authenticated
     optionFields.append( self.mappedUsers      )  # 3 Mapped users
     optionFields.append( self.mappedGroups     )  # 4 Mapped groups

     optionName = "MapRolesToUsers"

     #call super class Ctor
     InstallationOptions.__init__(self,optionName,optionFields)

#   def __init__(self):
#
#      optionName = "MapRolesToUsers"
#      optionFields = [
#      "ObserverUser",              # 0 Role
#      "AppDeploymentOption.Yes",   # 1 Everyone?
#      "AppDeploymentOption.Yes",   # 2 All authenticated?
#      '""',                        # 3 Mapped users
#      '""'                         # 4 Mapped groups
#      ]
#
#

################################################################################
#
# class AdminShell
#
# Class to provide informations about the wsadmin shell
#
################################################################################
class AdminShell:

  CON_STATE_ERROR        = "ERROR"
  CON_STATE_CONNECTED    = "CONNECTED"
  CON_STATE_DISCONNECTED = "DISCONNECTED"
  CON_STATE_UNKNOWN      = "UNKNOWN"

  # set the static instance of the AdminShell
  instance               = None

  #---------------------------------------------------------------------------
  # Ctor
  #---------------------------------------------------------------------------
  def __init__(self):

    self.connectionStatus = self.getConnectionStatus()

    # name of the profile we are running in
    self.profileName=os.path.basename( javaEnv["user.install.root"] )

  #---------------------------------------------------------------------------
  # getConnectionStatus: determine if wsadmin was called with -conntype none
  # by parsing the log file
  #---------------------------------------------------------------------------
  def getConnectionStatus (self):

    connectionStatus = AdminShell.CON_STATE_UNKNOWN
    if not ScriptingExtensionHelper.getCurrentScriptingConfigService():
      connectionStatus = AdminShell.CON_STATE_ERROR
    elif not ScriptingExtensionHelper.getCurrentScriptingAdminClient():
      connectionStatus = AdminShell.CON_STATE_DISCONNECTED
    else:
      connectionStatus = AdminShell.CON_STATE_CONNECTED
    #endif

    return connectionStatus


#---------------------------------------------------------------------------
# set the static instance of the AdminShell
#---------------------------------------------------------------------------
AdminShell.instance = AdminShell()


################################################################################
#
# class AppServer
#
################################################################################
class AppServer:

  # the follwing are statics,to be set from outside with
  # AppServer.AdminApp=xyz


  AdminApp     = None
  AdminControl = None
  AdminConfig  = None
  AdminTask    = None

  logLocation  = os.path.join(javaEnv["user.install.root"],"logs")   # ToDo: replace with logDirs func from beplib


  #---------------------------------------------------------------------------
  # Ctor
  #---------------------------------------------------------------------------
  def __init__(self):

    self.cell      = ""
    self.node      = ""
    self.server    = ""

    self.cellName= ""
    self.nodeName  = ""
    self.serverName= ""

    self.isClusterServer = 0
    self.isManagedServer = 0
    self.isConnected = ( AdminShell.instance.connectionStatus == AdminShell.CON_STATE_CONNECTED )


  #---------------------------------------------------------------------------
  # get the names of the current AppServer and check if we are connectd
  #---------------------------------------------------------------------------
  def setToConnected(self):

    AdminConfig =AppServer.AdminConfig
    AdminControl=AppServer.AdminControl

    if AdminConfig == None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminConfig")

    if self.isConnected:
      try:
        svr = AdminControl.queryNames("node="+AdminControl.getNode( )+",type=Server,*" )
        self.server = AdminControl.getConfigId( svr )

        self.cellName   = AdminControl.getCell()
        self.nodeName   = AdminControl.getNode()
        self.serverName = AdminConfig.showAttribute(self.server, "name")

        self.node   = AdminConfig.getid("/Node:"+self.nodeName)
        self.cell   = AdminConfig.getid("/Cell:"+self.cellName)
      except:
        pass

    if not self.server:
      # either not connected or a problem with AdminControl
      # do some guessing
      self.server = AdminConfig.list("Server").split(lineSep)[0] #just try the first
      self.serverName = AdminConfig.showAttribute(self.server, "name")

      self.nodeName  = javaEnv["local.node"]
      self.cellName  = javaEnv["local.cell"]

      self.node = AdminConfig.getid("/Node:"+self.nodeName)
      self.cell = AdminConfig.getid("/Cell:"+self.cellName)


    self.isManagedServer = ( AdminConfig.showAttribute(self.cell, "cellType") == "DISTRIBUTED")

    return

  #---------------------------------------------------------------------------
  # get the names of the current AppServer and check if we are connectd
  #---------------------------------------------------------------------------
  def setToServer(self,serverName=None, nodeName=None):

    AdminConfig =AppServer.AdminConfig
    AdminControl=AppServer.AdminControl

    if AdminConfig == None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminConfig")

    if nodeName==None:
      nodeName = self.nodeName

    if serverName==None:
      serverName = self.serverName

    serverId = AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/")
    if serverId=='':
      raise NameError, mcat.get("ErrorServerDoesNotExist", serverName)

    nodeId = AdminConfig.getid("/Node:"+nodeName+"/")
    if nodeId=='':
      raise NameError, mcat.get("ErrorServerDoesNotExist", nodeName)

    #server and node do exist, use it
    self.serverName = serverName
    self.server = serverId
    self.nodeName = nodeName
    self.node = nodeId
    self.isClusterServer = 0

    return

  #---------------------------------------------------------------------------
  #get the names of the current AppServer and check if we are connectd
  #---------------------------------------------------------------------------
  def setToCluster(self,serverName=None):

    AdminConfig =AppServer.AdminConfig
    AdminControl=AppServer.AdminControl

    if AdminConfig == None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminConfig")

    if serverName==None:
      serverName = self.serverName

    serverId = AdminConfig.getid("/ServerCluster:"+serverName+"/")
    if serverId=='':
      raise NameError, mcat.get("ErrorClusterDoesNotExist", serverName)

    #server does exist, use it
    self.serverName = serverName
    self.server = serverId
    self.isClusterServer = 1

    return

  #------------------------------------------------------------
  # start an application
  #------------------------------------------------------------
  def startApplication(self, appName):

     AdminControl=AppServer.AdminControl
     if AdminControl==None:
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminControl")


     # start the application
     # Attention: Ensure that in the cmdline there are no spaces between
     # values and commas. Else you will get "" as result
     try:

       if self.isConnected != 1:
         log( mcat.get("ServerOrClusterRequired") )
         raise Exception,mcat.get("ServerOrClusterRequired")

       if self.serverName == "<unknown>":
         log( mcat.get("AppServerNotRunning") )
         raise Exception,mcat.get("AppServerNotRunning")

       log( mcat.get("StartingApplication", appName) )
       cmdline =""
       cmdline += "cell=" + self.cellName
       cmdline += ",node=" + self.nodeName
       cmdline += ",process=" + self.serverName
       cmdline += ",type=ApplicationManager,* "
       appManager=AdminControl.queryNames(cmdline)

       AdminControl.invoke(appManager, "startApplication", appName )

       log( mcat.get("StartedSuccessfully", appName) )

     except com.ibm.ws.exception.ConfigurationWarning:
       log( mcat.get("StartedSuccessfully", appName) )

     except:
       log( mcat.get("StopFailed", appName) )
       handleException()

     return

  #------------------------------------------------------------
  # stop an application
  #------------------------------------------------------------
  def stopApplication(self, appName):


     AdminControl=AppServer.AdminControl
     if AdminControl==None:
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminControl")


     # start the application
     # Attention: Ensure that in the cmdline there are no spaces between
     # values and commas. Else you will get "" as result
     try:

       if self.isConnected != 1:
         log( mcat.get("ServerOrClusterRequired") )
         raise Exception,mcat.get("ServerOrClusterRequired")

       if self.serverName == "<unknown>":
         log( mcat.get("AppServerNotRunning") )
         raise Exception,mcat.get("AppServerNotRunning")

       log( mcat.get("StoppingApplication", appName) )
       cmdline =""
       cmdline += "cell=" + self.cellName
       cmdline += ",node=" + self.nodeName
       cmdline += ",process=" + self.serverName
       cmdline += ",type=ApplicationManager,* "
       appManager=AdminControl.queryNames(cmdline)

       try:
         AdminControl.invoke(appManager, "stopApplication", appName )
       except :
         e=str(sys.exc_info()[1])
         if e.find("Application " + appName + " not started") >-1:
           # ignore an alreay stopped
           log( mcat.get("AlreadyStopped", appName) )
           return
         else:
           raise

       log( mcat.get("StoppedSuccessfully", appName) )

     except com.ibm.ws.exception.ConfigurationWarning:
       log( mcat.get("StoppedSuccessfully", appName) )

     except:
       log( mcat.get("StartFailed", appName) )
       handleException()

     return

  #------------------------------------------------------------
  # make an application name that contains infrastructure infos
  #------------------------------------------------------------
  def makeBPCAppName(self, appName):

    if self.isClusterServer==1:
      name=appName + "_" + self.serverName
    else:
      name=appName + "_" + self.nodeName + "_" + self.serverName
    return name



################################################################################
#
# EnterpriseApplication
#
################################################################################
class EnterpriseApplication:


  #------------------------------------------------------------
  # Ctor
  #------------------------------------------------------------
  def __init__(self, appName, earFileName):

     self.appName=appName
     self.initialAppName=appName
     self.earFileName=earFileName
     self.earFilePath=os.path.join(javaEnv["was.install.root"],"ProcessChoreographer","internal")
     self.earFile=os.path.join(self.earFilePath,self.earFileName)
     self.doUpdate = 1
     self.doSave   = 1
     self.installOptions = list("")

     self.theAppServer = AppServer()  # Object serving infos about server names
     # do not set to current here, because some things can be done w/o connection
     #self.theAppServer.setToCurrent()

     #configuration profile
     self.profileJarName="configuration.jar" # depecrated
     self.profileFilename="com/ibm/bpe/observer/configuration.properties"

     # Console Handler
     self.console = console

     # profile access methods via AdminTask
     self.profileGetMethod=None
     self.profileSetMethod=None

  #------------------------------------------------------------
  # AlreadyExiting
  #------------------------------------------------------------
  def alreadyExists(self, appName=None):

    if appName==None:
      appName=self.appName


    AdminApp=AppServer.AdminApp

    if AdminApp==None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminApp")

    appList=AdminApp.list()

    if appList.find(appName) > -1:
       return 1

    return 0


  #------------------------------------------------------------
  # uninstall the App
  #------------------------------------------------------------
  def unInstall(self, appName=None):

    if appName==None:
      appName=self.appName

    AdminApp=AppServer.AdminApp
    if AdminApp==None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminApp")

    if self.alreadyExists(appName):
      AdminApp.uninstall(appName)
      log( mcat.get("AppUninstalled", appName) )
    else:
      log( mcat.get("NotInstalledNotFound", appName) )


  #------------------------------------------------------------
  # install the App
  #------------------------------------------------------------
  def install(self, appName=None):

     AdminApp=AppServer.AdminApp
     AdminConfig=AppServer.AdminConfig

     if AdminApp==None:
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminApp")
     if AdminConfig==None:
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminConfig")

     if appName==None:
       appName=self.appName

     # easy options that are only one word:

     # appname
     cmdline=" -appname " + self.appName

     # do an update
     if self.doUpdate:
       cmdline= cmdline + " -update "

     #complex options that are represented by an InstallationOptions child
     for option in self.installOptions:
       cmdline = cmdline + option.getOptionCommandLine()

     # install the application
     AdminApp.install(self.earFile, cmdline)

     #save the new admin configuration
     if self.doSave:
       AdminConfig.save()

  ################################################################################
  #
  # composeProfilePath
  #
  # compose the path of the ear
  # path on the dmgr is:
  #   <user.install.root>\config\cells\<cellName>\applications\<appName>\<appName>  #duplicate item is no typo
  # path on a server:
  #   <user.install.root>installedApps\<cellName>
  #
  ################################################################################

  def getProfile( self ):

    from bpcoProfile import ConfigurationProfile

    if self.profileGetMethod == None:
      raise UnboundLocalError, mcat.get("ErrorNotInitialized", "self.profileGetMethod")

    if not self.alreadyExists():
      print mcat.get("AppNotFound", self.appName)
      return

    print mget("ReadingProfile", self.appName)
    profile=ConfigurationProfile(self.profileFilename) # file no more used


    valuesAsString = self.profileGetMethod("-appName " + self.appName)
    profile.loadFromString(valuesAsString)

    return profile


  ################################################################################
  #
  # changeProfile
  #
  # Allow the user to update the application settings
  #
  ################################################################################
  def changeProfile(self):

    try:

      print mget("NewLine")
      if self.selectDeploymentTarget(mget("PromptInstalledTarget")) == self.console.CANCEL:
        return self.console.CANCEL

      if not self.alreadyExists():
        print mcat.get("AppNotFound", self.appName)
        return

      #do this here because bpcoProfile imports also bpelib -> error because log does not exist
      from bpcoProfile import ConfigurationProfile

      # get a new instance of the profile
      profile=self.getProfile()
      profile.editProfile()

      if profile.changed==1:
        answer=self.console.selectYesNo( mcat.get("PromptForSave"), self.console.YES )
        if answer==self.console.YES:
          options = "-appName " + self.appName + " "
          for key in profile.getKeyList():
            param=profile.knownKeys[key]
            if param.adminTaskParameter!=None:
              options+= param.adminTaskParameter + " " + str(param.value) + " "

          print mget("UpdatingProfile")

          self.profileSetMethod(options)
          AppServer.AdminConfig.save()

          print mcat.get("NewLine")
          print mcat.get("InfoRestartApplication", self.appName)

          if self.theAppServer.isManagedServer:
            log(mcat.get("InfoEnsureSync"))

          logOnly( mcat.get("ProfileUpdateSuccess", self.appName , "\n"+ profile.asString() ))


    except:
      handleException(0)

    return

  ################################################################################
  #
  # getProfileValue
  #
  # read a profile key for tha application
  #
  ################################################################################
  def getProfileValue(self, key):

    value=None

    try:

      #do this here because bpcoProfile imports also bpelib -> causes an error because log does not exist
      from bpcoProfile import ConfigurationProfile

      # get a new instance of the profile and read it
      profile=self.getProfile()

      # read value
      value=profile.getValue(key)

    except:
      handleException(0)

    return value


  ################################################################################
  #
  # setProfileValue
  #
  # set a profile key for an application
  #
  ################################################################################
  def setProfileValue(self, key, value):

    try:

      #do this here because bpcoProfile imports also bpelib -> causes an error because log does not exist
      from bpcoProfile import ConfigurationProfile

      # get a new instance of the profile and read it
      # profile=self.getProfile()

      profile=ConfigurationProfile(self.profileFilename) # file no more used

      # set the value and update the app
      profile.setValue(key,value)
      param=self.knownKeys[key]
      options = "-appName " + self.appName + " "
      options+= param.adminTaskParameter + " " + param.value + " "

      self.profileSetMethod(options)


    except:
      handleException(0)

    return value

  ################################################################################
  #
  # selectDeploymentTarget
  #
  # select the deployment target from a list
  #
  ################################################################################
  def selectDeploymentTarget(self,promptMessage="", updateAppServerInstance=1):

    # in an unmanaged environment, there is olny one server
    if self.theAppServer.isManagedServer==0:
      log(mget("WorkingOnServer",self.theAppServer.nodeName, self.theAppServer.serverName ))
      if updateAppServerInstance==1:
        self.appName=self.theAppServer.makeBPCAppName(self.initialAppName)
      return [self.theAppServer.nodeName, self.theAppServer.serverName]

    if promptMessage=="":
      promptMessage=mcat.get("PromptInstallTarget")

    AdminConfig=AppServer.AdminConfig

    cellName = self.theAppServer.cellName
    cell     = self.theAppServer.cell

    # have a list of tupels with server,node.
    # if node=None, server is a cluster
    targetList=[]

    # get all possible targets
    cellMgrs = AdminConfig.list("CellManager").split(lineSep)
    if cellMgrs == ['']:
      cellMgrs = []

    nodeAgents = AdminConfig.list("NodeAgent").split(lineSep)
    if nodeAgents == ['']:
      nodeAgents = []

    clusters = AdminConfig.list("ServerCluster").split(lineSep)
    if clusters == ['']:
      clusters = []

    servers = AdminConfig.list("Server").split(lineSep)
    if servers == ['']:
      servers = []

    selectionList = []

    # build list of  dmgr and nodeagents
    unusableTargets=[]
    for entry in nodeAgents:
      unusableTargets.append( AdminConfig.showAttribute(entry,"server") )
    for entry in cellMgrs:
      unusableTargets.append( AdminConfig.showAttribute(entry,"server") )

    # start with clusters
    for entry in clusters:
      name = AdminConfig.showAttribute(entry,"name")
      selEntry = mget("Cluster", name)
      selectionList.append(selEntry)
      targetList.append([name,''])


    for entry in servers:

      # skip dmgr and ndAgent
      if entry in unusableTargets:
        continue

      # for non-clustered server, get serverName and nodeName
      if (AdminConfig.showAttribute(entry,"clusterName")==None):
        serverName = AdminConfig.showAttribute(entry,"name")
        typeList=entry.split("/")
        nodeName=typeList[ typeList.index("nodes") + 1]
        selEntry = mget("Node", nodeName) + ", " + mget("Server", serverName)
        selectionList.append(selEntry)
        targetList.append([serverName,nodeName])

    #
    # show menu
    #

    print mcat.get("NewLine")
    print promptMessage

    selectionList[-1]=selectionList[-1]+"\n"
    validAnswers=range(1, len(selectionList)+1)

    # add exit menu
    validAnswers.append("0")
    selectionList.append( mget("Menu_Exit") + "\n" )
    targetList.append( ["",""] )

    ans = self.console.selectFromList("", validAnswers, 0, selectionList)

    server,node=targetList[ans]

    if server=="":
      return self.console.CANCEL

    if updateAppServerInstance==1:
      if node!="":
        self.theAppServer.setToServer(server,node)
      else:
        self.theAppServer.setToCluster(server)

      self.appName=self.theAppServer.makeBPCAppName(self.initialAppName)

    return targetList[ans]


#####################################################
## Classes representing some Admin objects
#####################################################
class AdminObject:

  def __init__(self, objectType=None, objectName="<unknown>" ):

    self.objectType  = objectType
    self.objectName  = objectName
    self.objectId    = None


#####################################################
class QueueDestination(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "SIBQueue", objectName)

    #queueName   = Attribute( "name", "BPCCEIConsumerQueueDestination" )
    self.queueName   = None # Attribute( "name", objectName )


#####################################################
class QueueConnectionFactory(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "J2CConnectionFactory", objectName)

    self.name          = None # Attribute( "name",                       objectName)
    self.factory       = None # Attribute( "connectionFactoryInterface", "javax.jms.QueueConnectionFactory")
    self.jndiName      = None # Attribute( "jndiName",                   "jms/BPCCEIConsumerQueueConnectionFactory")
    self.description   = None # Attribute( "description",                mcat.get("SIBQFDescription"))


#####################################################
class JMSQueue(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "J2CAdminObject", objectName)

    #name        = Attribute("name","BPCCEIConsumerQueue")
    self.name        = None # Attribute("name",objectName)
    self.jndiName    = None # Attribute("jndiName","jms/BPCCEIConsumerQueue")
    self.description = None # Attribute("description", mcat.get("SIBQDescription"))



#####################################################
class ActivationSpec(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "J2CActivationSpec", objectName)

    #name         = Attribute("name", "BPCCEIConsumerActivationSpec")
    self.name         = None # Attribute("name", objectName)
    self.jndiName     = None # Attribute("jndiName", "eis/BPCCEIConsumerActivationSpec")
    self.destJndiName = None # Attribute("destinationJndiName", JMSQueue.jndiName.value)
    self.description  = None # Attribute("description", mcat.get("ActivationSpecDesc"))
    self.authAlias    = None



#####################################################
class BFMEventGroupProfile(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "EventGroupProfile", objectName)

    #name           = Attribute("eventGroupName","BFMEvents")
    self.name           = None # Attribute("eventGroupName", objectName )
    self.selectorString = None # Attribute("eventSelectorString", "CommonBaseEvent[starts-with(@extensionName,'BPC.BFM.')]")


#####################################################
class DistributionQueue(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "DistributionQueue", objectName)

    self.connectionFactoy = None # Attribute("queueConnectionFactoryJNDIName", QueueConnectionFactory.jndiName.value)
    self.queueJNDIName    = None # Attribute("queueJNDIName", JMSQueue.jndiName.value)

#####################################################
class BPELFlowManager(AdminObject):

  def __init__(self, objectName ):

    AdminObject.__init__(self, "BPELFlowManager", objectName)

    self.BPEContainerApplicationName = None
    self.holdQueue                   = None
    self.retentionQueue              = None
    self.stateObserver               = None


################################################################################
#
# StringNameSpaceBinding
#
# encapsulate the WAS NameSpaceBining feature for strings
#
################################################################################
class StringNameSpaceBinding(AdminObject):

  def __init__(self, objectName, name="",nameInNameSpace="", stringToBind="" ):

    AdminObject.__init__(self, "StringNameSpaceBinding", objectName)

    self.name             = Attribute("name",name)
    self.nameInNameSpace  = Attribute("nameInNameSpace",nameInNameSpace)
    self.stringToBind     = Attribute("stringToBind",stringToBind)

  #------------------------------------------------------------
  # given the name and the name in namespace,
  # - set the objectId
  # - set the value of a StringNameSpaceBinding object
  #------------------------------------------------------------
  def lookup(self):

    try:

      if (self.name==None) or (self.nameInNameSpace==None):
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "StringNameSpaceBinding")

      attributeList=[ self.name , self.nameInNameSpace ]

      # lookup
      self.objectId = getObjectByAttributes(self.objectType, attributeList)

      if self.objectId != None:
        self.stringToBind.value = AppServer.AdminConfig.showAttribute( self.objectId, self.stringToBind.key  )
      else:
        self.stringToBind.value = None


    except:
      handleException(0)

    return self.stringToBind.value

  #------------------------------------------------------------
  # return the value of the binding
  #------------------------------------------------------------
  def getValue(self):
    return self.stringToBind.value


  #------------------------------------------------------------
  #
  # createOrUpdate
  #
  #------------------------------------------------------------
  def createOrUpdate(self,parent):

    rc=0

    try:

      if self.name==None and self.nameInNameSpace==None:
        raise UnboundLocalError, mcat.get("ErrorNotInitialized", "StringNameSpaceBinding")
      if parent==None:
        raise UnboundLocalError, mcat.get("ErrorParentObjectMissing",self.name)

      #save the value of the dbSchemaObserver, might be overwritten by the lookup below
      val=self.stringToBind.value

      # check if there is already a binding with this name
      self.lookup()

      #reset the value
      self.stringToBind.value=val


      #generate the param list
      param=list("")
      param.append( self.name.asList() )
      param.append( self.nameInNameSpace.asList())
      param.append( self.stringToBind.asList())

      # update or create
      if self.objectId == None:
        NameSpaceBinding=AppServer.AdminConfig.create("StringNameSpaceBinding", parent, param )
        self.objectId=NameSpaceBinding
        log( mcat.get("CreatedNameSpaceBinding") )
      else:
        NameSpaceBinding=AppServer.AdminConfig.modify(self.objectId, param )
        log( mcat.get("UpdatedNameSpaceBinding") )

      log( mcat.get("Name", self.name.value) )
      log( mcat.get("ID", NameSpaceBinding) )

    except:
      rc=-1
      handleException(0)

    return rc




################################################################################
################################################################################
################################################################################

# Static helper functions

################################################################################
################################################################################
################################################################################


#-------------------------------------------------------------------
#
# lookupEventCollector
#
# Lookup a Collector Installation
#
#-------------------------------------------------------------------
# have a static to do this only once
lookupEventCollectorApp = None

def lookupEventCollector(appServer):

  global lookupEventCollectorApp

  EventCollectorFound=0
  if lookupEventCollectorApp==None:
    lookupEventCollectorApp = EnterpriseApplication( "BPCECollector", "bpceventcollector.ear" ) # = defaultAppName

  theApp=lookupEventCollectorApp
  theApp.theAppServer = appServer
  theApp.appName=theApp.theAppServer.makeBPCAppName(theApp.initialAppName)
  theApp.profileGetMethod=AppServer.AdminTask.showBPCEventCollectorEnvEntries
  theApp.profileSetMethod=AppServer.AdminTask.updateBPCEventCollector
  return theApp.alreadyExists()


#-------------------------------------------------------------------
# getObjectbyAttributes
#
# static Helper to search for objects by attribute values
#
# Input:
#   - objectType    String containing the type, e.g. "EventGroupProfile"
#   - attributeList a list consisting of bpelib.Attribute items
#   - scope         a valid scope string, e.g.
#                   "(cells/fmtc2162Node01Cell|resources-cei.xml)"
#                   (This must be an exisiting xml file)
#
# Output:
#   - the first matching object if one found
#   - None if no object match found
#
#-------------------------------------------------------------------
def getObjectByAttributes(objectType, attributeList, scope=None):

  if str(type(attributeList)) != "org.python.core.PyList":
    raise TypeError("Type 'list' is expected for attributes list (2nd parameter)")

  AdminConfig=AppServer.AdminConfig

  if AdminConfig == None:
    raise UnboundLocalError, mcat.get("ErrorNotInitialized", "AdminConfig")

  if scope==None:
    objects=AdminConfig.list(objectType)
  else:
    objects=AdminConfig.list(objectType, scope)

  if objects=="":
    return None

  objects=objects.split(lineSep)
  for object in objects:

    for attribute in attributeList:
      try:

        if attribute.value != AdminConfig.showAttribute(object,attribute.key):
          # missmatch
          # continue with next object
          break

      except ScriptingException:

        if str(sys.exc_info()[1]).find("WASX7080E") > -1:
          log( "got different object type, skipping ", object )
          break

      #except :
      #  pdb.set_trace()

    else:
      # we went to all attributes w/o finding a missmatch -> object matches pattern
      return object
  # end of 'for object in objects'

  return None



#-------------------------------------------------------------------
# removeObject
#
# static Helper remove admin objects
#
# Input:
#   - objectName    String containing the type, e.g. "EventGroupProfile"
#   - objectId
#-------------------------------------------------------------------
def removeObject(objectId, objectName=""):

  if objectName != None:
    log( mcat.get("Removing", objectName) )
    log( mcat.get("ID", objectId) )
    AppServer.AdminConfig.remove(objectId)
    return 1

  return 0


#-------------------------------------------------------------------
# removeObjectbyAttribute
#
# static Helper remove admin objects that match the given Attributes
#
# Input:
#   - objectName    String containing the type, e.g. "EventGroupProfile"
#   - objectId
#-------------------------------------------------------------------
def removeObjectByAttributes(obj, attributeList, scope=None):


  objectType = obj.objectType
  objectName = obj.objectName
  objectId   = getObjectByAttributes(objectType, attributeList, scope)

  try:
    if objectId != None:
      removeObject(objectId, objectName )
    else:
      log( mcat.get("NoMatchingObject", objectName ) )
      return 0
  except:
    handleException(0)
    return 0

  return 1


#-------------------------------------------------------------------
# handleException
#
# static Helper for XCP handling
#
#-------------------------------------------------------------------
def handleException(Prompt=0, toLogOnly=0, mask="", logToUse=None):

   try:

     selfRisenExeption=0

     #no tracing here
     if Tracer.isTracing==1:
       sys.settrace(None)


     if logToUse==None:

       if toLogOnly==1:
         out=logOnly   # direct to function logOnly
         Prompt=0      # non prompt if log only
       else:
         out=log       # direct to function log and console

     else:

       out=logToUse.log
       ptc=logToUse.printToConsole
       logToUse.printToConsole= not toLogOnly



     tb=traceback.format_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
     info0=str(sys.exc_info()[0])
     info1=str(sys.exc_info()[1])
     if mask!="":
       info0=info0.replace(mask,"********")
       info1=info1.replace(mask,"********")

     if info0=="exceptions.SystemExit":
       # reraise the Systemexit
       if logToUse!=None:
         logToUse.printToConsole=ptc
       raise

     out( mcat.get("ExceptionOccured") )

     tb=traceback.format_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
     for l in tb:
       if mask!="":
         l=l.replace(mask,"********")
       out(l)
     out(mcat.get("NewLine") )

     if Prompt==1:

       out( mcat.get("PressCtoContinue") )
       out( mcat.get("NewLine") )

       while 1 :

         inp=raw_input()
         if inp in ("a","A"):
           selfRisenExeption=1
           if logToUse!=None:
             logToUse.printToConsole=ptc
           raise

         if inp in ("c","C"):
            break

     #no tracing here
     if Tracer.isTracing==1:
        sys.settrace(Tracer.theTracer.trace)

   except SystemExit:
     selfRisenExeption=1
     if logToUse!=None:
       logToUse.printToConsole=ptc
     raise

   except Exception, ex:

     if selfRisenExeption==1:
       if logToUse!=None:
         logToUse.printToConsole=ptc
       raise
     out('Sorry, an exeption occured while processing another exception ...')
     tb=traceback.format_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
     for l in tb:
       if mask!="":
         l=l.replace(mask,"********")
       out(l)
     out(mcat.get("NewLine") )
     if logToUse!=None:
       logToUse.printToConsole=ptc
     raise

   return

#-------------------------------------------------------------------
# getWASVariable
#
# get the value of a WebSphere variable
# in1: the name of the variable w/o ${}, eg for ${WAS_HOME}, name="WAS_HOME"
# in2: scope any valid scope, e.g. "cells:stickNode01Cell:nodes:stickNode01"
#-------------------------------------------------------------------
def getWASVariable(name,scope=None):

    attrib = Attribute("symbolicName" , name)
    value = None

    obj=getObjectByAttributes("VariableSubstitutionEntry", [attrib] )

    if obj!="" and obj!=None:
      value = AppServer.AdminConfig.showAttribute(obj,"value")

      if value==None:
        value=""
      else:
        expression=Pattern.compile(r".*(\$\{[^}]*\}).*")   # r"xxx" means raw string, no need for escaping \
        matcher=expression.matcher(java.lang.String(value))
        while matcher.matches():
          value=value.replace(matcher.group(1), getWASVariable( matcher.group(1)[2:-1], scope ))
          matcher=expression.matcher(java.lang.String(value))

    return value

