################################################################################
# BEGIN COPYRIGHT
################################################################################
#
# THIS PRODUCT CONTAINS RESTRICTED MATERIALS OF IBM
# All Rights Reserved * Licensed Materials - Property of IBM
# 5655-FLW (C) Copyright IBM Corporation 2005, 2009
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
################################################################################
# END COPYRIGHT
################################################################################
#
# Jython script to deploy the WebSphere BPC EventCollector enterprise archive
#
# Version: 1.2 08/11/05 06:13:29
#
################################################################################
__file__="collector.py"

################################################################################
#
# used external imports
#
################################################################################
import sys
import java.lang.Exception
import os.path


################################################################################
#
# Global settings
#
################################################################################

#get the global system properties, we get a dictionary here,
# -> access as follows: osname=sysEnv["os.name"]
javaEnv=java.lang.System.getProperties()
lineSep=javaEnv["line.separator"]


################################################################################
#
# own external modules
#
################################################################################
import bpelib

#just for dev
reload(bpelib)

#some shortcuts to have less typing
mcat = bpelib.mcat
mget = bpelib.mcat.get
log  = bpelib.log
logOnly  = bpelib.logOnly



#set the logfile
bpelib.LogFile.defaultLog=bpelib.LogFile(os.path.join(bpelib.AppServer.logLocation,"setupEventCollector.log"))

if bpelib.AdminShell.instance.connectionStatus == bpelib.AdminShell.CON_STATE_ERROR:
  log( mget("ErrorConnectingScript") )
  sys.exit()

#init the bpelib.AppServer statics
try:
  bpelib.AppServer.AdminConfig  = AdminConfig
  bpelib.AppServer.AdminControl = AdminControl
  bpelib.AppServer.AdminApp     = AdminApp
  bpelib.AppServer.AdminTask    = AdminTask
except NameError, missingName:
  logOnly( mget("ErrorAdminObjectMissing", str(missingName) ) )
  raise mget("ErrorAdminObjectMissing", str(missingName) )


################################################################################
#
# Main Class Event Collector (fromer Consumer)
#
################################################################################
class EventCollector(bpelib.EnterpriseApplication):


  #####################################################
  #
  # Member functions
  #
  #####################################################


  #####################################################
  # our constructor
  #####################################################
  def __init__(self, appName, earFileName):

    # call super constructor
    bpelib.EnterpriseApplication.__init__(self, appName, earFileName)

    self.theAppServer.setToConnected()
    svrname=self.theAppServer.serverName
    nodeName=self.theAppServer.nodeName

    # do as default the ear install
    self.doInstall = 1

    # do a silent install
    self.doSilent = 0

    # keep track of the selected JDNI name
    self.dataSourceJndiName = None

    # keep slos track of the selected DB schema
    self.dataSourceDBSchema=None     # schema used during db actions
    self.dataSourceSchema=None       # schema used to set in the application config settings
    self.profileKeyForDbSchema="observerSchemaName"

    # restricted mode: if we in local mode and didn#t get a server, allow db setup only
    self.restricted=0

    # buffer the database object we deal with
    self.db=None


    self.profileGetMethod=AdminTask.showBPCEventCollectorEnvEntries
    self.profileSetMethod=AdminTask.updateBPCEventCollector



    #####################################################
    ## Member classes holding the defaults for some Admin objects
    #####################################################

    # the SID resource adapter as parent of all

    #the authentication alias to be used
    n=nodeName + "/CommonEventInfrastructureJMSAuthAlias"
    self.alias = bpelib.Attribute("alias", n)
    #self.ceiAuthAlias = getObjectByAttributes("JAASAuthData", [alias] )


    self.queueDestination            = bpelib.QueueDestination("BPCCEIConsumerQueueDestination")
    self.queueDestination.queueName  = bpelib.Attribute( "name", "BPCCEIConsumerQueueDestination" ) # name only used by AdminTask
    self.queueDestination.identifier = bpelib.Attribute( "identifier", "BPCCEIConsumerQueueDestination" )

    self.queueConnectionFactory = bpelib.QueueConnectionFactory("BPCCEIConsumerQueueConnectionFactory")
    self.queueConnectionFactory.factory     = bpelib.Attribute( "connectionFactoryInterface", "javax.jms.QueueConnectionFactory")
    self.queueConnectionFactory.name        = bpelib.Attribute( "name",                       "BPCCEIConsumerQueueConnectionFactory")
    self.queueConnectionFactory.jndiName    = bpelib.Attribute( "jndiName",                   "jms/BPCCEIConsumerQueueConnectionFactory")
    self.queueConnectionFactory.description = bpelib.Attribute( "description",                mget("SIBQFDescription"))
    self.queueConnectionFactory.authAlias   = bpelib.Attribute( "authDataAlias",              self.alias.value )

    self.jmsQueue             = bpelib.JMSQueue("BPCCEIConsumerQueue")
    self.jmsQueue.name        = bpelib.Attribute("name","BPCCEIConsumerQueue")
    self.jmsQueue.jndiName    = bpelib.Attribute("jndiName","jms/BPCCEIConsumerQueue")
    self.jmsQueue.description = bpelib.Attribute("description",  mget("SIBQDescription"))

    self.activationSpec              = bpelib.ActivationSpec("BPCCEIConsumerActivationSpec")
    self.activationSpec.name         = bpelib.Attribute("name", "BPCCEIConsumerActivationSpec")
    self.activationSpec.jndiName     = bpelib.Attribute("jndiName", "eis/BPCCEIConsumerActivationSpec")
    self.activationSpec.destJndiName = bpelib.Attribute("destinationJndiName", self.jmsQueue.jndiName.value)
    self.activationSpec.description  = bpelib.Attribute("description",  mget("ActivationSpecDesc"))
    self.activationSpec.authAlias    = bpelib.Attribute("authenticationAlias", self.alias.value)

    self.transformerQueueDestination            = bpelib.QueueDestination("BPCTransformerQueueDestination")
    self.transformerQueueDestination.queueName  = bpelib.Attribute( "name", "BPCTransformerQueueDestination" ) # name only used by AdminTask
    self.transformerQueueDestination.identifier = bpelib.Attribute( "identifier", "BPCTransformerQueueDestination" )

    self.transformerJmsQueue             = bpelib.JMSQueue("BPCTransformerQueue")
    self.transformerJmsQueue.name        = bpelib.Attribute("name","BPCTransformerQueue")
    self.transformerJmsQueue.jndiName    = bpelib.Attribute("jndiName","jms/BPCTransformerQueue")

    self.transformerActivationSpec              = bpelib.ActivationSpec("BPCTransformerActivationSpec")
    self.transformerActivationSpec.name         = bpelib.Attribute("name", "BPCTransformerActivationSpec")
    self.transformerActivationSpec.jndiName     = bpelib.Attribute("jndiName", "eis/BPCTransformerActivationSpec")
    self.transformerActivationSpec.destJndiName = bpelib.Attribute("destinationJndiName", self.transformerJmsQueue.jndiName.value)

    self.bfmEventGroupProfile                = bpelib.BFMEventGroupProfile("BFMEvents")
    self.bfmEventGroupProfile.name           = bpelib.Attribute("eventGroupName","BFMEvents")
    self.bfmEventGroupProfile.selectorString = bpelib.Attribute("eventSelectorString", "CommonBaseEvent[starts-with(@extensionName,BPC.BFM.)]")

    self.distributionQueue                  = bpelib.DistributionQueue("BPCCEIDistributionQueue")  # name not used outside this script
    self.distributionQueue.connectionFactoy = bpelib.Attribute("queueConnectionFactoryJNDIName", self.queueConnectionFactory.jndiName.value)
    self.distributionQueue.queueJNDIName    = bpelib.Attribute("queueJNDIName", self.jmsQueue.jndiName.value)

    self.stringNameSpaceBindingSchema = bpelib.StringNameSpaceBinding("BPCCEIConsumerSchema","BPCCEIConsumerSchema","BPCCEIConsumerSchema")
    self.stringNameSpaceBindingSchema.lookup()

    self.WPS601SampleFound=0
    if self.stringNameSpaceBindingSchema.getValue()!=None:
      self.WPS601SampleFound=1

    # defines for the UDF and jar
    self.jarID="BPCODBUTIL"
    self.jarFileName="bpcodbutil.jar"

    # check if we in the z/OS USS
    self.isNativeZOS= (javaEnv["os.name"].find("z/OS") > -1)

    # indicate if the appserver was running prior derby db use
    self.appServerWasRunnning=0

    self.doDatabaseChecks=1

    # determine if the new Common DB framework should be used
    self.ignoreAntTask = 0



  ################################################################################
  #
  # Prepare the database with the cosunmer view
  #
  #
  ################################################################################
  def promptDatabaseType(self):

    bDone=0
    rc=0
    db=None

    # supported DBMS, init here
    DbmsTypeDB2           = 0
    DbmsTypeDB2iSeries    = 0
    DbmsTypeDB2zOSv8      = 0
    DbmsTypeDB2zOSv9      = 0
    DbmsTypeDerby         = 0
    DbmsTypeOracle        = 0

    keepOldInstance = 0
    oldInstanceExists=(self.db!=None)
    if oldInstanceExists:
      oldInstanceClass=str(self.db.__class__)


    try:

      ## generate menu

      menuList =[]
      validAnswers =[]
      itemCount=0

      # Item derby
      DbmsTypeDerby=itemCount
      menuList.append( mget("DbmsTypeDerby") )
      validAnswerDerby='c'
      validAnswers.append(validAnswerDerby)
      itemCount+=1

      # Item DB2
      DbmsTypeDB2=itemCount
      menuList.append( mget("DbmsTypeDB2") )
      validAnswerDB2='d'
      validAnswers.append(validAnswerDB2)
      itemCount+=1

      # item DB2iSeries
      DbmsTypeDB2iSeries=itemCount
      menuList.append( mget("DbmsTypeDB2iSeries") )
      validAnswerDB2iSeries='i'
      validAnswers.append(validAnswerDB2iSeries)
      itemCount+=1

      # Item z/os 8
      DbmsTypeDB2zOSv8=itemCount
      menuList.append( mget("DbmsTypeDB2zOSv8") )
      validAnswerDB2zOSv8='8'
      validAnswers.append(validAnswerDB2zOSv8)
      itemCount+=1

      # Item z/os 9
      DbmsTypeDB2zOSv9=itemCount
      menuList.append( mget("DbmsTypeDB2zOSv9") )
      validAnswerDB2zOSv9='9'
      validAnswers.append(validAnswerDB2zOSv9)
      itemCount+=1

      # Item Oracle
      DbmsTypeOracle=itemCount
      menuList.append( mget("DbmsTypeOracle") + "\n" )
      validAnswerOracle='o'
      validAnswers.append(validAnswerOracle)
      itemCount+=1

      # Item Exit
      ExitMenu=itemCount
      menuList.append( mget("Menu_Exit") + "\n" )
      validAnswers.append("0")   # press 0 to exit

      # outer while to control connections problems or typos on DB params
      while 1:


        ########################
        # prompt for database manager
        print mget("NewLine")
        print mget("SelectDBMSType",mget("ConsumerName"))

        defaultDbmsType=DbmsTypeDerby
        if oldInstanceExists:
          if oldInstanceClass=="bpelib.DB2":
            defaultDbmsType=DbmsTypeDB2
          if oldInstanceClass=="bpelib.DB2iSeries":
            defaultDbmsType=DbmsTypeDB2iSeries
          elif oldInstanceClass=="bpelib.DB2zOSV8":
            defaultDbmsType=DbmsTypeDB2zOSv8
          elif oldInstanceClass=="bpelib.DB2zOSV9":
            defaultDbmsType=DbmsTypeDB2zOSv9
          elif oldInstanceClass=="bpelib.Derby":
            defaultDbmsType=DbmsTypeDerby
          elif oldInstanceClass=="bpelib.Oracle":
            defaultDbmsType=DbmsTypeOracle

        # show menu
        inp= self.console.selectFromList( "", validAnswers, defaultDbmsType, menuList )

        try:

          if inp == ExitMenu:
            db=None

          elif inp == DbmsTypeDB2:
            if oldInstanceExists and oldInstanceClass=="bpelib.DB2":
              keepOldInstance = 1
            else:
              db = bpelib.DB2()

          elif inp == DbmsTypeDB2iSeries:
            if oldInstanceExists and oldInstanceClass=="bpelib.DB2iSeries":
              keepOldInstance = 1
            else:
              db = bpelib.DB2iSeries()

          elif inp == DbmsTypeDB2zOSv8:
            if oldInstanceExists and oldInstanceClass=="bpelib.DB2zOSV8":
              keepOldInstance = 1
            else:
              db = bpelib.DB2zOSV8()

          elif inp == DbmsTypeDB2zOSv9:
            if oldInstanceExists and oldInstanceClass=="bpelib.DB2zOSV9":
              keepOldInstance = 1
            else:
              db = bpelib.DB2zOSV9()

          elif inp == DbmsTypeDerby:
            if oldInstanceExists and oldInstanceClass=="bpelib.Derby":
              keepOldInstance = 1
            else:
              db = bpelib.Derby()

          elif inp == DbmsTypeOracle:
            if oldInstanceExists and oldInstanceClass=="bpelib.Oracle":
              keepOldInstance = 1
            else:
              db = bpelib.Oracle()

          break

        except ImportError:
          # cannot instanciate because of missing CLASSPATH settings
          continue

    except:
      bpelib.handleException(0)

    if keepOldInstance == 1:
      db=self.db
      db.resetSingletons()

    return db


  ################################################################################
  #
  # retrieveDbNameFromWCCM
  #
  # retrive the DB name from jndi name if possible
  #
  ################################################################################
  def retrieveDbNameFromWCCM(self):

    dbName=""

    if self.dataSourceJndiName == None:
      # not set
      return dbName

    ds=bpelib.getObjectByAttributes("DataSource", [ bpelib.Attribute("jndiName", self.dataSourceJndiName) ])
    if ds=="":  # we got no entry
      return dbName

    propertySet=self.theAppServer.AdminConfig.showAttribute(ds,"propertySet")
    if propertySet == "":
      return dbName

    props=self.theAppServer.AdminConfig.showAttribute(propertySet,"resourceProperties")
    if props=="" :
      return dbName

    datasourceList=props[1:-1].split(" ")

    for p in datasourceList :
      if p.find("databaseName") > -1:
         dbName=self.theAppServer.AdminConfig.showAttribute(p, "value")
         if dbName=="" :
           return dbName

         dbName=dbName.replace("${USER_INSTALL_ROOT}",javaEnv["user.install.root"])
         dbName=os.path.normpath(dbName)

    return dbName

  ################################################################################
  #
  # Propmt the database connection parameters
  #
  ################################################################################
  def promptDatabaseConnectionParameters(self, db, isNewInstance=1, bCreate=0):

    if db==None:
      raise AssertionError, mget("ErrorDbObjectNoInit")

    bDone=0
    rc=0

    if (db.delayConfig==0) or (self.doDatabaseChecks==1):

      ########################
      # JDBC driver to be used
      if db.dbmsType==db.DBMS_TYPE_DB2ISERIES:
        db.promptJdbcDriverImpl()

      ########################
      # JDBC connection type
      db.promptJdbcConnectionType()

    else:
      # for the correct prompts for db name, we need type 4
      db.jdbcDriverType=4


    ########################
    # database name
    print mget("NewLine")
    print mget("PromptDbNameHeading")

    if (db.dbmsType == db.DBMS_TYPE_DERBY):
      print mget("ProvideFullPathForCS")

    if isNewInstance:
      dbDefaultName=self.retrieveDbNameFromWCCM()
      db.databseName=dbDefaultName

    oldDbName=db.databaseName

    while 1:

      db.promptDatabaseName()
      if self.doDatabaseChecks==1:
        if (db.dbmsType == db.DBMS_TYPE_DERBY) and db.jdbcDriverType==2 and not os.path.exists(db.databaseName):
          print mget("ErrorPathDoesNotExist", db.databaseName)

          if db.delayConfig==1:
            a=self.console.selectYesNoCancel( mget("ContinueWithoutChecks"), self.console.NO )
            if a==self.console.NO:
              continue
            elif a==self.console.CANCEL:
              if isNewInstance:
                self.db=None
              return "Exit"
            else:
              self.doDatabaseChecks=0

          elif bCreate:
            a=self.console.selectYesNoCancel( mget("PromptCreateDatabase", db.databaseName), self.console.YES )
            if a==self.console.NO:
              continue
            elif a==self.console.CANCEL:
              if isNewInstance:
                self.db=None
              return "Exit"

            db.databaseName=db.databaseName+";create=true"

        elif (db.dbmsType == db.DBMS_TYPE_DERBY) and db.jdbcDriverType==2 and os.path.exists(db.databaseName) and self.theAppServer.isConnected==1  :
          # cannot update exisiting db when in connected mode due to bug in ant task:
          print mget("NewLine")
          print mget("ErrorNoUpdateWhenConnected", db.databaseName)
          print mget("InfoUseNetOrConnModeNone")
          return "repeat"


      break

    if (self.doDatabaseChecks==1) or (db.delayConfig==0):

      ########################
      # Host and port for type 4 DBs
      if db.jdbcDriverType==4 :
        db.promptDatabaseHostName()
        db.promptDatabasePort()

      ########################
      # JdbcDriverHome
      db.promptJdbcHome()

      ########################
      # User and password
      db.promptCredentials()

    if (db.dbmsType != db.DBMS_TYPE_DERBY) :
      db.setSchema(db.databaseUser.upper())

    return "OK"


  ################################################################################
  #
  # connect
  #
  ################################################################################
  def testConnection(self, db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    while 1:

      ########################
      ## try if we can connect
      print mget("NewLine")
      if (db.dbmsType==db.DBMS_TYPE_DERBY) :
        msg=mget("TryingConnectDerby", db.databaseName)
      else:
        msg=mget("TryingConnect", db.databaseName, db.getDatabaseUser())

      log(msg)

      try:

        db.attachToJdbcDriver()
        db.connect()
        break

      except LookupError, xcp:
        # JDBC driver could not be loaded
        return "promptNewParams"


      except java.sql.SQLException, xcp:

        print mget("ErrorConnecting")
        db.handleSqlException(xcp,1)  # display also on console

        if (db.dbmsType==db.DBMS_TYPE_DERBY) :
          if db.lastSqlException.getSQLState()=="XSDB6":
            # SQL Exception: Another instance of Derby may have already booted the database D:\dbs\dby\BPEDB.
            ans=self.promptForDerbyStop()
            if ans=="Exit":
              return ans
            else:
              continue

        print mget("NewLine")
        a=self.promptRetryConnect(db)
        if (a=="promptNewParams") or (a=="Exit"):
          return a
        elif a=="correctCredentials":
          db.promptCredentials()
          db.setSchema(db.databaseUser.upper())
        elif a=="retry":
          continue


    print mget("ConnectedTo", db.databaseName)
    return "connected"

  #----------------------------------------------------------------------------
  #
  # promptRetryConnect
  #
  #----------------------------------------------------------------------------
  def promptRetryConnect(self,db):

    rc = "Exit"

    ## generate menu

    menuList =[]
    validAnswers =[]
    itemCount=0

    newParms=itemCount
    menuList.append( mcat.get("Menu_EnterNewParams"))
    validAnswers.append('1')
    itemCount+=1

    retry=itemCount
    menuList.append( mcat.get("Menu_RetryWithoutNewParams") )
    validAnswers.append('2')
    itemCount+=1


    if db.dbmsType!=db.DBMS_TYPE_DERBY or db.jdbcDriverType==4:
      correctCredentials=itemCount
      menuList.append( mcat.get("Menu_CorrectUserOrPassword") + "\n"   )
      validAnswers.append('3')
      itemCount+=1
    else:
      correctCredentials=-1

    Exit=itemCount
    menuList.append( mcat.get("Menu_ExitDBSetup")+"\n")
    validAnswers.append('0')

    print mcat.get("NewLine")

    # show menu
    inp= self.console.selectFromList( mcat.get("PromptRetryConnect")+"\n", validAnswers, newParms , menuList )

    if inp == retry:
      rc = "retry"
    elif inp == correctCredentials:
      rc = "correctCredentials"
    elif inp == newParms:
      rc = "promptNewParams"
    else:
      rc = "Exit"

    return rc


  ################################################################################
  #
  # promptObserverSchema
  #
  ################################################################################
  def promptObserverSchema(self,db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    ########################
    # get the table schema
    schema = db.getSchema()

    while 1:

      print mget("NewLine")
      inp = self.console.selectString(mget("PromptSchema"), schema )
      if self.dataSourceSchema!=None and inp!=schema and schema!="" :
        print mget("DifferentFormerSchema", self.dataSourceSchema )
        a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if a!=self.console.YES: # 0 = yes
          continue

      break

    self.dataSourceDBSchema=inp.upper()
    db.setSchema(inp.upper())

  ################################################################################
  #
  # promptJarLocation
  #
  ################################################################################
  def promptJarLocation(self,db):

    print mget("NewLine")
    db.jarLocation = self.console.selectString(mget("PromptJarLocation", db.connectionHost), db.jarLocation )
    return db.jarLocation

  ################################################################################
  #
  # selectObserverSchemaFromList
  #
  ################################################################################
  def selectObserverSchemaFromList(self,db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    ########################
    # get the table schema
    schemas = db.listKnownObserverSchemas()

    menuList=schemas
    menuList[len(menuList)-1]=menuList[len(menuList)-1]+"\n"
    menuList.append(mget("Menu_Exit")+"\n")

    validAnswers=range(1,len(schemas))
    validAnswers.append("0")

    inp= self.console.selectFromList( mcat.get("FollowingSchemasExist", db.databaseName)+"\n", validAnswers, None , menuList )

    if menuList[inp]==len(menuList)-1:  # last Item == Exit
      return

    self.dataSourceDBSchema=menuList[inp]
    db.setSchema(menuList[inp])

    return


  ################################################################################
  #
  # promptForDerbyStop
  #
  ################################################################################
  def promptForDerbyStop(self):

    print mget("NewLine")
    print mget("InformAboutServerStop")
    print mget("StopServerOutsideWsadmin", self.theAppServer.serverName)
    print mget("EnterCAfterShutdown")
    print mget("StopServerNow", self.theAppServer.serverName)
    print mget("PressCtoContinue") , # leave comma to avoid newline

    while 1 :

       inp = self.console.selectString("")
       if inp in ("a","A"):
         return "Exit"

       if inp in ("c","C"):
         self.appServerWasRunnning=1
         return "Continue"


  ################################################################################
  #
  # promptForDerbyStart
  #
  ################################################################################
  def promptForDerbyStart(self):

    if self.theAppServer.isConnected==1 and self.appServerWasRunnning==1:

      print mget("StartServerNow", self.theAppServer.serverName)
      print mget("EnterCAfterStartup")
      print mget("PressCtoContinue") , # leave comma to avoid newline

      while 1 :

        inp = self.console.selectString("")
        if inp in ("a","A"):
          return "Exit"

        if inp in ("c","C"):
          self.theAppServer.AdminControl.reconnect()
          break

    else:
      print mget("RemindStartServer", self.theAppServer.serverName)

    return "Continue"

  ################################################################################
  #
  # Prepare the database for the observer
  #
  ################################################################################
  def prepareDatabase(self):

    bDone=0
    rc=0
    isNewInstance=(self.db==None)
    reuseConnection=0
    logToUse=None
    self.doDatabaseChecks=1

    ########################
    # Prompt to reuse DB object
    if isNewInstance==0:
      a=self.console.selectYesNoCancel( mget("PromptReuseDbConnection"), self.console.YES )
      if a==self.console.YES:
        reuseConnection=1
      elif a==self.console.CANCEL:
        return

    try:

      while 1:

        ########################
        # dbTpye
        if reuseConnection==0:
          db=self.promptDatabaseType()
          if db==None:
            return "Exit"
        else:
          db=self.db
          # to avoid error "Database 'B:\w\p\_Srv01\databases\BPEDB' not found. Error Code: 40000 SQLState: XJ004"
          if db.dbmsType == db.DBMS_TYPE_DERBY:
            db.resetSingletons()

        ########################
        # delayConfig
        if reuseConnection==0:
          print mget("NewLine")
          tmp=self.console.selectYesNo(mget("PromtCreateSqlOnly"), self.console.NO)
          db.delayConfig = ( tmp == self.console.YES )

          if db.delayConfig :

            if self.ignoreAntTask==1:
              print mget("NewLine")
              db.sqlOutFile=self.console.selectString(mget("PromptSqlOutFile") )

            print mget("NewLine")
            print mget("InfoPerformChecksInDb")
            tmp=self.console.selectYesNo(mget("PromptPerformChecksInDb"), self.console.YES)
            self.doDatabaseChecks= ( tmp == self.console.YES )




        ########################
        # db connection
        if reuseConnection==0:
          rs=self.promptDatabaseConnectionParameters(db,isNewInstance, 1) # for derby, allow new dbs
          if rs=="Exit":
            return
          elif rs=="repeat":
            # call ourself again
            return self.prepareDatabase()


        ########################
        # test if we can connect
        if self.doDatabaseChecks:
          rs=self.testConnection(db)

          if rs=="Exit" or rs==self.console.CANCEL:
            return
          elif rs=="promptNewParams":
            # call ourself again
            if db.connection!=None:
              db.disconnect()
            self.db=None
            return self.prepareDatabase()


        # db is valid, copy it to self.db (might already be done)
        self.db=db

        break

      ########################
      ## prompt schema
      self.promptObserverSchema(db)
      if db.delayConfig==0 and db.dbmsType==db.DBMS_TYPE_DB2ISERIES:
        # check if the collection exists. If not, prompt to create it
        collectionExists=db.checkForCollection()
        if collectionExists==0:
          print mget("InfoSchemaDoesNotExist",db.getSchema())

          while 1:
            a=self.console.selectYesNoCancel(mget("PromptCreateSchema", db.getSchema()), self.console.YES)
            if a==self.console.CANCEL:
              return
            elif a==self.console.NO:
              continue
            else:
              db.createDatabase=1     # means create collection here

            break

      elif db.delayConfig==1 and db.dbmsType==db.DBMS_TYPE_DB2ISERIES:
        # for the SQL file where we can ignore XCPs, always include the create schema stmt.
        db.createDatabase=1     # means create collection here

      ############################
      ## prompt which UDF to use
      if (db.dbmsType!=db.DBMS_TYPE_DERBY) :
        db.udfImplementation=self.promptJavaOrSqlUdf()
        if db.udfImplementation == self.console.CANCEL:
          return
      else:
        # for derby we need the path to the jarfile on the network server
        if db.jdbcDriverType==4:
          self.promptJarLocation(db)


      ############################
      ## more to prompt for z/OS
      if (db.dbmsType==db.DBMS_TYPE_DB2ZOSV8 or db.dbmsType==db.DBMS_TYPE_DB2ZOSV9) :
        db.promptStorageGroup()
        if db.udfImplementation==db.UDF_IMPL_JAVA:
          db.promptWlmEnvironmentId()

      ########################
      ## ok,  compose the path to our sql files
      sqlPath=os.path.join( javaEnv["was.install.root"], "dbscripts" )
      sqlPath=os.path.join( sqlPath, "ProcessChoreographer" )


      ########################
      ## check if the given db matches our prereqs
      if ( db.dbmsType!=db.DBMS_TYPE_DERBY      and
           db.dbmsType!=db.DBMS_TYPE_DB2ISERIES and
           self.doDatabaseChecks == 1 ) :

        print mget("NewLine")
        log(mget("CheckForTBS", db.ObserverTableSpaces))
        missingTbs=db.checkforObserverTablespace()

        if len(missingTbs) == 0:
          log(mget("AllTbsFound"))

        else:

          scriptFile="createTablespace_Observer.sql"
          sqlFile=os.path.join( sqlPath, os.path.join( db.getDbmsType()))
          sqlFile=os.path.join( sqlFile, scriptFile )

          print mget("NewLine")
          log(mget("ErrorTbsMissing", db.databaseName, str(missingTbs)[1:-1]))
          log(mget("CreateTbsScriptNotRun", scriptFile))

          if (db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV8 or db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV9) :
            # for z/OS no default tbs exists
            a=self.console.NO
          else:
            print mget("NewLine")
            print mget("ExplainDefaultTBSInstead")
            a=self.console.selectYesNo( mget("PromptUseDefaultTBSInstead"), self.console.YES )

          if a==self.console.YES:
            db.useDefaultTablespace=1
          else:
            print mget("NewLine")
            log(mget("ExecuteCreateTbsScript", sqlFile ))
            return


      #######################################
      #  go and execute
      if self.ignoreAntTask==1:
        schemaFile="createSchema_Observer.sql"

        ddlFile=os.path.join( db.dbmsType, schemaFile )
        sqlFile=os.path.join( sqlPath, ddlFile )
        failed=0
        logToUse=bpelib.LogFile(db.databaseLogfileName(1))
        failed=db.runDdlFile(sqlFile,0,logToUse)
        if failed!=0:
          raise Exception,mget("ErrorDatabaseSetupFailed")

      else:
        #######################################
        #  install jar
        if (db.delayConfig==0):
          if ( db.udfImplementation==db.UDF_IMPL_JAVA and
               db.dbmsType!=db.DBMS_TYPE_DERBY ):          # derby jar is installed from SQL script
            if self.installJar( db,1 ) < 0:
              raise Exception,mget("ErrorDatabaseSetupFailed")

        # disonnect here, because on Derby, ANT  will not be able to start
        # DB unless wsadmin is detached
        if self.doDatabaseChecks and (db!=None) and (db.connection!=None) :
           db.disconnect()
           if db.dbmsType==db.DBMS_TYPE_DERBY and db.jdbcDriverType==2:
             db.shutDownDatabase()

        print mget("NewLine")
        runMsg="ProcessSqlFile"
        if db.delayConfig:
          runMsg="WritingSqlFile"

        db.runSqlViaAdminTask(runMsg)
        if db.sqlViaAdminTaskResults["exception"]!="":
          raise Exception, db.sqlViaAdminTaskResults["exception"]

      if self.ignoreAntTask==1:
        # not nice, but currently easier:
        if ( db.udfImplementation==db.UDF_IMPL_JAVA and
             db.dbmsType!=db.DBMS_TYPE_DERBY ):

          failed=self.activateJavaUdf(db,silent=1)
          if failed!=0:
            raise Exception,mget("ErrorDatabaseSetupFailed")

        if self.doDatabaseChecks and (db!=None) and (db.connection!=None) :
           db.disconnect()

      if (db.delayConfig==1):
        if (db.dbmsType != db.DBMS_TYPE_DERBY) and (db.udfImplementation==db.UDF_IMPL_JAVA):
          log( mcat.get("InfoInstallJarManually") )
          log( mcat.get("NoteVisitDocumetation") )
      else:
        # grant execute on intervalin if connect user != schema
        if (db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV8 or db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV9) :
          if db.databaseUser.upper() != db.getSchema().upper() :
            # reconnect to execute grants
            db.connect()
            db.grantExecuteOnIntervalin()
            db.disconnect()

        log( mcat.get("DatabaseSetupSucess"))

      if db.sqlViaAdminTaskResults["sqlfile"]!="":
        log( mcat.get("SqlFileWritten", db.sqlViaAdminTaskResults["sqlfile"]) )

      if db.sqlViaAdminTaskResults["logfile"]!="":
        log( mcat.get("DbLogFileWritten", db.sqlViaAdminTaskResults["logfile"]) )


      #if AppServer was stopped to acess the db, restart it
      if (db.dbmsType == db.DBMS_TYPE_DERBY) :
        if self.appServerWasRunnning:
          self.promptForDerbyStart()

      # remind to setup a datasource for the database
      print mget("RemindCreateDatasource")

      #on z/OS, Java udf gets only active if db is restarted.
      if (db.udfImplementation == db.UDF_IMPL_JAVA) and (db.dbmsType == db.DBMS_TYPE_DB2ZOSV8 or db.dbmsType == db.DBMS_TYPE_DB2ZOSV9) :
        log(mcat.get("RestartDbToRefreshClasses", db.databaseName, db.getSchema()))

    except:

      bpelib.handleException(0)
      log( mcat.get("ErrorDatabaseSetupFailed"))
      if logToUse!=None:
        log( mcat.get("MoreInfoInLogFile", logToUse.fileName ))
      elif bpelib.LogFile.defaultLog!=None:
        log( mcat.get("MoreInfoInLogFile", bpelib.LogFile.defaultLog.fileName ))
      if db!=None:
        if db.sqlViaAdminTaskResults["logfile"]!="":
          log( mcat.get("DbLogFileWritten", db.sqlViaAdminTaskResults["logfile"]) )
        if (db.dbmsType == db.DBMS_TYPE_DERBY) :
          print mget("RemindStartServer", self.theAppServer.serverName)

      if (db!=None) and (db.connection!=None) :
        db.disconnect()

    # when writing the sql only, invalidate the db instance
    if db!=None:
      if db.delayConfig==1:
        self.db=None

    return

  ################################################################################
  #
  # remove all objects from the database
  #
  #
  ################################################################################
  def dropSchemaFromDatabase(self):

    bDone=0
    rc=0
    isNewInstance=(self.db==None)
    reuseConnection=0


    ########################
    # Prompt to reuse DB object
    if isNewInstance==0:
      a=self.console.selectYesNoCancel( mget("PromptReuseDbConnection"), self.console.YES )
      if a==self.console.YES:
        reuseConnection=1
      elif a==self.console.CANCEL:
        return

    try:
      while 1:

        ########################
        # dbTpye
        if reuseConnection==0:
          db=self.promptDatabaseType()
          if db==None:
            return "Exit"
        else:
          db=self.db
          # to avoid error "Database 'B:\w\p\_Srv01\databases\BPEDB' not found. Error Code: 40000 SQLState: XJ004"
          if db.dbmsType == db.DBMS_TYPE_DERBY:
            db.resetSingletons()


        ########################
        # db connection
        if reuseConnection==0:
          rs=self.promptDatabaseConnectionParameters(db,isNewInstance)
          if rs=="Exit":
            return


        ########################
        # test if we can connect
        rs=self.testConnection(db)

        if rs=="Exit" or rs==self.console.CANCEL :
          return
        elif rs=="promptNewParams":
          # recurse to our self
          if db.connection!=None:
            db.disconnect()
          self.db=None
          return self.dropSchemaFromDatabase()
        else:
          # db is valid, copy it ro self.db
          self.db=db

        ########################
        # get the table schema
        self.promptObserverSchema(db)


        ########################
        # Prompt if all is ok
        print mget("NewLine")
        a=self.console.selectYesNo( mget("PromptRemoveDb", db.getSchema(), db.databaseName), self.console.NO )
        if a!=self.console.YES:
          return

        ########################
        # check if we must drop the jar later
        udfImpl=bpelib.DatabaseManager.UDF_IMPL_SQL
        if db.dbmsType!=db.DBMS_TYPE_DERBY:
          udfImpl=self.determineUdfState(db,1)


        ########################
        ## compose the path to our sql files
        sqlPath=os.path.join( javaEnv["was.install.root"], "dbscripts" )
        sqlPath=os.path.join( sqlPath, "ProcessChoreographer" )


        #######################################
        #  go and execute
        sqlFile=os.path.join( db.dbmsType, "dropSchema_Observer.sql"  )
        sqlFile=os.path.join( sqlPath, sqlFile )
        ddlFailed=0

        logToUse=bpelib.LogFile(db.databaseLogfileName(1))

        ddlFailed=db.runDdlFile(sqlFile,0,logToUse)

        logToUse.close()

        #######################################
        # remove the bpcodbutil.jar
        removeJarfailed=0
        if udfImpl==db.UDF_IMPL_JAVA:
          if (db.dbmsType != db.DBMS_TYPE_DERBY) :
            # derby jar is remove from within SQL file
            removeJarfailed=self.removeJar(db,1)

        #######################################
        # summarize
        if (removeJarfailed!=0) or (ddlFailed!=0):
          log( mcat.get("SchemaRemoveFailed"))
          if bpelib.LogFile.defaultLog!=None:
            log( mcat.get("MoreInfoInLogFile", logToUse.fileName ))

        else:
            log( mcat.get("SchemaRemoveSucess"))

        if db!=None and db.connection!=None :
           db.disconnect()

        if rc==0:  # all went ok
          if (db.dbmsType == db.DBMS_TYPE_DERBY) :

            #if AppServer was stopped to acess the db, restart it
            if (db.dbmsType == db.DBMS_TYPE_DERBY) :
              if self.appServerWasRunnning:
                self.promptForDerbyStart()

            else:
              print mget("RemindStartServer", self.theAppServer.serverName)

        else:
          if (db.dbmsType == db.DBMS_TYPE_DERBY) :
            print mget("RemindStartServer", self.theAppServer.serverName)

        break

    except:
      bpelib.handleException(0)
      if db!=None:
        if (db.dbmsType == db.DBMS_TYPE_DERBY) :
          print mget("RemindStartServer", self.theAppServer.serverName)

    if db!=None and db.connection!=None :
      db.disconnect()

    return

  ################################################################################
  #
  # collectInstallOptions
  #
  ################################################################################
  def collectInstallOptions(self):

    # init

    #create binding option for the external DB
    mapDBRes=bpelib.MapResRefToEJB( "bpcobservereventconsumerejb",             # 0  "Module"
                                    "ObserverEventConsumer",                   # 1  "EJB"
                                    "ceiconsumerejb.jar,META-INF/ejb-jar.xml", # 2  "URI"
                                    "jdbc/OBSERVERDB",                         # 3  "Reference binding, name  withing code "
                                    "javax.sql.DataSource",                    # 4  "Resource type"
                                    "jdbc/BPEDB",                              # 5  "JNDI name of the DB"
                                    '""',                                      # 6  "Login Configuration Name"
                                    '""')                                      # 7  "Properties"


    print mget("SearchAppInstall", mget("ConsumerShortName"), self.theAppServer.serverName)
    alreadyExists  = self.alreadyExists()


    self.doUpdate  = alreadyExists
    self.doInstall = 1

    self.dataSourceJndiName = "jdbc/BPEDB"


    self.dataSourceSchema=""
    self.stringNameSpaceBindingSchema.stringToBind.value=""


    # Mapping Activationb Spec to jndi name
    mapActSpecToJndi=bpelib.BindJndiForEJBMessageBinding(
                        "bpcobservereventconsumerejb",              # 1) EJB module
                        "ObserverEventConsumer",                    # 2) EJB
                        "ceiconsumerejb.jar,META-INF/ejb-jar.xml",  # 3) URI
                        "[]",                                       # 4) Listener port
                        self.activationSpec.jndiName.value,         # 5) JNDI name
                        self.jmsQueue.jndiName.value,               # 6) Destination JNDI Name
                        self.alias.value)                          # 7) ActivationSpec Authentication Alias

    ######################################
    #
    if self.doSilent==1:

      # all done in non-interactive mode
      self.installOptions.append(mapActSpecToJndi)
      self.installOptions.append(mapDBRes)

      return


    ######################################
    #
    # promt for values
    #

    #provide a list of valid dataSources
    jndiNamesList = list("")
    try:
      dataSourceList=AdminConfig.list('DataSource')
    except java.lang.Exception :
       jndiNamesList.append( mget("NoServerNoList"))
       pass

    else:
      dataSourceList=dataSourceList.split(lineSep)
      for dataSource in dataSourceList:
         # the call to rstrip() below is to be platform independend
         # for win it means get rid of "\r" = Caridge return
         # jndiNamesList.append(AdminConfig.showAttribute(dataSource.rstrip(),"jndiName"))
         jName=AdminConfig.showAttribute(dataSource,"jndiName")
         if not jName in jndiNamesList:
           jndiNamesList.append(jName)

    jndiNamesList.sort()

    ###########################################
    # prompt for the different instal options
    ##########################################

      #########################
      # the app name & doUpdate

    self.doUpdate = 0

    if alreadyExists == 1:

      print mget("NewLine")
      print mget("ApplicationExists",self.appName)
      description   = [ mget("Overwrite"),       # 0
                        mget("Abort")]           # 1
                        # mget("SkipInstall") ]    # 2 no more supported with AdminTasks

      answer=self.console.selectFromList( mget("PromptForUpdate"),
                                          None,
                                          0,
                                          description )

      if answer == 0 :  # = yes
        self.doUpdate = 1
        self.doStart  = 0

      elif answer == 2 :   # # skip, no more supported
        self.doInstall = 0

      else:
        return self.console.CANCEL   # abort

    else:
      print mget("AppNotFound",self.appName)


    ######################################
    # ask for JNDI mapping for DB Resource

    defaultDb=mapDBRes.jndiName.value
    if not mapDBRes.jndiName.value in jndiNamesList:
      defaultDb=""

    while 1 :

      print mget("NewLine")
      print mget("WhichConsumerDBToUse")
      print mget("EnterQMarkForList")

      answer = self.console.selectString(mget("YourSelection"), defaultDb )

      if answer in jndiNamesList:
        break

      if answer=="?":
        for name in jndiNamesList:
          print name
        continue

      if not answer in jndiNamesList:
        print mget("NewLine")
        print mget("ValueNotInList")
        a=self.console.selectYesNoCancel( mget("ContinueAnyway"), self.console.NO )
        if a==self.console.NO:
          continue
        elif a==self.console.CANCEL:
          return self.console.CANCEL
        else:
          break


    mapDBRes.jndiName.value = answer
    self.dataSourceJndiName = answer


    ########################
    # get the table schema
    schema = ""
    if alreadyExists != 0:
      # get the schema of the already installed collector
      schema = self.getProfileValue(self.profileKeyForDbSchema)
      if schema==None:
        log( mget("ErrorProfileKeyNotFound", self.profileKeyForDbSchema))
    else:
      # for backward compatibility, look for the NameSpaceBinding
      if self.stringNameSpaceBindingSchema.stringToBind.value!=None:
        self.dataSourceSchema = self.stringNameSpaceBindingSchema.stringToBind.value
      elif self.dataSourceDBSchema!=None:
        self.dataSourceSchema = self.dataSourceDBSchema

    while 1:

      print mget("NewLine")
      print mget("PromptSchema")

      inp=self.console.selectString(mget("LeaveEmptyForDSUser"),schema)

      if inp==schema:
        # user accepted default, noe need to check
        break

      if inp!="":

        if self.dataSourceDBSchema!=None and schema!="" :
          print  mget("DifferentFormerSchema", schema )
          a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
          if a==self.console.YES:
            break

        elif alreadyExists!=0 and schema!="" :
          print  mget("DifferentConsumerSchema", schema )
          a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
          if a==self.console.YES:
            break
        else:
          break


    self.dataSourceSchema=inp
    self.stringNameSpaceBindingSchema.stringToBind.value=inp


    #######################################
    #
    # all is done, set the install options gatherd

    self.installOptions.append(mapActSpecToJndi)
    self.installOptions.append(mapDBRes)

    return


  ##############################################################################
  #
  # promptCluster
  #
  ##############################################################################
  def promptCluster(self,clusterPrompt=""):

    if clusterPrompt=="":
      clusterPrompt=mget("PromptCluster")

    clusterFound=0
    clusters=AdminConfig.list("ServerCluster")

    dc=clusters.split(lineSep)[0]
    if dc!="":
      default=AdminConfig.showAttribute(dc,"name")
    else:
      default=""

    while clusterFound==0:

      print mget("NewLine")
      clusterName=self.console.selectString(clusterPrompt,default)

      if clusterName=="?":
        clusters=AdminConfig.list("ServerCluster")
        for s in clusters.split(lineSep) :
          print AdminConfig.showAttribute(s,"name")
        continue


      # retrieve Node name for server, if there are multiple, list them
      clusters=AdminConfig.getid("/ServerCluster:"+clusterName+"/")
      if clusters=="":
        print mget("ErrorClusterDoesNotExist",clusterName)
        continue

      clusterFound=1

    return clusterName


  ##############################################################################
  #
  # promptServerAndNode
  #
  ##############################################################################
  def promptServerAndNode(self,serverPrompt="",nodePrompt=""):

    if serverPrompt=="":
      serverPrompt=mget("PromptServer")

    if nodePrompt=="":
      nodePrompt=mget("PromptNode")

    serverFound =0

    while serverFound==0:

      print mget("NewLine")
      serverName=self.console.selectString(serverPrompt,self.theAppServer.serverName)

      if serverName=="?":
        servers=AdminConfig.list("Server")
        for s in servers.split(lineSep) :
          print AdminConfig.showAttribute(s,"name")
        continue

      # retrieve Node name for server, if there are multiple, list them
      servers=AdminConfig.getid("/Server:"+serverName+"/")
      if servers=="":
        print mget("ErrorServerDoesNotExist",serverName)
        continue


      servers=servers.split(lineSep)
      typeList=servers[0].split("/")
      nodeName=typeList[ typeList.index("nodes") + 1]

      if len(servers)>1:
        print mget("MultipleServerFound")
        for s in servers :
          print s

      nodeFound=0

      while nodeFound==0 :

        print mget("NewLine")

        answer = self.console.selectString(nodePrompt,nodeName)

        if answer=="?":
          nodes=AdminConfig.list("Node")
          for s in nodes.split(lineSep) :
            print AdminConfig.showAttribute(s,"name")
          continue

        node=AdminConfig.getid("/Node:"+answer+"/")
        if node=="":
          print mget("ErrorNodeDoesNotExist",answer)
          continue

        serverTest=AdminConfig.getid("/Node:"+nodeName+"/Server:"+serverName+"/")
        if serverTest=="":
          print mget("ErrorServerNotExistOnNode",serverName,nodeName)
          continue

        nodeName=answer
        nodeFound=1
        serverFound=1

    return serverName,nodeName


  ################################################################################
  #
  # isCeiEventLoggingEnabled
  #
  ################################################################################
  def isCeiEventLoggingEnabled(self):

    rc="WarningCEIStateNotKnown"

    appName=bpelib.Attribute("BPEContainerApplicationName", self.theAppServer.makeBPCAppName("BPEContainer"))
    objectId = bpelib.getObjectByAttributes( "BPELFlowManager", [appName])

    if objectId != None:
      stateObserver=self.theAppServer.AdminConfig.showAttribute(objectId, "stateObserver")
      if stateObserver!="CEI":
        rc="WarningCEIEventsDisabled"
      elif stateObserver=="CEI":
        rc="InfoCEIEventsEnabled"
    else:
      rc="WarningBFMNotFound"

    return rc

  ################################################################################
  #
  # enableCeiEventLogging
  #
  ################################################################################
  def enableCeiEventLogging(self):

    rc=1

    appName= bpelib.Attribute("BPEContainerApplicationName", self.theAppServer.makeBPCAppName("BPEContainer"))
    logging= bpelib.Attribute("stateObserver", "CEI")

    objectId = bpelib.getObjectByAttributes( "BPELFlowManager", [appName])

    if objectId != None:
      if self.theAppServer.AdminConfig.modify( objectId, [logging.asList()] ) == "":
        rc=0

    return rc



  ################################################################################
  #
  # prepareAndInstall
  #
  ################################################################################
  def prepareAndInstall(self):

    rc=0

    doAppServerRestart=0

    print mget("NewLine")
    log(mget("InstallAppHeading",mget("ConsumerName")))
    print mget("NewLine")

    if self.selectDeploymentTarget() == self.console.CANCEL:
      return self.console.CANCEL

    try:

      # check for the 6.0.1 sample by lookup for the NameSpaceBinding
      if self.WPS601SampleFound==1:
        print mget("NewLine")
        log(mget("Warning601ConsInstalled"))
        samplePath=os.path.join( javaEnv["was.install.root"], "ProcessChoreographer" )
        samplePath=os.path.join( samplePath, "sample" )
        samplePath=os.path.join( samplePath, "observer" )
        log(mget("RemoveUsingScript", "removeConsumerSample", samplePath, mget("ConsumerShortName")))
        print mget("NewLine")
        answer=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if answer!=self.console.YES:
          return;

      self.doSave=0
      if self.collectInstallOptions() == self.console.CANCEL:
        return

      alreadyExists=self.alreadyExists()

      #do the install now
      if self.doInstall == 1:
        self.installConsumer()

      #check if CEI event logging is enabled
      print mget("NewLine")
      log(mget("InfoCheckCEIEvents"))
      ceiMsg=self.isCeiEventLoggingEnabled()

#       WarningCEIStateNotKnown
#       WarningCEIEventsDisabled
#       InfoCEIEventsEnabled
#       WarningBFMNotFound

      log(mget(ceiMsg,self.theAppServer.serverName))
      if ceiMsg=="WarningCEIEventsDisabled":
        log(mget("InfoCEIEventsRequired"))
        answer=self.console.selectYesNoCancel( mget("PromptEnableCEIEvents", self.theAppServer.serverName), self.console.YES )
        if answer==self.console.YES:
          self.enableCeiEventLogging()
          print mget("InfoRestartAppServer", self.theAppServer.serverName)
          doAppServerRestart=1
        elif answer==self.console.CANCEL:
          return
      elif ceiMsg=="WarningBFMNotFound":
        log(mget("InfoCEIEventsRequired"))
        log(mget("InfoEnableCEIOnOtherServer"))


      # ask for save
      print mget("NewLine")
      answer=self.console.selectYesNo( mget("PromptForSave"), self.console.YES )

      print mget("NewLine")
      if answer==self.console.YES:
        AdminConfig.save()
        if self.theAppServer.isManagedServer:
          log(mget("InfoEnsureSync"))
        if doAppServerRestart==0 and alreadyExists==1:
          print mget("InfoRestartApplication", self.appName)
      else:
        AdminConfig.reset()
        log( mget("ConfigResetDone") )
        print mget("NewLine")

    except :

      log( mget("ErrorDoReset") )
      bpelib.handleException(not self.doSilent)

      AdminConfig.reset()
      print mget("NewLine")
      log(  mget("ConfigResetDone") )
      print mget("NewLine")

      raise


    print mget("NewLine")
    return rc



  ################################################################################
  #
  # installConsumer
  #
  ################################################################################
  def installConsumer(self):

    print mget("NewLine")
    log(  mget("InstallHeadingConsumer"))  #" -- Deploying the WebSphere BPC consumer ... "
    print mget("NewLine")

    try:

       # try if the Ear file exists
       open(self.earFile)

       #######################################################
       # install the collector
       #######################################################
       log(  mget("Installing") )

       #provide the dataSource name
       dsClause =' -dataSource "' + self.dataSourceJndiName + '" '

       # the deployment target
       srvClause=" -serverName " + self.theAppServer.serverName + " -nodeName "   + self.theAppServer.nodeName + " "
       if self.theAppServer.isClusterServer==1:
         srvClause= " -clusterName " + self.theAppServer.serverName + " "

       # the schema name under which the observer tables live
       schemaClause=""
       if self.dataSourceSchema!=None and self.dataSourceSchema!="":
         schemaClause=" -" + self.profileKeyForDbSchema + " " + self.dataSourceSchema + " "

       #set the default for schemaOnFristTouch: always false.
       schemaOnFristTouchClause=" -observerCreateTables false"

       # indicate that we generate the DB scripts ourselfs using processBPCDatabaseScript
       dBScriptGenClause = " -suppressDBScriptGeneration true"


       #compose the command
       cmdline="[" + dsClause + srvClause + schemaClause + schemaOnFristTouchClause + dBScriptGenClause + "]"

       logout='AdminTask.installBPCEventCollector("' + cmdline + '")'

       logOnly( logout  )
       AdminTask.installBPCEventCollector( cmdline )

       print mget("NewLine")
       log(  mget("InstallSuccess",mget("ConsumerName")) )

    except:

      bpelib.handleException(not self.doSilent)
      raise

    return



  ################################################################################
  #
  # remove objects and enterprisApp
  #
  ################################################################################
  def removeApp(self):

    print mget("NewLine")
    log(mget("RemoveHeading",mget("ConsumerName")))

    if self.doSilent:

      answer=self.console.YES

      # in the silent case log the server we are working on, if interactive
      # this is shown in the mainMenu
      if self.theAppServer.serverName=="<unknown>":
        log(mget("NoServerSpecified"))
        return


    else:

      if self.selectDeploymentTarget(mget("PromptRemoveTarget")) == self.console.CANCEL:
        return self.console.CANCEL

    if self.alreadyExists()!=1:
      log(mget("AppNotFound",self.appName))
      if self.doSilent==0:
        answer=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if answer==self.console.NO:
          print mget("NewLine")
          answer=self.console.selectYesNo( mget("PromptAlsoRemoveDb"), self.console.NO )
          if answer==self.console.YES:
            self.dropSchemaFromDatabase()
          return

    else:
      if self.doSilent==0:
        answer=self.console.selectYesNo( mget("PromptRemoveApp",  mget("ConsumerName")), self.console.NO )

      if answer!=self.console.YES:
        return

      print mget("ChangesUndoable")

    self.removeWasObjects()
    self.unInstall()

    if self.doSilent==0:
      print mget("NewLine")
      answer=self.console.selectYesNo( mget("PromptForSave"), self.console.YES )

    if answer==self.console.YES:
      AdminConfig.save()
      if self.theAppServer.isManagedServer:
        print
        log(mget("InfoEnsureSync"))

      print mget("NewLine")
      if self.doSilent==0:
        print mget("NewLine")
        answer=self.console.selectYesNo( mget("PromptAlsoRemoveDb"), self.console.NO )
        if answer==self.console.YES:
          self.dropSchemaFromDatabase()
      rc=0
    else:
      AdminConfig.reset()
      log( mget("ConfigResetDone") )
      rc=1

    return rc

  ################################################################################
  #
  # removeConsumerPrereqs
  #
  # Remove the WAS admin objects created by us
  #
  #
  ################################################################################
  def removeWasObjects(self):


    #############################################
    # remove the EventGroupProfile, this also removes the DistributionQueue


    attribList=[ self.bfmEventGroupProfile.name ]

    while bpelib.removeObjectByAttributes(self.bfmEventGroupProfile, attribList, self.theAppServer.cell):
      pass


    #############################################
    # remove the Activation Spec

    attribList=[ self.activationSpec.name,
                 self.activationSpec.jndiName,
                 self.activationSpec.destJndiName ]

    while bpelib.removeObjectByAttributes(self.activationSpec, attribList, self.theAppServer.server):
      pass


    #############################################
    # remove the Transformer Activation Spec

    attribList=[ self.transformerActivationSpec.name,
                 self.transformerActivationSpec.jndiName,
                 self.transformerActivationSpec.destJndiName ]

    while bpelib.removeObjectByAttributes(self.transformerActivationSpec, attribList, self.theAppServer.server):
      pass


    #############################################
    # remove the JMSQueue

    consumerQueue = self.jmsQueue
    consumerQueue.name.value=self.theAppServer.makeBPCAppName(consumerQueue.name.value)

    attribList=[ consumerQueue.name,
                 consumerQueue.jndiName ]

    while bpelib.removeObjectByAttributes(consumerQueue, attribList, self.theAppServer.server):
      pass

    #############################################
    # remove the Transformer JMSQueue

    transformerQueue = self.transformerJmsQueue
    transformerQueue.name.value=self.theAppServer.makeBPCAppName(transformerQueue.name.value)

    attribList=[ transformerQueue.name,
                 transformerQueue.jndiName ]

    while bpelib.removeObjectByAttributes(transformerQueue, attribList, self.theAppServer.server):
      pass


    #############################################
    # remove the QueueConnectionFactory

    attribList=[ self.queueConnectionFactory.name,
                 self.queueConnectionFactory.jndiName ]

    while bpelib.removeObjectByAttributes(self.queueConnectionFactory, attribList, self.theAppServer.server):
      pass


    #############################################
    # remove the QueueDestination

    queueDestination = self.queueDestination
    queueDestination.identifier.value=self.theAppServer.makeBPCAppName(queueDestination.identifier.value)

    attribList=[ queueDestination.identifier ]

    while bpelib.removeObjectByAttributes(queueDestination, attribList, self.theAppServer.cell):
      pass

    #############################################
    # remove the Transformer QueueDestination

    transformerQueueDestination = self.transformerQueueDestination
    transformerQueueDestination.identifier.value=self.theAppServer.makeBPCAppName(transformerQueueDestination.identifier.value)
    attribList=[ transformerQueueDestination.identifier ]

    while bpelib.removeObjectByAttributes(transformerQueueDestination, attribList, self.theAppServer.cell):
      pass


    #############################################
    # remove the AuthAlias

    object = bpelib.AdminObject("JAASAuthData","BPCEventCollectorJMSAuthenticationAlias")
    aliasId    = self.theAppServer.makeBPCAppName("BPCEventCollectorJMSAuthenticationAlias")
    alias      =  bpelib.Attribute("alias", aliasId)
    attribList=[ alias ]

    while bpelib.removeObjectByAttributes(object, attribList, self.theAppServer.cell):
      pass

    #############################################
    # remove the StringNameSpaceBinding

    attribList=[ self.stringNameSpaceBindingSchema.name,
                 self.stringNameSpaceBindingSchema.nameInNameSpace ]

    while bpelib.removeObjectByAttributes(self.stringNameSpaceBindingSchema, attribList, self.theAppServer.server):
      pass

    return


  ##############################################################################
  #
  # activateJavaUdf
  #
  # Modify DB to use java based funcitons
  #
  # * determine if java UDF is active
  # * install the jar file
  # * drop the mappers to plain SQL functions
  # * create function definitions to map to the java UDF.
  # TBD: check by timestamp if the correct jar file is installed. if yes, skip
  # TDB: else reload
  # TBD: verify the activation by executing a test query
  #
  ##############################################################################
  def activateJavaUdf(self, db, silent=0):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    if (db.dbmsType==db.DBMS_TYPE_DERBY) :
      print mget("CurrentlyNotSupported", self.dbmsType)
      return

    failed=0
    logToUse=None

    try:

      if silent==0:
        active=self.determineUdfState(db,1)

        #############################
        # Warn if already java is active
        if active==bpelib.DatabaseManager.UDF_IMPL_JAVA:
          print mget("NewLine")
          print mget("WarningImplAlreadyActive", active)
          a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
          if a!=self.console.YES:
            return

        #############################
        # ensure user wants to switch
        print mget("NewLine")
        print(mget("WarningSwitchUdfImpl"))
        a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if a!=self.console.YES:
          return


        #############################
        # query SCHEMA
        self.promptObserverSchema(db)

        #############################
        # query WLM ID
        if (db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV8 or db.getDbmsType() == db.DBMS_TYPE_DB2ZOSV9) :
          db.promptWlmEnvironmentId()


      ########################
      ## compose the path to our sql files
      sqlPath=os.path.join( javaEnv["was.install.root"], "dbscripts" )
      sqlPath=os.path.join( sqlPath, "ProcessChoreographer" )

      logToUse=bpelib.LogFile(db.databaseLogfileName(1))

      #######################################
      #  drop functions
      if silent==0:
        log(mget("RemovingFunctions"))
      sqlFile1=os.path.join( db.dbmsType, "dropFunctions_Observer.sql"  )
      sqlFile1=os.path.join( sqlPath, sqlFile1 )
      db.runDdlFile(sqlFile1,0,logToUse) # ignore errors on drop. if create is ok, we are fine

      #######################################
      #  install jar
      if db.delayConfig==0:
        if self.installJar( db,1 ) < 0:
          failed=failed+1

      #######################################
      #  create functions
      print mget("NewLine")
      log(mget("CreatingJavaBasedFunctions"))
      sqlFile2=os.path.join( db.dbmsType, "createFunctionsJava_Observer.sql"  )
      sqlFile2=os.path.join( sqlPath, sqlFile2 )
      if db.runDdlFile(sqlFile2,0,logToUse) < 0:
        failed=failed+1

      #####################
      # explicit commit to enusre changes
      db.commit()

    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp)
      failed=failed+1

    except Exception, e:
      bpelib.handleException()
      failed=failed+1

    if silent==0:
      if failed:
        log( mget("ErrorDatabaseSetupFailed") )
        log( mget("MoreInfoInLogFile", logToUse.fileName ))
      else:
        log( mget("DatabaseSetupSucess") )
        if db.delayConfig==1:
          log( mcat.get("InfoInstallJarManually"))
          log( mcat.get("NoteVisitDocumetation"))

    if logToUse!=None:
      logToUse.close()

    return failed


  ##############################################################################
  #
  # activateSqlUdf
  #
  # Modify DB to use SQL based funcitons
  #
  # * determine if plain SQL functions are already activated
  # * drop the functions definitions for the UDF
  # * create the mappers to the plain SQL function.
  # * if not, remove the jar on demand ( user may let it reside in the DB)
  # TBD: verify the activation by executing a test query
  #
  ##############################################################################
  def activateSqlUdf(self,db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    if (db.dbmsType==db.DBMS_TYPE_DERBY) :
      print mget("CurrentlyNotSupported", self.dbmsType)
      return

    failed=0
    logToUse=None

    try:

      active=self.determineUdfState(db,1)

      if active==bpelib.DatabaseManager.UDF_IMPL_SQL:
        print mget("NewLine")
        print mget("WarningImplAlreadyActive", active)
        a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if a!=self.console.YES:
          return

      #############################
      # ensure user wants to switch
      print mget("NewLine")
      print(mget("WarningSwitchUdfImpl"))
      a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
      if a!=self.console.YES:
        return

      ####################################
      ## compose the path to our sql files
      sqlPath=os.path.join( javaEnv["was.install.root"], "dbscripts" )
      sqlPath=os.path.join( sqlPath, "ProcessChoreographer" )

      logToUse=bpelib.LogFile(db.databaseLogfileName(1))


      #######################################
      #  drop functions
      log(mget("RemovingFunctions"))
      sqlFile1=os.path.join( db.dbmsType, "dropFunctions_Observer.sql"  )
      sqlFile1=os.path.join( sqlPath, sqlFile1 )
      db.runDdlFile(sqlFile1,0,logToUse) # ignore errors on drop. if create is ok, we are fine


      #######################################
      #  create functions
      log(mget("CreatingSqlBasedFunctions"))
      sqlFile2=os.path.join( db.dbmsType, "createFunctionsSql_Observer.sql"  )
      sqlFile2=os.path.join( sqlPath, sqlFile2 )
      if db.runDdlFile(sqlFile2,0,logToUse) < 0:
        failed=failed+1

      #######################################
      #  remove jar
      a=self.console.selectYesNo( mget("PromptRemoveAlsoJar"), self.console.YES )
      if a==self.console.YES:
        if self.removeJar(db,1) < 0:   #do not warn
          failed=failed+1

      #####################
      # explicit commit to enusre changes
      db.commit()


    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp)
      failed=failed+1

    except Exception, e:
      bpelib.handleException()
      failed=failed+1

    if failed:
      log( mget("ErrorDatabaseSetupFailed") )
      log( mget("MoreInfoInLogFile", logToUse.fileName ))
    else:
      log( mget("DatabaseSetupSucess") )

    if logToUse!=None:
      logToUse.close()

    return failed


  ##############################################################################
  #
  # installJar
  #
  ##############################################################################
  def installJar(self, db, silent=0):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    rc=0
    oldDbSchema=db.getSchema()

    try:

      if silent==0:

        print mget("NewLine")
        inp = self.console.selectString(mget("PromptSchema"), db.getSchema() )
        db.setSchema(inp.upper())
        print mget("NewLine")


      #######################################
      #  install the bpcodbutil.jar

      # check if we have the file
      dbutil=os.path.join( javaEnv["was.install.root"], "lib" )
      dbutil=os.path.join( dbutil, self.jarFileName )

      if db.dbmsType==db.DBMS_TYPE_ORACLE:
        jarFileName=dbutil
      else:
        jarFileName=self.jarID

      if not os.path.exists(dbutil):
        log(mget("ErrorJarMissing", os.path.normpath(dbutil)))
        log(mcat.get("JarFileInstallFailed",jarFileName))
        rc=-1

      else:
        db.installJarFile(dbutil,db.getSchema() + '.' + self.jarID)  # BPCODBUTIL not used for oracle
        log(mcat.get("JarFileInstallSucess",jarFileName))


    except IOError, msg:
      print msg
      rc=1

    except java.io.IOException:
      bpelib.handleException(0, 0, db.databasePassword)
      if db.dbmsType==DBMS_TYPE_ORACLE:
        log(mcat.get("CheckOracleEnvironment"))
      rc=-1

    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp,1)
      rc=-1

    except :
      bpelib.handleException()
      rc-1

    #restore connect schema
    db.setSchema(oldDbSchema)
    if rc==-1:
      log(mget("ErrorInstallingJar", dbutil))

    return rc


  ##############################################################################
  #
  # removeJar
  #
  ##############################################################################
  def removeJar(self, db, silent=0):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")
    removeJarfailed=0

    oldDbSchema=db.getSchema()

    try:

      if silent==0:

        print mget("NewLine")
        inp = self.console.selectString(mget("PromptSchemaRemoveUdfFrom"), db.getSchema() )
        db.setSchema(inp.upper())
        print mget("NewLine")


        #############################
        # ensure user wants to remove
        print mget("NewLine")
        print(mget("WarningRemoveJar"))
        a=self.console.selectYesNo( mget("ContinueAnyway"), self.console.NO )
        if a!=self.console.YES:
          return
      jar=db.getSchema() + "." + self.jarID
      if (db.dbmsType == db.DBMS_TYPE_ORACLE):
        jar=self.jarFileName

      dbutil=os.path.join( javaEnv["was.install.root"], "lib" )
      dbutil=os.path.join( dbutil, self.jarFileName )

      db.removeJarFile(self.jarID, dbutil)  # BPCODBUTIL not used for oracle
      log(mcat.get("JarFileRemoveSucess",jar, db.databaseName))

    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp,1)
      removeJarfailed=1

    except IOError, msg:
      print msg
      removeJarfailed=1

    except :
      bpelib.handleException()
      removeJarfailed=1

    #restore connect schema
    db.setSchema(oldDbSchema)
    if removeJarfailed==1:
      log( mget("ErrorRemovingJar", jar ))

    return removeJarfailed

  ##############################################################################
  #
  # listJars
  #
  ##############################################################################
  def listJars(self,db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    try:

      db.listJars(self.jarID)
      print mget("NewLine")

    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp,1)

    except Exception, e:
      bpelib.handleException()

  ##############################################################################
  #
  # administerUdfJar
  #
  # Administer the jar containing the related user-defined functions
  #
  # * Install the jar file into a database
  # * Remove the jar file from a database
  # * List installed jar files
  #
  ##############################################################################
  def administerUdfJar(self, db):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    ###############
    ## show menu

    menuList =[]
    validAnswers =[]
    itemCount=0
    bDone=0

    # Item 1) Install the jar file into a database
    installJar=itemCount
    menuList.append( mget("Menu_InstallJar"))
    validAnswers.append(str(installJar+1))
    itemCount+=1


    # Item 2) Remove the jar file from a database
    removeJar=itemCount
    menuList.append( mget("Menu_RemoveJar"))
    validAnswers.append(str(removeJar+1))
    itemCount+=1

    # Item 3) List installed jar files
    listJars=itemCount
    if db.dbmsType!=db.DBMS_TYPE_ORACLE:
      menuList.append( mget("Menu_ListJars") + "\n")
    else:
      menuList.append( mget("Menu_ListTimestampUtil") + "\n")
    validAnswers.append(str(listJars+1))
    itemCount+=1


    # Item Exit
    ExitMenu=itemCount
    menuList.append( mget("Menu_Exit") + "\n" )
    validAnswers.append("0")   # press 0 to exit

    while bDone == 0:

      print mget("NewLine")
      print mget("Menu_AdministrateUdf")

      # display the menu
      inp= self.console.selectFromList( "", validAnswers, None, menuList )

      if inp == installJar:
        self.installJar(db)

      elif inp == listJars:
        self.listJars(db)

      elif inp == removeJar:
        self.removeJar(db)

      if inp == ExitMenu:
        bDone=1

    return 0

  ##############################################################################
  #
  # promptJavaOrSqlUdf
  #
  ##############################################################################
  def promptJavaOrSqlUdf(self):

    ###############
    ## show menu

    menuList =[]
    validAnswers =[]
    itemCount=0
    rc=bpelib.DatabaseManager.UDF_IMPL_JAVA

    # Item 1) Install the jar file into a database
    MenuJava=itemCount
    menuList.append( mget("Java"))
    validAnswers.append(str(MenuJava+1))
    itemCount+=1


    # Item 2) Remove the jar file from a database
    MenuSql=itemCount
    menuList.append( mget("SQL") + "\n")
    validAnswers.append(str(MenuSql+1))
    itemCount+=1

    # Item Exit
    ExitMenu=itemCount
    menuList.append( mget("Menu_Exit") + "\n" )
    validAnswers.append("0")   # press 0 to exit

    print mget("NewLine")
    print mget("PromptJavaOrSqlUdfHeading")
    print mget("NewLine")
    print " ", mget("NoteOnUdfImplementation")
    print " ", mget("NoteVisitDocumetation")

    # display the menu
    inp= self.console.selectFromList( "", validAnswers, MenuJava, menuList )
    if inp == ExitMenu:
      rc=self.console.CANCEL
    elif inp == MenuSql:
      rc=bpelib.DatabaseManager.UDF_IMPL_SQL
    else :
      rc=bpelib.DatabaseManager.UDF_IMPL_JAVA  # default

    return rc

  ##############################################################################
  #
  # determineUdfState
  #
  # check if plain SQL or java UDF is active
  # complex: if java UDF is active, check if jar file is installed and up-to-date
  #
  ##############################################################################
  def determineUdfState(self,db, silent=0):

    if db==None: raise AssertionError, mget("ErrorDbObjectNoInit")

    rc=mget("undefined")
    rc1=mget("notWorking")

    try:

      print mget("NewLine")
      rc=db.isJavaOrSqlUdfActive()
      rc1=db.isUdfWorking()

    except java.sql.SQLException, xcp:
      db.handleSqlException(xcp, (silent==0) )

    except Exception, e:
      bpelib.handleException()

    if silent==0 :
      log(mget("ActiveUdfImplementation", rc))
      log(mget("TestUdf", rc1))

    return rc


  ##############################################################################
  #
  # Administer Observer related user-defined functions
  #
  ##############################################################################
  def administerUdf(self):

    bDone=0
    rc=0
    isNewInstance=(self.db==None)
    reuseConnection=0

    ########################
    # Prompt to reuse DB object
    if isNewInstance==0:
      a=self.console.selectYesNoCancel( mget("PromptReuseDbConnection"), self.console.YES )
      if a==self.console.YES:
        reuseConnection=1
      elif a==self.console.CANCEL:
        return

    ###################################
    # collect DB connection parameters
    try:

      while 1:

        ########################
        # dbTpye
        if reuseConnection==0:
          db=self.promptDatabaseType()
          if db==None:
            return "Exit"
        else:
          db=self.db
          # to avoid error "Database 'B:\w\p\_Srv01\databases\BPEDB' not found. Error Code: 40000 SQLState: XJ004"
          if db.dbmsType == db.DBMS_TYPE_DERBY:
            db.resetSingletons()



        ########################
        # db connection
        if reuseConnection==0:
          rs=self.promptDatabaseConnectionParameters(db,isNewInstance)
          if rs=="Exit":
            return


        ########################
        # test if we can connect
        rs=self.testConnection(db)

        if rs=="Exit" or rs==self.console.CANCEL:
          return
        elif rs=="promptNewParams":
          #recurse to our self
          if db.connection!=None:
            db.disconnect()
          self.db=None
          return self.administerUdf()
        else:
          # db is valid, copy it ro self.db
          self.db=db

        break


      ###############
      ## show menu

      menuList =[]
      validAnswers =[]
      itemCount=0

      if (db.dbmsType!=db.DBMS_TYPE_DERBY):
        # Item 1) Activate Java based user-defined functions
        activateJava=itemCount
        menuList.append( mget("Menu_ActivateJavaUdf"))
        validAnswers.append(str(activateJava+1))
        itemCount+=1
      else:
        activateJava=-1


      if (db.dbmsType!=db.DBMS_TYPE_DERBY):
        # Item 2) Use pure SQL based user-defined functions (disable the java based functions)
        activateSql=itemCount
        menuList.append( mget("Menu_ActivateSqlUdf"))
        validAnswers.append(str(activateSql+1))
        itemCount+=1
      else:
        activateSql=-1

      # Item 3) Determine which function type is currently used: java based or pur SQL
      determineState=itemCount
      menuList.append( mget("Menu_DetermineFktType"))
      validAnswers.append(str(determineState+1))
      itemCount+=1

      # Item 4) Install or remove the jar file containing the java based functions
      administerJar=itemCount
      menuList.append( mget("Menu_AdministerJar") + "\n")
      validAnswers.append(str(administerJar+1))
      itemCount+=1

      # Item Exit
      ExitMenu=itemCount
      menuList.append( mget("Menu_Exit") + "\n" )
      validAnswers.append("0")   # press 0 to exit

      while bDone == 0:

        print mget("NewLine")
        print mget("Menu_AdministrateUdf")

        # display the menu
        inp= self.console.selectFromList( "", validAnswers, None, menuList )

        if inp == activateJava:
          self.activateJavaUdf(db)

        elif inp == activateSql:
          self.promptObserverSchema(db)
          self.activateSqlUdf(db)

        elif inp == administerJar:
          self.administerUdfJar(db)

        elif inp == determineState:
          self.promptObserverSchema(db)
          self.determineUdfState(db)

        if inp == ExitMenu:
          bDone=1

    except Exception, e:
      bpelib.handleException()

    return 0


  ################################################################################
  #
  # main menu
  #
  ################################################################################
  def mainMenu(self):

    bDone=0
    rc=0

    ## show menu

    menuList =[]
    validAnswers =[]
    itemCount=0


    # Item intall application
    PrepareDb=itemCount
    menuList.append( mget("Menu_PrepareDb"))
    validAnswers.append(str(PrepareDb+1))
    itemCount+=1

    # Item intall application
    InstallApp=itemCount
    menuList.append( mget("Menu_InstallApp", mget("ConsumerShortName")) )
    validAnswers.append(str(InstallApp+1))
    itemCount+=1

    # item Remove app
    RemoveApp=itemCount
    menuList.append( mget("Menu_Remove", mget("ConsumerShortName")) )
    validAnswers.append(str(RemoveApp+1))
    itemCount+=1

    # Item Change Profile
    ChangeProfile=itemCount
    menuList.append( mget("Menu_ChangeProfile", mget("ConsumerShortName")) )
    validAnswers.append(str(ChangeProfile+1))
    itemCount+=1

    # Item remove schema from db
    DropSchema=itemCount
    menuList.append( mget("Menu_DropSchema"))
    validAnswers.append(str(DropSchema+1))
    itemCount+=1

    # Item Administer Observer related user-defined functions
    UdfAdmin=itemCount
    menuList.append( mget("Menu_AdministrateUdf") +"\n" )
    validAnswers.append(str(UdfAdmin+1))
    itemCount+=1

    # Item Exit
    ExitMenu=itemCount
    menuList.append( mget("Menu_Exit") + "\n" )
    validAnswers.append("0")   # press 0 to exit

    while bDone == 0:

      try:

        itemCount=1

        print mget("NewLine")
        print mget("MenuHeading")

        # display the menu
        inp= self.console.selectFromList( "", validAnswers, None, menuList )

        if inp == PrepareDb:
          self.prepareDatabase()

        elif inp == DropSchema:
          self.dropSchemaFromDatabase()

        elif inp == UdfAdmin:
          self.administerUdf()

        elif inp == InstallApp:
          self.prepareAndInstall()

        elif inp == RemoveApp:
          self.removeApp()

        elif inp == ChangeProfile:
          self.changeProfile()

        if inp == ExitMenu:
          bDone=1

      except SystemExit, e:
        if e.code == "ReturnToMain":
          continue
        else:
          raise

    return 0


  ################################################################################
  #
  # THE MAIN PROGRAM
  #
  ################################################################################
  def main(self):


    doRemove   = 0
    i          = 0
    gotANode   = 0
    gotAServer = 0
    gotACluster= 0
    argerr   = None
    argc     = len(sys.argv)
    rc=0

    ######################################
    # check for command line parameters
    while ( i < argc ):
      p=sys.argv[i]       # .lower()
      if p == "-r" or p == "-remove" :
        doRemove=1


      elif p =="-silent" :
        self.doSilent=1


      elif p =="-ignoreAntTask" :
        self.ignoreAntTask=1
        print "Using internal SQL processor instead of Ant"


      elif p =="-trace" :
        #just ignore trace
        if (i+1 < argc):
          if not '-' in sys.argv[i+1]:
            i=i+1  #ignore the optional file name passed with -trace


      elif p =="-earpath" :
        i=i+1
        if (i < argc):
          self.earFilePath = sys.argv[i]
        else:
          argerr = mget("ErrorCmdLineArgMissing", '-earpath')

      elif p =="-cluster" :
        self.theAppServer.isClusterServer=1
        gotACluster=1
        i=i+1
        if (i < argc):
          self.theAppServer.serverName = sys.argv[i]
        else:
          argerr = mget("ErrorCmdLineArgMissing", '-cluster')


      elif p =="-server" :
        i=i+1
        gotAServer=1
        if (i < argc):
          self.theAppServer.serverName = sys.argv[i]
        else:
          argerr = mget("ErrorCmdLineArgMissing", '-server')


      elif p =="-node" :
        i=i+1
        if (i < argc):
          self.theAppServer.nodeName = sys.argv[i]
          gotANode=1
        else:
          argerr = mget("ErrorCmdLineArgMissing", '-node')

      elif p.find(__file__)>-1 :
        # ignore the filename given when debugging
        pass

      else:
        argerr = mget("ErrorInvalidCommandLine", p)


      i=i+1 #endwhile


    ######################################
    # check for errors on the command line
    if (argerr == None) and gotACluster and gotAServer:
      argerr=mget("ErrorOnlyServerOrCluster")

    if (argerr == None) and gotAServer==0 and gotACluster==0 and self.theAppServer.isManagedServer==1 and self.doSilent==1:
        argerr=mget("ServerOrClusterRequired") + "\n" + mget("SpecifyServerOrCluster")
        self.restricted==0

    if (argerr == None) and gotAServer==1 and gotANode==0 and self.theAppServer.isManagedServer==1 and self.doSilent==1:
        argerr=mget("ErrorNodeRequiredInND" )
        self.restricted==0

    if argerr != None:
      print( mget("NewLine") )
      log( argerr )
      print( mget("NewLine") )
      if self.restricted==0:
        return -1


    ######################################
    # reinit the server dependend objects
    if gotAServer:
      self.theAppServer.setToServer()
      self.appName=self.theAppServer.makeBPCAppName(self.initialAppName)
    elif gotACluster:
      self.theAppServer.setToCluster()
      self.appName=self.theAppServer.makeBPCAppName(self.initialAppName)


    ######################################
    # do it

    if doRemove != 0:
      rc = self.removeApp()

    else:
      rc = self.mainMenu()


    return rc



################################################################################
################################################################################
################################################################################
#
# Execute the script
#
################################################################################

print mget("LineSep")
print mget("WelcomeComsumer")
print mget("LineSep")
print mget("LogfileLocation", bpelib.LogFile.defaultLog.fileName)
print mget("Initializing")
print mget("NewLine")

try:

  if globals().has_key('developersMode'):
    bpelib.Tracer.theTracer=bpelib.Tracer("z:/collector.trace")
    bpelib.Tracer.theTracer.start()

  theApp = None
  theApp = EventCollector( "BPCECollector", "bpceventcollector.ear" ) # = defaultAppName
  rc=theApp.main()
  if theApp.doSilent==1:
    if rc==0:
      log( mget("SetupEndedOk") )
    else:
      log( mget("SetupEndedWithErrors",rc) )


except SystemExit:
  pass

except:
  bpelib.handleException()
  if theApp!=None and theApp.doSilent==1:
    log( mget("SetupEndedWithErrors","-99") )
    rc=-99

try:

  #shutdown test connection
  if theApp!=None and theApp.db!=None and theApp.db.connection!=None :
    theApp.db.disconnect()

  # close the log
  if bpelib.LogFile.defaultLog!=None:
    bpelib.LogFile.defaultLog.close()


finally:
  if bpelib.Tracer.isTracing==1:
    bpelib.Tracer.theTracer.stop()
  #os._exit(rc)





################################################################################
